@@ -1,4 +1,5 @@
 """Support for Agent camera streaming."""
+
 from datetime import timedelta
 import logging
 
@@ -6,17 +7,16 @@ from agent import AgentError
 
 from homeassistant.components.camera import CameraEntityFeature
 from homeassistant.components.mjpeg import MjpegCamera, filter_urllib3_logging
-from homeassistant.const import ATTR_ATTRIBUTION
-from homeassistant.helpers import entity_platform
-from homeassistant.helpers.entity import DeviceInfo
-
-from .const import (
-    ATTRIBUTION,
-    CAMERA_SCAN_INTERVAL_SECS,
-    CONNECTION,
-    DOMAIN as AGENT_DOMAIN,
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.entity_platform import (
+    AddConfigEntryEntitiesCallback,
+    async_get_current_platform,
 )
 
+from . import AgentDVRConfigEntry
+from .const import ATTRIBUTION, CAMERA_SCAN_INTERVAL_SECS, DOMAIN
+
 SCAN_INTERVAL = timedelta(seconds=CAMERA_SCAN_INTERVAL_SECS)
 
 _LOGGER = logging.getLogger(__name__)
@@ -37,13 +37,15 @@ CAMERA_SERVICES = {
 
 
 async def async_setup_entry(
-    hass, config_entry, async_add_entities, discovery_info=None
-):
+    hass: HomeAssistant,
+    config_entry: AgentDVRConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
+) -> None:
     """Set up the Agent cameras."""
     filter_urllib3_logging()
     cameras = []
 
-    server = hass.data[AGENT_DOMAIN][config_entry.entry_id][CONNECTION]
+    server = config_entry.runtime_data
     if not server.devices:
         _LOGGER.warning("Could not fetch cameras from Agent server")
         return
@@ -55,36 +57,39 @@ async def async_setup_entry(
 
     async_add_entities(cameras)
 
-    platform = entity_platform.async_get_current_platform()
+    platform = async_get_current_platform()
     for service, method in CAMERA_SERVICES.items():
-        platform.async_register_entity_service(service, {}, method)
+        platform.async_register_entity_service(service, None, method)
 
 
 class AgentCamera(MjpegCamera):
     """Representation of an Agent Device Stream."""
 
+    _attr_attribution = ATTRIBUTION
+    _attr_should_poll = True  # Cameras default to False
     _attr_supported_features = CameraEntityFeature.ON_OFF
+    _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(self, device):
         """Initialize as a subclass of MjpegCamera."""
         self.device = device
         self._removed = False
-        self._attr_name = f"{device.client.name} {device.name}"
-        self._attr_unique_id = f"{device._client.unique}_{device.typeID}_{device.id}"
+        self._attr_unique_id = f"{device.client.unique}_{device.typeID}_{device.id}"
         super().__init__(
             name=device.name,
-            mjpeg_url=f"{device.client._server_url}{device.mjpeg_image_url}&size={device.mjpegStreamWidth}x{device.mjpegStreamHeight}",
-            still_image_url=f"{device.client._server_url}{device.still_image_url}&size={device.mjpegStreamWidth}x{device.mjpegStreamHeight}",
+            mjpeg_url=f"{device.client._server_url}{device.mjpeg_image_url}&size={device.mjpegStreamWidth}x{device.mjpegStreamHeight}",  # noqa: SLF001
+            still_image_url=f"{device.client._server_url}{device.still_image_url}&size={device.mjpegStreamWidth}x{device.mjpegStreamHeight}",  # noqa: SLF001
         )
         self._attr_device_info = DeviceInfo(
-            identifiers={(AGENT_DOMAIN, self.unique_id)},
+            identifiers={(DOMAIN, self.unique_id)},
             manufacturer="Agent",
             model="Camera",
-            name=self.name,
+            name=f"{device.client.name} {device.name}",
             sw_version=device.client.version,
         )
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update our state from the Agent API."""
         try:
             await self.device.update()
@@ -101,7 +106,6 @@ class AgentCamera(MjpegCamera):
             self._attr_icon = "mdi:camcorder"
         self._attr_available = self.device.client.is_available
         self._attr_extra_state_attributes = {
-            ATTR_ATTRIBUTION: ATTRIBUTION,
             "editable": False,
             "enabled": self.is_on,
             "connected": self.connected,
@@ -111,11 +115,6 @@ class AgentCamera(MjpegCamera):
             "alerts_enabled": self.device.alerts_active,
         }
 
-    @property
-    def should_poll(self) -> bool:
-        """Update the state periodically."""
-        return True
-
     @property
     def is_recording(self) -> bool:
         """Return whether the monitor is recording."""
@@ -142,7 +141,7 @@ class AgentCamera(MjpegCamera):
         return self.device.online
 
     @property
-    def motion_detection_enabled(self):
+    def motion_detection_enabled(self) -> bool:
         """Return the camera motion detection status."""
         return self.device.detector_active
 
@@ -154,11 +153,11 @@ class AgentCamera(MjpegCamera):
         """Disable alerts."""
         await self.device.alerts_off()
 
-    async def async_enable_motion_detection(self):
+    async def async_enable_motion_detection(self) -> None:
         """Enable motion detection."""
         await self.device.detector_on()
 
-    async def async_disable_motion_detection(self):
+    async def async_disable_motion_detection(self) -> None:
         """Disable motion detection."""
         await self.device.detector_off()
 
@@ -170,7 +169,7 @@ class AgentCamera(MjpegCamera):
         """Stop recording."""
         await self.device.record_stop()
 
-    async def async_turn_on(self):
+    async def async_turn_on(self) -> None:
         """Enable the camera."""
         await self.device.enable()
 
@@ -178,6 +177,6 @@ class AgentCamera(MjpegCamera):
         """Take a snapshot."""
         await self.device.snapshot()
 
-    async def async_turn_off(self):
+    async def async_turn_off(self) -> None:
         """Disable the camera."""
         await self.device.disable()
