@@ -1,32 +1,32 @@
 """Adds config flow for Airly."""
+
 from __future__ import annotations
 
+from asyncio import timeout
 from http import HTTPStatus
 from typing import Any
 
 from aiohttp import ClientSession
 from airly import Airly
 from airly.exceptions import AirlyError
-import async_timeout
 import voluptuous as vol
 
-from homeassistant import config_entries
+from homeassistant.config_entries import ConfigFlow, ConfigFlowResult
 from homeassistant.const import CONF_API_KEY, CONF_LATITUDE, CONF_LONGITUDE, CONF_NAME
-from homeassistant.data_entry_flow import FlowResult
+from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
-import homeassistant.helpers.config_validation as cv
 
 from .const import CONF_USE_NEAREST, DOMAIN, NO_AIRLY_SENSORS
 
 
-class AirlyFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
+class AirlyFlowHandler(ConfigFlow, domain=DOMAIN):
     """Config flow for Airly."""
 
     VERSION = 1
 
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle a flow initialized by the user."""
         errors = {}
         use_nearest = False
@@ -39,14 +39,14 @@ class AirlyFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             )
             self._abort_if_unique_id_configured()
             try:
-                location_point_valid = await test_location(
+                location_point_valid = await check_location(
                     websession,
                     user_input["api_key"],
                     user_input["latitude"],
                     user_input["longitude"],
                 )
                 if not location_point_valid:
-                    await test_location(
+                    location_nearest_valid = await check_location(
                         websession,
                         user_input["api_key"],
                         user_input["latitude"],
@@ -60,6 +60,8 @@ class AirlyFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
                     errors["base"] = "wrong_location"
             else:
                 if not location_point_valid:
+                    if not location_nearest_valid:
+                        return self.async_abort(reason="wrong_location")
                     use_nearest = True
                 return self.async_create_entry(
                     title=user_input[CONF_NAME],
@@ -86,7 +88,7 @@ class AirlyFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
 
-async def test_location(
+async def check_location(
     client: ClientSession,
     api_key: str,
     latitude: float,
@@ -103,7 +105,7 @@ async def test_location(
         measurements = airly.create_measurements_session_point(
             latitude=latitude, longitude=longitude
         )
-    with async_timeout.timeout(10):
+    async with timeout(10):
         await measurements.update()
 
     current = measurements.current
