@@ -1,4 +1,5 @@
 """Support for the Airzone sensors."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
@@ -22,10 +23,9 @@ from homeassistant.components.binary_sensor import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from .const import DOMAIN
-from .coordinator import AirzoneUpdateCoordinator
+from .coordinator import AirzoneConfigEntry, AirzoneUpdateCoordinator
 from .entity import AirzoneEntity, AirzoneSystemEntity, AirzoneZoneEntity
 
 
@@ -74,40 +74,61 @@ ZONE_BINARY_SENSOR_TYPES: Final[tuple[AirzoneBinarySensorEntityDescription, ...]
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: AirzoneConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Add Airzone binary sensors from a config_entry."""
-    coordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator = entry.runtime_data
 
-    binary_sensors: list[AirzoneBinarySensor] = []
+    added_systems: set[str] = set()
+    added_zones: set[str] = set()
 
-    for system_id, system_data in coordinator.data[AZD_SYSTEMS].items():
-        for description in SYSTEM_BINARY_SENSOR_TYPES:
-            if description.key in system_data:
-                binary_sensors.append(
-                    AirzoneSystemBinarySensor(
-                        coordinator,
-                        description,
-                        entry,
-                        system_id,
-                        system_data,
-                    )
+    def _async_entity_listener() -> None:
+        """Handle additions of binary sensors."""
+
+        entities: list[AirzoneBinarySensor] = []
+
+        systems_data = coordinator.data.get(AZD_SYSTEMS, {})
+        received_systems = set(systems_data)
+        new_systems = received_systems - added_systems
+        if new_systems:
+            entities.extend(
+                AirzoneSystemBinarySensor(
+                    coordinator,
+                    description,
+                    entry,
+                    system_id,
+                    systems_data.get(system_id),
                 )
+                for system_id in new_systems
+                for description in SYSTEM_BINARY_SENSOR_TYPES
+                if description.key in systems_data.get(system_id)
+            )
+            added_systems.update(new_systems)
 
-    for system_zone_id, zone_data in coordinator.data[AZD_ZONES].items():
-        for description in ZONE_BINARY_SENSOR_TYPES:
-            if description.key in zone_data:
-                binary_sensors.append(
-                    AirzoneZoneBinarySensor(
-                        coordinator,
-                        description,
-                        entry,
-                        system_zone_id,
-                        zone_data,
-                    )
+        zones_data = coordinator.data.get(AZD_ZONES, {})
+        received_zones = set(zones_data)
+        new_zones = received_zones - added_zones
+        if new_zones:
+            entities.extend(
+                AirzoneZoneBinarySensor(
+                    coordinator,
+                    description,
+                    entry,
+                    system_zone_id,
+                    zones_data.get(system_zone_id),
                 )
+                for system_zone_id in new_zones
+                for description in ZONE_BINARY_SENSOR_TYPES
+                if description.key in zones_data.get(system_zone_id)
+            )
+            added_zones.update(new_zones)
 
-    async_add_entities(binary_sensors)
+        async_add_entities(entities)
+
+    entry.async_on_unload(coordinator.async_add_listener(_async_entity_listener))
+    _async_entity_listener()
 
 
 class AirzoneBinarySensor(AirzoneEntity, BinarySensorEntity):
