@@ -1,8 +1,10 @@
 """Support for Amcrest IP cameras."""
+
 from __future__ import annotations
 
-from collections.abc import Callable
-from contextlib import suppress
+import asyncio
+from collections.abc import AsyncIterator, Callable
+from contextlib import asynccontextmanager, suppress
 from dataclasses import dataclass
 from datetime import datetime, timedelta
 import logging
@@ -11,16 +13,10 @@ from typing import Any
 
 import aiohttp
 from amcrest import AmcrestError, ApiWrapper, LoginError
+import httpx
 import voluptuous as vol
 
-from homeassistant.auth.models import User
-from homeassistant.auth.permissions.const import POLICY_CONTROL
-from homeassistant.components.binary_sensor import DOMAIN as BINARY_SENSOR
-from homeassistant.components.camera import DOMAIN as CAMERA
-from homeassistant.components.sensor import DOMAIN as SENSOR
-from homeassistant.components.switch import DOMAIN as SWITCH
 from homeassistant.const import (
-    ATTR_ENTITY_ID,
     CONF_AUTHENTICATION,
     CONF_BINARY_SENSORS,
     CONF_HOST,
@@ -31,21 +27,17 @@ from homeassistant.const import (
     CONF_SENSORS,
     CONF_SWITCHES,
     CONF_USERNAME,
-    ENTITY_MATCH_ALL,
-    ENTITY_MATCH_NONE,
     HTTP_BASIC_AUTHENTICATION,
+    Platform,
 )
-from homeassistant.core import HomeAssistant, ServiceCall
-from homeassistant.exceptions import Unauthorized, UnknownUser
-from homeassistant.helpers import discovery
-import homeassistant.helpers.config_validation as cv
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers import config_validation as cv, discovery
 from homeassistant.helpers.dispatcher import async_dispatcher_send, dispatcher_send
-from homeassistant.helpers.event import track_time_interval
-from homeassistant.helpers.service import async_extract_entity_ids
+from homeassistant.helpers.event import async_track_time_interval
 from homeassistant.helpers.typing import ConfigType
 
 from .binary_sensor import BINARY_SENSOR_KEYS, BINARY_SENSORS, check_binary_sensors
-from .camera import CAMERA_SERVICES, STREAM_SOURCE_LIST
+from .camera import STREAM_SOURCE_LIST
 from .const import (
     CAMERAS,
     COMM_RETRIES,
@@ -53,11 +45,13 @@ from .const import (
     DATA_AMCREST,
     DEVICES,
     DOMAIN,
+    RESOLUTION_LIST,
     SERVICE_EVENT,
     SERVICE_UPDATE,
 )
 from .helpers import service_signal
 from .sensor import SENSOR_KEYS
+from .services import async_setup_services
 from .switch import SWITCH_KEYS
 
 _LOGGER = logging.getLogger(__name__)
@@ -77,8 +71,6 @@ RECHECK_INTERVAL = timedelta(minutes=1)
 NOTIFICATION_ID = "amcrest_notification"
 NOTIFICATION_TITLE = "Amcrest Camera Setup"
 
-RESOLUTION_LIST = {"high": 0, "low": 1}
-
 SCAN_INTERVAL = timedelta(seconds=10)
 
 AUTHENTICATION_LIST = {"basic": "basic"}
@@ -147,9 +139,12 @@ class AmcrestChecker(ApiWrapper):
         self._wrap_name = name
         self._wrap_errors = 0
         self._wrap_lock = threading.Lock()
+        self._async_wrap_lock = asyncio.Lock()
         self._wrap_login_err = False
         self._wrap_event_flag = threading.Event()
         self._wrap_event_flag.set()
+        self._async_wrap_event_flag = asyncio.Event()
+        self._async_wrap_event_flag.set()
         self._unsub_recheck: Callable[[], None] | None = None
         super().__init__(
             host,
@@ -167,13 +162,22 @@ class AmcrestChecker(ApiWrapper):
 
     @property
     def available_flag(self) -> threading.Event:
-        """Return threading event flag that indicates if camera's API is responding."""
+        """Return event flag that indicates if camera's API is responding."""
         return self._wrap_event_flag
 
-    def _start_recovery(self) -> None:
-        self._wrap_event_flag.clear()
-        dispatcher_send(self._hass, service_signal(SERVICE_UPDATE, self._wrap_name))
-        self._unsub_recheck = track_time_interval(
+    @property
+    def async_available_flag(self) -> asyncio.Event:
+        """Return event flag that indicates if camera's API is responding."""
+        return self._async_wrap_event_flag
+
+    @callback
+    def _async_start_recovery(self) -> None:
+        self.available_flag.clear()
+        self.async_available_flag.clear()
+        async_dispatcher_send(
+            self._hass, service_signal(SERVICE_UPDATE, self._wrap_name)
+        )
+        self._unsub_recheck = async_track_time_interval(
             self._hass, self._wrap_test_online, RECHECK_INTERVAL
         )
 
@@ -182,43 +186,134 @@ class AmcrestChecker(ApiWrapper):
         try:
             ret = super().command(*args, **kwargs)
         except LoginError as ex:
-            with self._wrap_lock:
-                was_online = self.available
-                was_login_err = self._wrap_login_err
-                self._wrap_login_err = True
-            if not was_login_err:
-                _LOGGER.error("%s camera offline: Login error: %s", self._wrap_name, ex)
-            if was_online:
-                self._start_recovery()
+            self._handle_offline(ex)
             raise
         except AmcrestError:
-            with self._wrap_lock:
-                was_online = self.available
-                errs = self._wrap_errors = self._wrap_errors + 1
-                offline = not self.available
-            _LOGGER.debug("%s camera errs: %i", self._wrap_name, errs)
-            if was_online and offline:
-                _LOGGER.error("%s camera offline: Too many errors", self._wrap_name)
-                self._start_recovery()
+            self._handle_error()
             raise
+        self._set_online()
+        return ret
+
+    async def async_command(self, *args: Any, **kwargs: Any) -> httpx.Response:
+        """amcrest.ApiWrapper.command wrapper to catch errors."""
+        async with self._async_command_wrapper():
+            return await super().async_command(*args, **kwargs)
+
+    @asynccontextmanager
+    async def async_stream_command(
+        self, *args: Any, **kwargs: Any
+    ) -> AsyncIterator[httpx.Response]:
+        """amcrest.ApiWrapper.command wrapper to catch errors."""
+        async with (
+            self._async_command_wrapper(),
+            super().async_stream_command(*args, **kwargs) as ret,
+        ):
+            yield ret
+
+    @asynccontextmanager
+    async def _async_command_wrapper(self) -> AsyncIterator[None]:
+        try:
+            yield
+        except LoginError as ex:
+            async with self._async_wrap_lock:
+                self._async_handle_offline(ex)
+            raise
+        except AmcrestError:
+            async with self._async_wrap_lock:
+                self._async_handle_error()
+            raise
+        async with self._async_wrap_lock:
+            self._async_set_online()
+
+    def _handle_offline_thread_safe(self, ex: Exception) -> bool:
+        """Handle camera offline status shared between threads and event loop.
+
+        Returns if the camera was online as a bool.
+        """
+        with self._wrap_lock:
+            was_online = self.available
+            was_login_err = self._wrap_login_err
+            self._wrap_login_err = True
+        if not was_login_err:
+            _LOGGER.error("%s camera offline: Login error: %s", self._wrap_name, ex)
+        return was_online
+
+    def _handle_offline(self, ex: Exception) -> None:
+        """Handle camera offline status from a thread."""
+        if self._handle_offline_thread_safe(ex):
+            self._hass.loop.call_soon_threadsafe(self._async_start_recovery)
+
+    @callback
+    def _async_handle_offline(self, ex: Exception) -> None:
+        if self._handle_offline_thread_safe(ex):
+            self._async_start_recovery()
+
+    def _handle_error_thread_safe(self) -> bool:
+        """Handle camera error status shared between threads and event loop.
+
+        Returns if the camera was online and is now offline as
+        a bool.
+        """
+        with self._wrap_lock:
+            was_online = self.available
+            errs = self._wrap_errors = self._wrap_errors + 1
+            offline = not self.available
+        _LOGGER.debug("%s camera errs: %i", self._wrap_name, errs)
+        return was_online and offline
+
+    def _handle_error(self) -> None:
+        """Handle camera error status from a thread."""
+        if self._handle_error_thread_safe():
+            _LOGGER.error("%s camera offline: Too many errors", self._wrap_name)
+            self._hass.loop.call_soon_threadsafe(self._async_start_recovery)
+
+    @callback
+    def _async_handle_error(self) -> None:
+        """Handle camera error status from the event loop."""
+        if self._handle_error_thread_safe():
+            _LOGGER.error("%s camera offline: Too many errors", self._wrap_name)
+            self._async_start_recovery()
+
+    def _set_online_thread_safe(self) -> bool:
+        """Set camera online status shared between threads and event loop.
+
+        Returns if the camera was offline as a bool.
+        """
         with self._wrap_lock:
             was_offline = not self.available
             self._wrap_errors = 0
             self._wrap_login_err = False
-        if was_offline:
-            assert self._unsub_recheck is not None
-            self._unsub_recheck()
-            self._unsub_recheck = None
-            _LOGGER.error("%s camera back online", self._wrap_name)
-            self._wrap_event_flag.set()
-            dispatcher_send(self._hass, service_signal(SERVICE_UPDATE, self._wrap_name))
-        return ret
+        return was_offline
 
-    def _wrap_test_online(self, now: datetime) -> None:
+    def _set_online(self) -> None:
+        """Set camera online status from a thread."""
+        if self._set_online_thread_safe():
+            self._hass.loop.call_soon_threadsafe(self._async_signal_online)
+
+    @callback
+    def _async_set_online(self) -> None:
+        """Set camera online status from the event loop."""
+        if self._set_online_thread_safe():
+            self._async_signal_online()
+
+    @callback
+    def _async_signal_online(self) -> None:
+        """Signal that camera is back online."""
+        assert self._unsub_recheck is not None
+        self._unsub_recheck()
+        self._unsub_recheck = None
+        _LOGGER.error("%s camera back online", self._wrap_name)
+        self.available_flag.set()
+        self.async_available_flag.set()
+        async_dispatcher_send(
+            self._hass, service_signal(SERVICE_UPDATE, self._wrap_name)
+        )
+
+    async def _wrap_test_online(self, now: datetime) -> None:
         """Test if camera is back online."""
         _LOGGER.debug("Testing if %s back online", self._wrap_name)
         with suppress(AmcrestError):
-            self.current_time  # pylint: disable=pointless-statement
+            await self.async_current_time
 
 
 def _monitor_events(
@@ -230,7 +325,7 @@ def _monitor_events(
     while True:
         api.available_flag.wait()
         try:
-            for code, payload in api.event_actions("All", retries=5):
+            for code, payload in api.event_actions("All"):
                 event_data = {"camera": name, "event": code, "payload": payload}
                 hass.bus.fire("amcrest", event_data)
                 if code in event_codes:
@@ -262,7 +357,7 @@ def _start_event_monitor(
     thread.start()
 
 
-def setup(hass: HomeAssistant, config: ConfigType) -> bool:
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Amcrest IP Camera component."""
     hass.data.setdefault(DATA_AMCREST, {DEVICES: {}, CAMERAS: []})
 
@@ -301,83 +396,60 @@ def setup(hass: HomeAssistant, config: ConfigType) -> bool:
             control_light,
         )
 
-        discovery.load_platform(hass, CAMERA, DOMAIN, {CONF_NAME: name}, config)
+        hass.async_create_task(
+            discovery.async_load_platform(
+                hass, Platform.CAMERA, DOMAIN, {CONF_NAME: name}, config
+            )
+        )
 
         event_codes = set()
         if binary_sensors:
-            discovery.load_platform(
-                hass,
-                BINARY_SENSOR,
-                DOMAIN,
-                {CONF_NAME: name, CONF_BINARY_SENSORS: binary_sensors},
-                config,
+            hass.async_create_task(
+                discovery.async_load_platform(
+                    hass,
+                    Platform.BINARY_SENSOR,
+                    DOMAIN,
+                    {CONF_NAME: name, CONF_BINARY_SENSORS: binary_sensors},
+                    config,
+                )
             )
             event_codes = {
-                sensor.event_code
+                event_code
                 for sensor in BINARY_SENSORS
                 if sensor.key in binary_sensors
                 and not sensor.should_poll
-                and sensor.event_code is not None
+                and sensor.event_codes is not None
+                for event_code in sensor.event_codes
             }
 
         _start_event_monitor(hass, name, api, event_codes)
 
         if sensors:
-            discovery.load_platform(
-                hass, SENSOR, DOMAIN, {CONF_NAME: name, CONF_SENSORS: sensors}, config
+            hass.async_create_task(
+                discovery.async_load_platform(
+                    hass,
+                    Platform.SENSOR,
+                    DOMAIN,
+                    {CONF_NAME: name, CONF_SENSORS: sensors},
+                    config,
+                )
             )
 
         if switches:
-            discovery.load_platform(
-                hass, SWITCH, DOMAIN, {CONF_NAME: name, CONF_SWITCHES: switches}, config
+            hass.async_create_task(
+                discovery.async_load_platform(
+                    hass,
+                    Platform.SWITCH,
+                    DOMAIN,
+                    {CONF_NAME: name, CONF_SWITCHES: switches},
+                    config,
+                )
             )
 
     if not hass.data[DATA_AMCREST][DEVICES]:
         return False
 
-    def have_permission(user: User | None, entity_id: str) -> bool:
-        return not user or user.permissions.check_entity(entity_id, POLICY_CONTROL)
-
-    async def async_extract_from_service(call: ServiceCall) -> list[str]:
-        if call.context.user_id:
-            user = await hass.auth.async_get_user(call.context.user_id)
-            if user is None:
-                raise UnknownUser(context=call.context)
-        else:
-            user = None
-
-        if call.data.get(ATTR_ENTITY_ID) == ENTITY_MATCH_ALL:
-            # Return all entity_ids user has permission to control.
-            return [
-                entity_id
-                for entity_id in hass.data[DATA_AMCREST][CAMERAS]
-                if have_permission(user, entity_id)
-            ]
-
-        if call.data.get(ATTR_ENTITY_ID) == ENTITY_MATCH_NONE:
-            return []
-
-        call_ids = await async_extract_entity_ids(hass, call)
-        entity_ids = []
-        for entity_id in hass.data[DATA_AMCREST][CAMERAS]:
-            if entity_id not in call_ids:
-                continue
-            if not have_permission(user, entity_id):
-                raise Unauthorized(
-                    context=call.context, entity_id=entity_id, permission=POLICY_CONTROL
-                )
-            entity_ids.append(entity_id)
-        return entity_ids
-
-    async def async_service_handler(call: ServiceCall) -> None:
-        args = []
-        for arg in CAMERA_SERVICES[call.service][2]:
-            args.append(call.data[arg])
-        for entity_id in await async_extract_from_service(call):
-            async_dispatcher_send(hass, service_signal(call.service, entity_id), *args)
-
-    for service, params in CAMERA_SERVICES.items():
-        hass.services.register(DOMAIN, service, async_service_handler, params[0])
+    async_setup_services(hass)
 
     return True
 
