@@ -1,4 +1,5 @@
 """The Homeassistant Analytics integration."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
@@ -14,10 +15,11 @@ from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 
-from .const import CONF_TRACKED_INTEGRATIONS, DOMAIN
+from .const import CONF_TRACKED_INTEGRATIONS
 from .coordinator import HomeassistantAnalyticsDataUpdateCoordinator
 
 PLATFORMS: list[Platform] = [Platform.SENSOR]
+type AnalyticsInsightsConfigEntry = ConfigEntry[AnalyticsInsightsData]
 
 
 @dataclass(frozen=True)
@@ -28,7 +30,9 @@ class AnalyticsInsightsData:
     names: dict[str, str]
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+async def async_setup_entry(
+    hass: HomeAssistant, entry: AnalyticsInsightsConfigEntry
+) -> bool:
     """Set up Homeassistant Analytics from a config entry."""
     client = HomeassistantAnalyticsClient(session=async_get_clientsession(hass))
 
@@ -44,13 +48,11 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
             continue
         names[integration] = integrations[integration].title
 
-    coordinator = HomeassistantAnalyticsDataUpdateCoordinator(hass, client)
+    coordinator = HomeassistantAnalyticsDataUpdateCoordinator(hass, entry, client)
 
     await coordinator.async_config_entry_first_refresh()
 
-    hass.data.setdefault(DOMAIN, {})[entry.entry_id] = AnalyticsInsightsData(
-        coordinator=coordinator, names=names
-    )
+    entry.runtime_data = AnalyticsInsightsData(coordinator=coordinator, names=names)
 
     await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
     entry.async_on_unload(entry.add_update_listener(update_listener))
@@ -58,14 +60,15 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+async def async_unload_entry(
+    hass: HomeAssistant, entry: AnalyticsInsightsConfigEntry
+) -> bool:
     """Unload a config entry."""
-    if unload_ok := await hass.config_entries.async_unload_platforms(entry, PLATFORMS):
-        hass.data[DOMAIN].pop(entry.entry_id)
-
-    return unload_ok
+    return await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
 
-async def update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None:
+async def update_listener(
+    hass: HomeAssistant, entry: AnalyticsInsightsConfigEntry
+) -> None:
     """Handle options update."""
     await hass.config_entries.async_reload(entry.entry_id)
