@@ -1,4 +1,5 @@
 """Rest API for Home Assistant."""
+
 import asyncio
 from asyncio import shield, timeout
 from functools import lru_cache
@@ -10,9 +11,9 @@ from aiohttp import web
 from aiohttp.web_exceptions import HTTPBadRequest
 import voluptuous as vol
 
+from homeassistant import core as ha
 from homeassistant.auth.models import User
 from homeassistant.auth.permissions.const import POLICY_READ
-from homeassistant.bootstrap import DATA_LOGGING
 from homeassistant.components.http import (
     KEY_HASS,
     KEY_HASS_USER,
@@ -23,6 +24,7 @@ from homeassistant.const import (
     CONTENT_TYPE_JSON,
     EVENT_HOMEASSISTANT_STOP,
     EVENT_STATE_CHANGED,
+    KEY_DATA_LOGGING as DATA_LOGGING,
     MATCH_ALL,
     URL_API,
     URL_API_COMPONENTS,
@@ -35,8 +37,7 @@ from homeassistant.const import (
     URL_API_STREAM,
     URL_API_TEMPLATE,
 )
-import homeassistant.core as ha
-from homeassistant.core import Event, HomeAssistant
+from homeassistant.core import Event, EventStateChangedData, HomeAssistant
 from homeassistant.exceptions import (
     InvalidEntityFormatError,
     InvalidStateError,
@@ -44,11 +45,11 @@ from homeassistant.exceptions import (
     TemplateError,
     Unauthorized,
 )
-from homeassistant.helpers import config_validation as cv, template
-from homeassistant.helpers.event import EventStateChangedData
+from homeassistant.helpers import config_validation as cv, recorder, template
 from homeassistant.helpers.json import json_dumps, json_fragment
 from homeassistant.helpers.service import async_get_all_descriptions
-from homeassistant.helpers.typing import ConfigType, EventType
+from homeassistant.helpers.typing import ConfigType
+from homeassistant.util.event_type import EventType
 from homeassistant.util.json import json_loads
 
 _LOGGER = logging.getLogger(__name__)
@@ -117,8 +118,11 @@ class APICoreStateView(HomeAssistantView):
         Home Assistant core is running. Its primary use case is for supervisor
         to check if Home Assistant is running.
         """
-        hass: HomeAssistant = request.app[KEY_HASS]
-        return self.json({"state": hass.state.value})
+        hass = request.app[KEY_HASS]
+        migration = recorder.async_migration_in_progress(hass)
+        live = recorder.async_migration_is_live(hass)
+        recorder_state = {"migration_in_progress": migration, "migration_is_live": live}
+        return self.json({"state": hass.state.value, "recorder_state": recorder_state})
 
 
 class APIEventStream(HomeAssistantView):
@@ -130,13 +134,13 @@ class APIEventStream(HomeAssistantView):
     @require_admin
     async def get(self, request: web.Request) -> web.StreamResponse:
         """Provide a streaming interface for the event bus."""
-        hass: HomeAssistant = request.app[KEY_HASS]
+        hass = request.app[KEY_HASS]
         stop_obj = object()
         to_write: asyncio.Queue[object | str] = asyncio.Queue()
 
-        restrict: list[str] | None = None
+        restrict: list[EventType[Any] | str] | None = None
         if restrict_str := request.query.get("restrict"):
-            restrict = restrict_str.split(",") + [EVENT_HOMEASSISTANT_STOP]
+            restrict = [*restrict_str.split(","), EVENT_HOMEASSISTANT_STOP]
 
         async def forward_events(event: Event) -> None:
             """Forward events to the open request."""
@@ -175,7 +179,7 @@ class APIEventStream(HomeAssistantView):
                     msg = f"data: {payload}\n\n"
                     _LOGGER.debug("STREAM %s WRITING %s", id(stop_obj), msg.strip())
                     await response.write(msg.encode("UTF-8"))
-                except asyncio.TimeoutError:
+                except TimeoutError:
                     await to_write.put(STREAM_PING_PAYLOAD)
 
         except asyncio.CancelledError:
@@ -197,8 +201,7 @@ class APIConfigView(HomeAssistantView):
     @ha.callback
     def get(self, request: web.Request) -> web.Response:
         """Get current configuration."""
-        hass: HomeAssistant = request.app[KEY_HASS]
-        return self.json(hass.config.as_dict())
+        return self.json(request.app[KEY_HASS].config.as_dict())
 
 
 class APIStatesView(HomeAssistantView):
@@ -211,7 +214,7 @@ class APIStatesView(HomeAssistantView):
     def get(self, request: web.Request) -> web.Response:
         """Get current states."""
         user: User = request[KEY_HASS_USER]
-        hass: HomeAssistant = request.app[KEY_HASS]
+        hass = request.app[KEY_HASS]
         if user.is_admin:
             states = (state.as_dict_json for state in hass.states.async_all())
         else:
@@ -222,7 +225,7 @@ class APIStatesView(HomeAssistantView):
                 if entity_perm(state.entity_id, "read")
             )
         response = web.Response(
-            body=b"[" + b",".join(states) + b"]",
+            body=b"".join((b"[", b",".join(states), b"]")),
             content_type=CONTENT_TYPE_JSON,
             zlib_executor_size=32768,
         )
@@ -240,7 +243,7 @@ class APIEntityStateView(HomeAssistantView):
     def get(self, request: web.Request, entity_id: str) -> web.Response:
         """Retrieve state of entity."""
         user: User = request[KEY_HASS_USER]
-        hass: HomeAssistant = request.app[KEY_HASS]
+        hass = request.app[KEY_HASS]
         if not user.permissions.check_entity(entity_id, POLICY_READ):
             raise Unauthorized(entity_id=entity_id)
 
@@ -256,12 +259,19 @@ class APIEntityStateView(HomeAssistantView):
         user: User = request[KEY_HASS_USER]
         if not user.is_admin:
             raise Unauthorized(entity_id=entity_id)
-        hass: HomeAssistant = request.app[KEY_HASS]
+        hass = request.app[KEY_HASS]
+
+        body = await request.text()
+
         try:
-            data = await request.json()
+            data: Any = json_loads(body) if body else None
         except ValueError:
             return self.json_message("Invalid JSON specified.", HTTPStatus.BAD_REQUEST)
 
+        if not isinstance(data, dict):
+            return self.json_message(
+                "State data should be a JSON object.", HTTPStatus.BAD_REQUEST
+            )
         if (new_state := data.get("state")) is None:
             return self.json_message("No state specified.", HTTPStatus.BAD_REQUEST)
 
@@ -284,7 +294,8 @@ class APIEntityStateView(HomeAssistantView):
 
         # Read the state back for our response
         status_code = HTTPStatus.CREATED if is_new_state else HTTPStatus.OK
-        assert (state := hass.states.get(entity_id))
+        state = hass.states.get(entity_id)
+        assert state
         resp = self.json(state.as_dict(), status_code)
 
         resp.headers.add("Location", f"/api/states/{entity_id}")
@@ -296,8 +307,7 @@ class APIEntityStateView(HomeAssistantView):
         """Remove entity."""
         if not request[KEY_HASS_USER].is_admin:
             raise Unauthorized(entity_id=entity_id)
-        hass: HomeAssistant = request.app[KEY_HASS]
-        if hass.states.async_remove(entity_id):
+        if request.app[KEY_HASS].states.async_remove(entity_id):
             return self.json_message("Entity removed.")
         return self.json_message("Entity not found.", HTTPStatus.NOT_FOUND)
 
@@ -311,8 +321,7 @@ class APIEventListenersView(HomeAssistantView):
     @ha.callback
     def get(self, request: web.Request) -> web.Response:
         """Get event listeners."""
-        hass: HomeAssistant = request.app[KEY_HASS]
-        return self.json(async_events_json(hass))
+        return self.json(async_events_json(request.app[KEY_HASS]))
 
 
 class APIEventView(HomeAssistantView):
@@ -346,8 +355,7 @@ class APIEventView(HomeAssistantView):
                 if state:
                     event_data[key] = state
 
-        hass: HomeAssistant = request.app[KEY_HASS]
-        hass.bus.async_fire(
+        request.app[KEY_HASS].bus.async_fire(
             event_type, event_data, ha.EventOrigin.remote, self.context(request)
         )
 
@@ -362,8 +370,7 @@ class APIServicesView(HomeAssistantView):
 
     async def get(self, request: web.Request) -> web.Response:
         """Get registered services."""
-        hass: HomeAssistant = request.app[KEY_HASS]
-        services = await async_services_json(hass)
+        services = await async_services_json(request.app[KEY_HASS])
         return self.json(services)
 
 
@@ -380,7 +387,7 @@ class APIDomainServicesView(HomeAssistantView):
 
         Returns a list of changed states.
         """
-        hass: HomeAssistant = request.app[KEY_HASS]
+        hass = request.app[KEY_HASS]
         body = await request.text()
         try:
             data = json_loads(body) if body else None
@@ -390,37 +397,63 @@ class APIDomainServicesView(HomeAssistantView):
             )
 
         context = self.context(request)
+        if not hass.services.has_service(domain, service):
+            raise HTTPBadRequest from ServiceNotFound(domain, service)
+
+        if response_requested := "return_response" in request.query:
+            if (
+                hass.services.supports_response(domain, service)
+                is ha.SupportsResponse.NONE
+            ):
+                return self.json_message(
+                    "Service does not support responses. Remove return_response from request.",
+                    HTTPStatus.BAD_REQUEST,
+                )
+        elif (
+            hass.services.supports_response(domain, service) is ha.SupportsResponse.ONLY
+        ):
+            return self.json_message(
+                "Service call requires responses but caller did not ask for responses. "
+                "Add ?return_response to query parameters.",
+                HTTPStatus.BAD_REQUEST,
+            )
+
         changed_states: list[json_fragment] = []
 
         @ha.callback
         def _async_save_changed_entities(
-            event: EventType[EventStateChangedData],
+            event: Event[EventStateChangedData],
         ) -> None:
             if event.context == context and (state := event.data["new_state"]):
                 changed_states.append(state.json_fragment)
 
         cancel_listen = hass.bus.async_listen(
             EVENT_STATE_CHANGED,
-            _async_save_changed_entities,  # type: ignore[arg-type]
-            run_immediately=True,
+            _async_save_changed_entities,
         )
 
         try:
             # shield the service call from cancellation on connection drop
-            await shield(
+            response = await shield(
                 hass.services.async_call(
                     domain,
                     service,
                     data,  # type: ignore[arg-type]
                     blocking=True,
                     context=context,
+                    return_response=response_requested,
                 )
             )
         except (vol.Invalid, ServiceNotFound) as ex:
-            raise HTTPBadRequest() from ex
+            raise HTTPBadRequest from ex
         finally:
             cancel_listen()
 
+        if response_requested:
+            return self.json(
+                {"changed_states": changed_states, "service_response": response}
+            )
+
         return self.json(changed_states)
 
 
@@ -433,8 +466,7 @@ class APIComponentsView(HomeAssistantView):
     @ha.callback
     def get(self, request: web.Request) -> web.Response:
         """Get current loaded components."""
-        hass: HomeAssistant = request.app[KEY_HASS]
-        return self.json(list(hass.config.components))
+        return self.json(request.app[KEY_HASS].config.components)
 
 
 @lru_cache
@@ -452,9 +484,19 @@ class APITemplateView(HomeAssistantView):
     @require_admin
     async def post(self, request: web.Request) -> web.Response:
         """Render a template."""
+        body = await request.text()
+
+        try:
+            data: Any = json_loads(body) if body else None
+        except ValueError:
+            return self.json_message("Invalid JSON specified.", HTTPStatus.BAD_REQUEST)
+
+        if not isinstance(data, dict):
+            return self.json_message(
+                "Template data should be a JSON object.", HTTPStatus.BAD_REQUEST
+            )
+        tpl = _cached_template(data["template"], request.app[KEY_HASS])
         try:
-            data = await request.json()
-            tpl = _cached_template(data["template"], request.app[KEY_HASS])
             return tpl.async_render(variables=data.get("variables"), parse_result=False)  # type: ignore[no-any-return]
         except (ValueError, TemplateError) as ex:
             return self.json_message(
@@ -471,8 +513,10 @@ class APIErrorLog(HomeAssistantView):
     @require_admin
     async def get(self, request: web.Request) -> web.FileResponse:
         """Retrieve API error log."""
-        hass: HomeAssistant = request.app[KEY_HASS]
-        return web.FileResponse(hass.data[DATA_LOGGING])
+        hass = request.app[KEY_HASS]
+        response = web.FileResponse(hass.data[DATA_LOGGING])
+        response.enable_compression()
+        return response
 
 
 async def async_services_json(hass: HomeAssistant) -> list[dict[str, Any]]:
