@@ -1,48 +1,145 @@
 """Support for Awair sensors."""
+
 from __future__ import annotations
 
-from typing import cast
+from dataclasses import dataclass
+from typing import Any, cast
 
 from python_awair.air_data import AirData
 from python_awair.devices import AwairBaseDevice, AwairLocalDevice
 
-from homeassistant.components.sensor import SensorEntity
+from homeassistant.components.sensor import (
+    SensorDeviceClass,
+    SensorEntity,
+    SensorEntityDescription,
+    SensorStateClass,
+)
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_CONNECTIONS, ATTR_SW_VERSION
+from homeassistant.const import (
+    ATTR_CONNECTIONS,
+    ATTR_SW_VERSION,
+    CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
+    CONCENTRATION_PARTS_PER_BILLION,
+    CONCENTRATION_PARTS_PER_MILLION,
+    LIGHT_LUX,
+    PERCENTAGE,
+    UnitOfSoundPressure,
+    UnitOfTemperature,
+)
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers import device_registry as dr
-from homeassistant.helpers.entity import DeviceInfo
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
-from . import AwairDataUpdateCoordinator, AwairResult
 from .const import (
+    API_CO2,
     API_DUST,
+    API_HUMID,
+    API_LUX,
+    API_PM10,
     API_PM25,
     API_SCORE,
+    API_SPL_A,
     API_TEMP,
     API_VOC,
     ATTRIBUTION,
     DOMAIN,
-    DUST_ALIASES,
-    SENSOR_TYPE_SCORE,
-    SENSOR_TYPES,
-    SENSOR_TYPES_DUST,
-    AwairSensorEntityDescription,
+)
+from .coordinator import AwairConfigEntry, AwairDataUpdateCoordinator
+
+DUST_ALIASES = [API_PM25, API_PM10]
+
+
+@dataclass(frozen=True, kw_only=True)
+class AwairSensorEntityDescription(SensorEntityDescription):
+    """Describes Awair sensor entity."""
+
+    unique_id_tag: str
+
+
+SENSOR_TYPE_SCORE = AwairSensorEntityDescription(
+    key=API_SCORE,
+    native_unit_of_measurement=PERCENTAGE,
+    translation_key="score",
+    unique_id_tag="score",  # matches legacy format
+    state_class=SensorStateClass.MEASUREMENT,
+)
+
+SENSOR_TYPES: tuple[AwairSensorEntityDescription, ...] = (
+    AwairSensorEntityDescription(
+        key=API_HUMID,
+        device_class=SensorDeviceClass.HUMIDITY,
+        native_unit_of_measurement=PERCENTAGE,
+        unique_id_tag="HUMID",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_LUX,
+        device_class=SensorDeviceClass.ILLUMINANCE,
+        native_unit_of_measurement=LIGHT_LUX,
+        unique_id_tag="illuminance",
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_SPL_A,
+        device_class=SensorDeviceClass.SOUND_PRESSURE,
+        native_unit_of_measurement=UnitOfSoundPressure.WEIGHTED_DECIBEL_A,
+        translation_key="sound_level",
+        unique_id_tag="sound_level",
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_VOC,
+        device_class=SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS_PARTS,
+        native_unit_of_measurement=CONCENTRATION_PARTS_PER_BILLION,
+        unique_id_tag="VOC",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_TEMP,
+        device_class=SensorDeviceClass.TEMPERATURE,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
+        unique_id_tag="TEMP",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_CO2,
+        device_class=SensorDeviceClass.CO2,
+        native_unit_of_measurement=CONCENTRATION_PARTS_PER_MILLION,
+        unique_id_tag="CO2",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+)
+
+SENSOR_TYPES_DUST: tuple[AwairSensorEntityDescription, ...] = (
+    AwairSensorEntityDescription(
+        key=API_PM25,
+        device_class=SensorDeviceClass.PM25,
+        native_unit_of_measurement=CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
+        unique_id_tag="PM25",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
+    AwairSensorEntityDescription(
+        key=API_PM10,
+        device_class=SensorDeviceClass.PM10,
+        native_unit_of_measurement=CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
+        unique_id_tag="PM10",  # matches legacy format
+        state_class=SensorStateClass.MEASUREMENT,
+    ),
 )
 
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    config_entry: AwairConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up Awair sensor entity based on a config entry."""
-    coordinator = hass.data[DOMAIN][config_entry.entry_id]
+    coordinator = config_entry.runtime_data
     entities = []
 
-    data: list[AwairResult] = coordinator.data.values()
-    for result in data:
+    for result in coordinator.data.values():
         if result.air_data:
             entities.append(AwairSensor(result.device, coordinator, SENSOR_TYPE_SCORE))
             device_sensors = result.air_data.sensors.keys()
@@ -156,7 +253,7 @@ class AwairSensor(CoordinatorEntity[AwairDataUpdateCoordinator], SensorEntity):
         return round(state, 2)
 
     @property
-    def extra_state_attributes(self) -> dict:
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Return the Awair Index alongside state attributes.
 
         The Awair Index is a subjective score ranging from 0-4 (inclusive) that
@@ -178,7 +275,7 @@ class AwairSensor(CoordinatorEntity[AwairDataUpdateCoordinator], SensorEntity):
         https://docs.developer.getawair.com/?version=latest#awair-score-and-index
         """
         sensor_type = self.entity_description.key
-        attrs: dict = {}
+        attrs: dict[str, Any] = {}
         if not self._air_data:
             return attrs
         if sensor_type in self._air_data.indices:
@@ -195,6 +292,7 @@ class AwairSensor(CoordinatorEntity[AwairDataUpdateCoordinator], SensorEntity):
             identifiers={(DOMAIN, self._device.uuid)},
             manufacturer="Awair",
             model=self._device.model,
+            model_id=self._device.device_type,
             name=(
                 self._device.name
                 or cast(ConfigEntry, self.coordinator.config_entry).title
