@@ -1,4 +1,5 @@
 """Support for Balboa Spa Wifi adaptor."""
+
 from __future__ import annotations
 
 from enum import IntEnum
@@ -13,7 +14,6 @@ from homeassistant.components.climate import (
     HVACAction,
     HVACMode,
 )
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_TEMPERATURE,
     PRECISION_HALVES,
@@ -21,8 +21,9 @@ from homeassistant.const import (
     UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
+from . import BalboaConfigEntry
 from .const import DOMAIN
 from .entity import BalboaEntity
 
@@ -44,16 +45,17 @@ TEMPERATURE_UNIT_MAP = {
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: BalboaConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the spa climate entity."""
-    async_add_entities([BalboaClimateEntity(hass.data[DOMAIN][entry.entry_id])])
+    async_add_entities([BalboaClimateEntity(entry.runtime_data)])
 
 
 class BalboaClimateEntity(BalboaEntity, ClimateEntity):
     """Representation of a Balboa spa climate entity."""
 
-    _attr_icon = "mdi:hot-tub"
     _attr_hvac_modes = [HVACMode.HEAT, HVACMode.OFF]
     _attr_supported_features = (
         ClimateEntityFeature.TARGET_TEMPERATURE
@@ -63,7 +65,6 @@ class BalboaClimateEntity(BalboaEntity, ClimateEntity):
     )
     _attr_translation_key = DOMAIN
     _attr_name = None
-    _enable_turn_on_off_backwards_compatibility = False
 
     def __init__(self, client: SpaClient) -> None:
         """Initialize the climate entity."""
