@@ -1,4 +1,5 @@
 """Support for BMW car locks with BMW ConnectedDrive."""
+
 from __future__ import annotations
 
 import logging
@@ -9,38 +10,33 @@ from bimmer_connected.vehicle import MyBMWVehicle
 from bimmer_connected.vehicle.doors_windows import LockState
 
 from homeassistant.components.lock import LockEntity
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from . import BMWBaseEntity
-from .const import DOMAIN
+from . import DOMAIN, BMWConfigEntry
 from .coordinator import BMWDataUpdateCoordinator
+from .entity import BMWBaseEntity
+
+PARALLEL_UPDATES = 1
 
 DOOR_LOCK_STATE = "door_lock_state"
+
 _LOGGER = logging.getLogger(__name__)
 
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    config_entry: BMWConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the MyBMW lock from config entry."""
-    coordinator: BMWDataUpdateCoordinator = hass.data[DOMAIN][config_entry.entry_id]
+    coordinator = config_entry.runtime_data
 
-    entities: list[BMWLock] = []
-
-    for vehicle in coordinator.account.vehicles:
-        if not coordinator.read_only:
-            entities.append(
-                BMWLock(
-                    coordinator,
-                    vehicle,
-                )
-            )
-    async_add_entities(entities)
+    if not coordinator.read_only:
+        async_add_entities(
+            BMWLock(coordinator, vehicle) for vehicle in coordinator.account.vehicles
+        )
 
 
 class BMWLock(BMWBaseEntity, LockEntity):
@@ -57,7 +53,7 @@ class BMWLock(BMWBaseEntity, LockEntity):
         super().__init__(coordinator, vehicle)
 
         self._attr_unique_id = f"{vehicle.vin}-lock"
-        self.door_lock_state_available = DOOR_LOCK_STATE in vehicle.available_attributes
+        self.door_lock_state_available = vehicle.is_lsc_enabled
 
     async def async_lock(self, **kwargs: Any) -> None:
         """Lock the car."""
@@ -71,11 +67,17 @@ class BMWLock(BMWBaseEntity, LockEntity):
         try:
             await self.vehicle.remote_services.trigger_remote_door_lock()
         except MyBMWAPIError as ex:
-            self._attr_is_locked = False
+            # Set the state to unknown if the command fails
+            self._attr_is_locked = None
             self.async_write_ha_state()
-            raise HomeAssistantError(ex) from ex
-
-        self.coordinator.async_update_listeners()
+            raise HomeAssistantError(
+                translation_domain=DOMAIN,
+                translation_key="remote_service_error",
+                translation_placeholders={"exception": str(ex)},
+            ) from ex
+        finally:
+            # Always update the listeners to get the latest state
+            self.coordinator.async_update_listeners()
 
     async def async_unlock(self, **kwargs: Any) -> None:
         """Unlock the car."""
@@ -89,18 +91,22 @@ class BMWLock(BMWBaseEntity, LockEntity):
         try:
             await self.vehicle.remote_services.trigger_remote_door_unlock()
         except MyBMWAPIError as ex:
-            self._attr_is_locked = True
+            # Set the state to unknown if the command fails
+            self._attr_is_locked = None
             self.async_write_ha_state()
-            raise HomeAssistantError(ex) from ex
-
-        self.coordinator.async_update_listeners()
+            raise HomeAssistantError(
+                translation_domain=DOMAIN,
+                translation_key="remote_service_error",
+                translation_placeholders={"exception": str(ex)},
+            ) from ex
+        finally:
+            # Always update the listeners to get the latest state
+            self.coordinator.async_update_listeners()
 
     @callback
     def _handle_coordinator_update(self) -> None:
         """Handle updated data from the coordinator."""
         _LOGGER.debug("Updating lock data of %s", self.vehicle.name)
-        # Set default attributes
-        self._attr_extra_state_attributes = self._attrs
 
         # Only update the HA state machine if the vehicle reliably reports its lock state
         if self.door_lock_state_available:
@@ -108,8 +114,8 @@ class BMWLock(BMWBaseEntity, LockEntity):
                 LockState.LOCKED,
                 LockState.SECURED,
             }
-            self._attr_extra_state_attributes[
-                "door_lock_state"
-            ] = self.vehicle.doors_and_windows.door_lock_state.value
+            self._attr_extra_state_attributes = {
+                DOOR_LOCK_STATE: self.vehicle.doors_and_windows.door_lock_state.value
+            }
 
         super()._handle_coordinator_update()
