@@ -1,4 +1,5 @@
 """Support for the CO2signal platform."""
+
 from __future__ import annotations
 
 from collections.abc import Callable
@@ -11,15 +12,14 @@ from homeassistant.components.sensor import (
     SensorEntityDescription,
     SensorStateClass,
 )
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import PERCENTAGE
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.device_registry import DeviceEntryType, DeviceInfo
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from .const import ATTRIBUTION, DOMAIN
-from .coordinator import CO2SignalCoordinator
+from .coordinator import CO2SignalConfigEntry, CO2SignalCoordinator
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -28,9 +28,9 @@ class CO2SensorEntityDescription(SensorEntityDescription):
 
     # For backwards compat, allow description to override unique ID key to use
     unique_id: str | None = None
-    unit_of_measurement_fn: Callable[
-        [CarbonIntensityResponse], str | None
-    ] | None = None
+    unit_of_measurement_fn: Callable[[CarbonIntensityResponse], str | None] | None = (
+        None
+    )
     value_fn: Callable[[CarbonIntensityResponse], float | None]
 
 
@@ -52,10 +52,12 @@ SENSORS = (
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: CO2SignalConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the CO2signal sensor."""
-    coordinator: CO2SignalCoordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator = entry.runtime_data
     async_add_entities(
         [CO2Sensor(coordinator, description) for description in SENSORS], False
     )
