@@ -1,7 +1,14 @@
 """Offer API to configure Home Assistant auth."""
+
+from __future__ import annotations
+
+from typing import Any
+
 import voluptuous as vol
 
+from homeassistant.auth.models import User
 from homeassistant.components import websocket_api
+from homeassistant.core import HomeAssistant, callback
 
 WS_TYPE_LIST = "config/auth/list"
 SCHEMA_WS_LIST = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
@@ -14,22 +21,27 @@ SCHEMA_WS_DELETE = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
 )
 
 
-async def async_setup(hass):
+@callback
+def async_setup(hass: HomeAssistant) -> bool:
     """Enable the Home Assistant views."""
-    hass.components.websocket_api.async_register_command(
-        WS_TYPE_LIST, websocket_list, SCHEMA_WS_LIST
+    websocket_api.async_register_command(
+        hass, WS_TYPE_LIST, websocket_list, SCHEMA_WS_LIST
     )
-    hass.components.websocket_api.async_register_command(
-        WS_TYPE_DELETE, websocket_delete, SCHEMA_WS_DELETE
+    websocket_api.async_register_command(
+        hass, WS_TYPE_DELETE, websocket_delete, SCHEMA_WS_DELETE
     )
-    hass.components.websocket_api.async_register_command(websocket_create)
-    hass.components.websocket_api.async_register_command(websocket_update)
+    websocket_api.async_register_command(hass, websocket_create)
+    websocket_api.async_register_command(hass, websocket_update)
     return True
 
 
 @websocket_api.require_admin
 @websocket_api.async_response
-async def websocket_list(hass, connection, msg):
+async def websocket_list(
+    hass: HomeAssistant,
+    connection: websocket_api.ActiveConnection,
+    msg: dict[str, Any],
+) -> None:
     """Return a list of users."""
     result = [_user_info(u) for u in await hass.auth.async_get_users()]
 
@@ -38,7 +50,11 @@ async def websocket_list(hass, connection, msg):
 
 @websocket_api.require_admin
 @websocket_api.async_response
-async def websocket_delete(hass, connection, msg):
+async def websocket_delete(
+    hass: HomeAssistant,
+    connection: websocket_api.ActiveConnection,
+    msg: dict[str, Any],
+) -> None:
     """Delete a user."""
     if msg["user_id"] == connection.user.id:
         connection.send_message(
@@ -60,7 +76,6 @@ async def websocket_delete(hass, connection, msg):
 
 
 @websocket_api.require_admin
-@websocket_api.async_response
 @websocket_api.websocket_command(
     {
         vol.Required("type"): "config/auth/create",
@@ -69,7 +84,12 @@ async def websocket_delete(hass, connection, msg):
         vol.Optional("local_only"): bool,
     }
 )
-async def websocket_create(hass, connection, msg):
+@websocket_api.async_response
+async def websocket_create(
+    hass: HomeAssistant,
+    connection: websocket_api.ActiveConnection,
+    msg: dict[str, Any],
+) -> None:
     """Create a user."""
     user = await hass.auth.async_create_user(
         msg["name"], group_ids=msg.get("group_ids"), local_only=msg.get("local_only")
@@ -81,7 +101,6 @@ async def websocket_create(hass, connection, msg):
 
 
 @websocket_api.require_admin
-@websocket_api.async_response
 @websocket_api.websocket_command(
     {
         vol.Required("type"): "config/auth/update",
@@ -92,12 +111,17 @@ async def websocket_create(hass, connection, msg):
         vol.Optional("local_only"): bool,
     }
 )
-async def websocket_update(hass, connection, msg):
+@websocket_api.async_response
+async def websocket_update(
+    hass: HomeAssistant,
+    connection: websocket_api.ActiveConnection,
+    msg: dict[str, Any],
+) -> None:
     """Update a user."""
     if not (user := await hass.auth.async_get_user(msg.pop("user_id"))):
         connection.send_message(
             websocket_api.error_message(
-                msg["id"], websocket_api.const.ERR_NOT_FOUND, "User not found"
+                msg["id"], websocket_api.ERR_NOT_FOUND, "User not found"
             )
         )
         return
@@ -132,7 +156,7 @@ async def websocket_update(hass, connection, msg):
     )
 
 
-def _user_info(user):
+def _user_info(user: User) -> dict[str, Any]:
     """Format a user."""
 
     ha_username = next(
