@@ -9,18 +9,18 @@ from pydeconz.models.scene import Scene as PydeconzScene
 from pydeconz.models.sensor.presence import Presence
 
 from homeassistant.components.button import (
-    DOMAIN,
+    DOMAIN as BUTTON_DOMAIN,
     ButtonDeviceClass,
     ButtonEntity,
     ButtonEntityDescription,
 )
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from .deconz_device import DeconzDevice, DeconzSceneMixin
-from .gateway import DeconzGateway, get_gateway_from_config_entry
+from . import DeconzConfigEntry
+from .entity import DeconzDevice, DeconzSceneMixin
+from .hub import DeconzHub
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -46,61 +46,61 @@ ENTITY_DESCRIPTIONS = {
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    config_entry: DeconzConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the deCONZ button entity."""
-    gateway = get_gateway_from_config_entry(hass, config_entry)
-    gateway.entities[DOMAIN] = set()
+    hub = config_entry.runtime_data
+    hub.entities[BUTTON_DOMAIN] = set()
 
     @callback
     def async_add_scene(_: EventType, scene_id: str) -> None:
         """Add scene button from deCONZ."""
-        scene = gateway.api.scenes[scene_id]
+        scene = hub.api.scenes[scene_id]
         async_add_entities(
-            DeconzSceneButton(scene, gateway, description)
+            DeconzSceneButton(scene, hub, description)
             for description in ENTITY_DESCRIPTIONS.get(PydeconzScene, [])
         )
 
-    gateway.register_platform_add_device_callback(
+    hub.register_platform_add_device_callback(
         async_add_scene,
-        gateway.api.scenes,
+        hub.api.scenes,
     )
 
     @callback
     def async_add_presence_sensor(_: EventType, sensor_id: str) -> None:
         """Add presence sensor reset button from deCONZ."""
-        sensor = gateway.api.sensors.presence[sensor_id]
+        sensor = hub.api.sensors.presence[sensor_id]
         if sensor.presence_event is not None:
-            async_add_entities([DeconzPresenceResetButton(sensor, gateway)])
+            async_add_entities([DeconzPresenceResetButton(sensor, hub)])
 
-    gateway.register_platform_add_device_callback(
+    hub.register_platform_add_device_callback(
         async_add_presence_sensor,
-        gateway.api.sensors.presence,
+        hub.api.sensors.presence,
     )
 
 
 class DeconzSceneButton(DeconzSceneMixin, ButtonEntity):
     """Representation of a deCONZ button entity."""
 
-    TYPE = DOMAIN
+    TYPE = BUTTON_DOMAIN
 
     def __init__(
         self,
         device: PydeconzScene,
-        gateway: DeconzGateway,
+        hub: DeconzHub,
         description: DeconzButtonDescription,
     ) -> None:
         """Initialize deCONZ number entity."""
         self.entity_description: DeconzButtonDescription = description
-        super().__init__(device, gateway)
+        super().__init__(device, hub)
 
         self._attr_name = f"{self._attr_name} {description.suffix}"
 
     async def async_press(self) -> None:
         """Store light states into scene."""
         async_button_fn = getattr(
-            self.gateway.api.scenes,
+            self.hub.api.scenes,
             self.entity_description.button_fn,
         )
         await async_button_fn(self._device.group_id, self._device.id)
@@ -119,11 +119,11 @@ class DeconzPresenceResetButton(DeconzDevice[Presence], ButtonEntity):
     _attr_entity_category = EntityCategory.CONFIG
     _attr_device_class = ButtonDeviceClass.RESTART
 
-    TYPE = DOMAIN
+    TYPE = BUTTON_DOMAIN
 
     async def async_press(self) -> None:
         """Store reset presence state."""
-        await self.gateway.api.sensors.presence.set_config(
+        await self.hub.api.sensors.presence.set_config(
             id=self._device.resource_id,
             reset_presence=True,
         )
