@@ -1,18 +1,26 @@
 """Support for Ecobee binary sensors."""
+
 from __future__ import annotations
 
 from homeassistant.components.binary_sensor import (
-    DEVICE_CLASS_OCCUPANCY,
+    BinarySensorDeviceClass,
     BinarySensorEntity,
 )
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
+from . import EcobeeConfigEntry
 from .const import DOMAIN, ECOBEE_MODEL_TO_NAME, MANUFACTURER
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: EcobeeConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
+) -> None:
     """Set up ecobee binary (occupancy) sensors."""
-    data = hass.data[DOMAIN]
+    data = config_entry.runtime_data
     dev = []
     for index in range(len(data.ecobee.thermostats)):
         for sensor in data.ecobee.get_remote_sensors(index):
@@ -28,21 +36,17 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class EcobeeBinarySensor(BinarySensorEntity):
     """Representation of an Ecobee sensor."""
 
+    _attr_device_class = BinarySensorDeviceClass.OCCUPANCY
+    _attr_has_entity_name = True
+
     def __init__(self, data, sensor_name, sensor_index):
         """Initialize the Ecobee sensor."""
         self.data = data
-        self._name = f"{sensor_name} Occupancy"
         self.sensor_name = sensor_name
         self.index = sensor_index
-        self._state = None
 
     @property
-    def name(self):
-        """Return the name of the Ecobee sensor."""
-        return self._name.rstrip()
-
-    @property
-    def unique_id(self):
+    def unique_id(self) -> str | None:
         """Return a unique identifier for this sensor."""
         for sensor in self.data.ecobee.get_remote_sensors(self.index):
             if sensor["name"] == self.sensor_name:
@@ -50,6 +54,7 @@ class EcobeeBinarySensor(BinarySensorEntity):
                     return f"{sensor['code']}-{self.device_class}"
                 thermostat = self.data.ecobee.get_thermostat(self.index)
                 return f"{thermostat['identifier']}-{sensor['id']}-{self.device_class}"
+        return None
 
     @property
     def device_info(self) -> DeviceInfo | None:
@@ -84,22 +89,12 @@ class EcobeeBinarySensor(BinarySensorEntity):
         return None
 
     @property
-    def available(self):
+    def available(self) -> bool:
         """Return true if device is available."""
         thermostat = self.data.ecobee.get_thermostat(self.index)
         return thermostat["runtime"]["connected"]
 
-    @property
-    def is_on(self):
-        """Return the status of the sensor."""
-        return self._state == "true"
-
-    @property
-    def device_class(self):
-        """Return the class of this sensor, from DEVICE_CLASSES."""
-        return DEVICE_CLASS_OCCUPANCY
-
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Get the latest state of the sensor."""
         await self.data.update()
         for sensor in self.data.ecobee.get_remote_sensors(self.index):
@@ -108,5 +103,5 @@ class EcobeeBinarySensor(BinarySensorEntity):
             for item in sensor["capability"]:
                 if item["type"] != "occupancy":
                     continue
-                self._state = item["value"]
+                self._attr_is_on = item["value"] == "true"
                 break
