@@ -1,47 +1,38 @@
 """Support for EZVIZ button controls."""
+
 from __future__ import annotations
 
 from collections.abc import Callable
 from dataclasses import dataclass
 from typing import Any
 
-from pyezviz import EzvizClient
-from pyezviz.constants import SupportExt
-from pyezviz.exceptions import HTTPError, PyEzvizError
+from pyezvizapi import EzvizClient
+from pyezvizapi.constants import SupportExt
+from pyezvizapi.exceptions import HTTPError, PyEzvizError
 
 from homeassistant.components.button import ButtonEntity, ButtonEntityDescription
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import HomeAssistantError
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from .const import DATA_COORDINATOR, DOMAIN
-from .coordinator import EzvizDataUpdateCoordinator
+from .coordinator import EzvizConfigEntry, EzvizDataUpdateCoordinator
 from .entity import EzvizEntity
 
 PARALLEL_UPDATES = 1
 
 
-@dataclass(frozen=True)
-class EzvizButtonEntityDescriptionMixin:
-    """Mixin values for EZVIZ button entities."""
+@dataclass(frozen=True, kw_only=True)
+class EzvizButtonEntityDescription(ButtonEntityDescription):
+    """Describe a EZVIZ Button."""
 
     method: Callable[[EzvizClient, str, str], Any]
     supported_ext: str
 
 
-@dataclass(frozen=True)
-class EzvizButtonEntityDescription(
-    ButtonEntityDescription, EzvizButtonEntityDescriptionMixin
-):
-    """Describe a EZVIZ Button."""
-
-
 BUTTON_ENTITIES = (
     EzvizButtonEntityDescription(
         key="ptz_up",
         translation_key="ptz_up",
-        icon="mdi:pan",
         method=lambda pyezviz_client, serial, run: pyezviz_client.ptz_control(
             "UP", serial, run
         ),
@@ -50,7 +41,6 @@ BUTTON_ENTITIES = (
     EzvizButtonEntityDescription(
         key="ptz_down",
         translation_key="ptz_down",
-        icon="mdi:pan",
         method=lambda pyezviz_client, serial, run: pyezviz_client.ptz_control(
             "DOWN", serial, run
         ),
@@ -59,7 +49,6 @@ BUTTON_ENTITIES = (
     EzvizButtonEntityDescription(
         key="ptz_left",
         translation_key="ptz_left",
-        icon="mdi:pan",
         method=lambda pyezviz_client, serial, run: pyezviz_client.ptz_control(
             "LEFT", serial, run
         ),
@@ -68,7 +57,6 @@ BUTTON_ENTITIES = (
     EzvizButtonEntityDescription(
         key="ptz_right",
         translation_key="ptz_right",
-        icon="mdi:pan",
         method=lambda pyezviz_client, serial, run: pyezviz_client.ptz_control(
             "RIGHT", serial, run
         ),
@@ -78,12 +66,12 @@ BUTTON_ENTITIES = (
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: EzvizConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up EZVIZ button based on a config entry."""
-    coordinator: EzvizDataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id][
-        DATA_COORDINATOR
-    ]
+    coordinator = entry.runtime_data
 
     # Add button entities if supportExt value indicates PTZ capbility.
     # Could be missing or "0" for unsupported.
@@ -92,9 +80,9 @@ async def async_setup_entry(
     async_add_entities(
         EzvizButtonEntity(coordinator, camera, entity_description)
         for camera in coordinator.data
-        for capibility, value in coordinator.data[camera]["supportExt"].items()
+        for capability, value in coordinator.data[camera]["supportExt"].items()
         for entity_description in BUTTON_ENTITIES
-        if capibility == entity_description.supported_ext
+        if capability == entity_description.supported_ext
         if value == "1"
     )
 
