@@ -1,11 +1,15 @@
 """The IntelliFire integration."""
+
 from __future__ import annotations
 
+from dataclasses import dataclass
 from typing import Any
 
 import pyflume
 from pyflume import FlumeAuth, FlumeData, FlumeDeviceList
+from requests import Session
 
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 
@@ -18,13 +22,34 @@ from .const import (
 )
 
 
+@dataclass
+class FlumeRuntimeData:
+    """Runtime data for the Flume config entry."""
+
+    devices: FlumeDeviceList
+    auth: FlumeAuth
+    http_session: Session
+    notifications_coordinator: FlumeNotificationDataUpdateCoordinator
+
+
+type FlumeConfigEntry = ConfigEntry[FlumeRuntimeData]
+
+
 class FlumeDeviceDataUpdateCoordinator(DataUpdateCoordinator[None]):
     """Data update coordinator for an individual flume device."""
 
-    def __init__(self, hass: HomeAssistant, flume_device: FlumeData) -> None:
+    config_entry: FlumeConfigEntry
+
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        config_entry: FlumeConfigEntry,
+        flume_device: FlumeData,
+    ) -> None:
         """Initialize the Coordinator."""
         super().__init__(
             hass,
+            config_entry=config_entry,
             name=DOMAIN,
             logger=_LOGGER,
             update_interval=DEVICE_SCAN_INTERVAL,
@@ -48,10 +73,18 @@ class FlumeDeviceDataUpdateCoordinator(DataUpdateCoordinator[None]):
 class FlumeDeviceConnectionUpdateCoordinator(DataUpdateCoordinator[None]):
     """Date update coordinator to read connected status from Devices endpoint."""
 
-    def __init__(self, hass: HomeAssistant, flume_devices: FlumeDeviceList) -> None:
+    config_entry: FlumeConfigEntry
+
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        config_entry: FlumeConfigEntry,
+        flume_devices: FlumeDeviceList,
+    ) -> None:
         """Initialize the Coordinator."""
         super().__init__(
             hass,
+            config_entry=config_entry,
             name=DOMAIN,
             logger=_LOGGER,
             update_interval=DEVICE_CONNECTION_SCAN_INTERVAL,
@@ -79,10 +112,15 @@ class FlumeDeviceConnectionUpdateCoordinator(DataUpdateCoordinator[None]):
 class FlumeNotificationDataUpdateCoordinator(DataUpdateCoordinator[None]):
     """Data update coordinator for flume notifications."""
 
-    def __init__(self, hass: HomeAssistant, auth: FlumeAuth) -> None:
+    config_entry: FlumeConfigEntry
+
+    def __init__(
+        self, hass: HomeAssistant, config_entry: FlumeConfigEntry, auth: FlumeAuth
+    ) -> None:
         """Initialize the Coordinator."""
         super().__init__(
             hass,
+            config_entry=config_entry,
             name=DOMAIN,
             logger=_LOGGER,
             update_interval=NOTIFICATION_SCAN_INTERVAL,
