@@ -1,4 +1,5 @@
 """Geolocation support for GeoNet NZ Quakes Feeds."""
+
 from __future__ import annotations
 
 from collections.abc import Callable
@@ -8,17 +9,15 @@ from typing import Any
 from aio_geojson_geonetnz_quakes.feed_entry import GeonetnzQuakesFeedEntry
 
 from homeassistant.components.geo_location import GeolocationEvent
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_TIME, UnitOfLength
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.util.unit_conversion import DistanceConverter
 from homeassistant.util.unit_system import US_CUSTOMARY_SYSTEM
 
-from . import GeonetnzQuakesFeedEntityManager
-from .const import DOMAIN, FEED
+from . import GeonetnzQuakesConfigEntry, GeonetnzQuakesFeedEntityManager
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -37,10 +36,12 @@ SOURCE = "geonetnz_quakes"
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: GeonetnzQuakesConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the GeoNet NZ Quakes Feed platform."""
-    manager: GeonetnzQuakesFeedEntityManager = hass.data[DOMAIN][FEED][entry.entry_id]
+    manager = entry.runtime_data
 
     @callback
     def async_add_geolocation(
@@ -155,16 +156,16 @@ class GeonetnzQuakesEvent(GeolocationEvent):
     @property
     def extra_state_attributes(self) -> dict[str, Any]:
         """Return the device state attributes."""
-        attributes = {}
-        for key, value in (
-            (ATTR_EXTERNAL_ID, self._external_id),
-            (ATTR_DEPTH, self._depth),
-            (ATTR_LOCALITY, self._locality),
-            (ATTR_MAGNITUDE, self._magnitude),
-            (ATTR_MMI, self._mmi),
-            (ATTR_QUALITY, self._quality),
-            (ATTR_TIME, self._time),
-        ):
-            if value or isinstance(value, bool):
-                attributes[key] = value
-        return attributes
+        return {
+            key: value
+            for key, value in (
+                (ATTR_EXTERNAL_ID, self._external_id),
+                (ATTR_DEPTH, self._depth),
+                (ATTR_LOCALITY, self._locality),
+                (ATTR_MAGNITUDE, self._magnitude),
+                (ATTR_MMI, self._mmi),
+                (ATTR_QUALITY, self._quality),
+                (ATTR_TIME, self._time),
+            )
+            if value or isinstance(value, bool)
+        }
