@@ -1,16 +1,15 @@
 """Support for Harmony Hub select activities."""
+
 from __future__ import annotations
 
 import logging
 
 from homeassistant.components.select import SelectEntity
-from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_NAME
-from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.core import HassJob, HomeAssistant, callback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from .const import ACTIVITY_POWER_OFF, DOMAIN, HARMONY_DATA
-from .data import HarmonyData
+from .const import ACTIVITY_POWER_OFF, DOMAIN
+from .data import HarmonyConfigEntry, HarmonyData
 from .entity import HarmonyEntity
 from .subscriber import HarmonyCallback
 
@@ -20,14 +19,12 @@ TRANSLATABLE_POWER_OFF = "power_off"
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: HarmonyConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up harmony activities select."""
-    data: HarmonyData = hass.data[DOMAIN][entry.entry_id][HARMONY_DATA]
-    _LOGGER.debug("creating select for %s hub activities", entry.data[CONF_NAME])
-    async_add_entities(
-        [HarmonyActivitySelect(f"{entry.data[CONF_NAME]} Activities", data)]
-    )
+    async_add_entities([HarmonyActivitySelect(entry.runtime_data)])
 
 
 class HarmonyActivitySelect(HarmonyEntity, SelectEntity):
@@ -35,25 +32,17 @@ class HarmonyActivitySelect(HarmonyEntity, SelectEntity):
 
     _attr_translation_key = "activities"
 
-    def __init__(self, name: str, data: HarmonyData) -> None:
+    def __init__(self, data: HarmonyData) -> None:
         """Initialize HarmonyActivitySelect class."""
         super().__init__(data=data)
         self._data = data
         self._attr_unique_id = self._data.unique_id
         self._attr_device_info = self._data.device_info(DOMAIN)
-        self._attr_name = name
-
-    @property
-    def icon(self) -> str:
-        """Return a representative icon."""
-        if not self.available or self.current_option == TRANSLATABLE_POWER_OFF:
-            return "mdi:remote-tv-off"
-        return "mdi:remote-tv"
 
     @property
     def options(self) -> list[str]:
         """Return a set of selectable options."""
-        return [TRANSLATABLE_POWER_OFF] + sorted(self._data.activity_names)
+        return [TRANSLATABLE_POWER_OFF, *sorted(self._data.activity_names)]
 
     @property
     def current_option(self) -> str | None:
@@ -72,13 +61,14 @@ class HarmonyActivitySelect(HarmonyEntity, SelectEntity):
 
     async def async_added_to_hass(self) -> None:
         """Call when entity is added to hass."""
+        activity_update_job = HassJob(self._async_activity_update)
         self.async_on_remove(
             self._data.async_subscribe(
                 HarmonyCallback(
-                    connected=self.async_got_connected,
-                    disconnected=self.async_got_disconnected,
-                    activity_starting=self._async_activity_update,
-                    activity_started=self._async_activity_update,
+                    connected=HassJob(self.async_got_connected),
+                    disconnected=HassJob(self.async_got_disconnected),
+                    activity_starting=activity_update_job,
+                    activity_started=activity_update_job,
                     config_updated=None,
                 )
             )
