@@ -1,10 +1,13 @@
 """Model Classes for here_travel_time."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
-from datetime import time
+from datetime import datetime
 from typing import TypedDict
 
+from here_routing import RoutingMode
+
 
 class HERETravelTimeData(TypedDict):
     """Routing information."""
@@ -20,16 +23,12 @@ class HERETravelTimeData(TypedDict):
 
 
 @dataclass
-class HERETravelTimeConfig:
-    """Configuration for HereTravelTimeDataUpdateCoordinator."""
+class HERETravelTimeAPIParams:
+    """Configuration for polling the HERE API."""
 
-    destination_latitude: float | None
-    destination_longitude: float | None
-    destination_entity_id: str | None
-    origin_latitude: float | None
-    origin_longitude: float | None
-    origin_entity_id: str | None
+    destination: list[str]
+    origin: list[str]
     travel_mode: str
-    route_mode: str
-    arrival: time | None
-    departure: time | None
+    route_mode: RoutingMode
+    arrival: datetime | None
+    departure: datetime | None
