@@ -87,7 +87,9 @@ def get_knx_module(hass: HomeAssistant) -> KNXModule:
     try:
         return hass.data[KNX_MODULE_KEY]
     except KeyError as err:
-        raise HomeAssistantError("KNX entry not loaded") from err
+        raise HomeAssistantError(
+            translation_domain=DOMAIN, translation_key="integration_not_loaded"
+        ) from err
 
 
 SERVICE_KNX_EVENT_REGISTER_SCHEMA = vol.Schema(
@@ -126,7 +128,7 @@ async def service_event_register_modify(call: ServiceCall) -> None:
         transcoder := DPTBase.parse_transcoder(dpt)
     ):
         knx_module.group_address_transcoder.update(
-            {_address: transcoder for _address in group_addresses}
+            dict.fromkeys(group_addresses, transcoder)
         )
     for group_address in group_addresses:
         if group_address in knx_module.knx_event_callback.group_addresses:
@@ -166,7 +168,11 @@ async def service_exposure_register_modify(call: ServiceCall) -> None:
             removed_exposure = knx_module.service_exposures.pop(group_address)
         except KeyError as err:
             raise ServiceValidationError(
-                f"Could not find exposure for '{group_address}' to remove."
+                translation_domain=DOMAIN,
+                translation_key="service_exposure_remove_not_found",
+                translation_placeholders={
+                    "group_address": group_address,
+                },
             ) from err
 
         removed_exposure.async_remove()
@@ -234,13 +240,17 @@ async def service_send_to_knx_bus(call: ServiceCall) -> None:
         transcoder = DPTBase.parse_transcoder(attr_type)
         if transcoder is None:
             raise ServiceValidationError(
-                f"Invalid type for knx.send service: {attr_type}"
+                translation_domain=DOMAIN,
+                translation_key="service_send_invalid_type",
+                translation_placeholders={"type": attr_type},
             )
         try:
             payload = transcoder.to_knx(attr_payload)
         except ConversionError as err:
             raise ServiceValidationError(
-                f"Invalid payload for knx.send service: {err}"
+                translation_domain=DOMAIN,
+                translation_key="service_send_invalid_payload",
+                translation_placeholders={"error": str(err)},
             ) from err
     elif isinstance(attr_payload, int):
         payload = DPTBinary(attr_payload)
