@@ -1,6 +1,9 @@
 """Support for LG soundbars."""
+
 from __future__ import annotations
 
+from typing import Any
+
 import temescal
 
 from homeassistant.components.media_player import (
@@ -12,7 +15,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_HOST, CONF_PORT
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.device_registry import DeviceInfo
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
 from .const import DOMAIN
 
@@ -20,7 +23,7 @@ from .const import DOMAIN
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up media_player from a config entry created in the integrations UI."""
     async_add_entities(
@@ -42,6 +45,8 @@ class LGDevice(MediaPlayerEntity):
     _attr_supported_features = (
         MediaPlayerEntityFeature.VOLUME_SET
         | MediaPlayerEntityFeature.VOLUME_MUTE
+        | MediaPlayerEntityFeature.TURN_ON
+        | MediaPlayerEntityFeature.TURN_OFF
         | MediaPlayerEntityFeature.SELECT_SOURCE
         | MediaPlayerEntityFeature.SELECT_SOUND_MODE
     )
@@ -90,16 +95,9 @@ class LGDevice(MediaPlayerEntity):
 
     def handle_event(self, response):
         """Handle responses from the speakers."""
-        data = response["data"] if "data" in response else {}
+        data = response.get("data") or {}
         if response["msg"] == "EQ_VIEW_INFO":
-            if "i_bass" in data:
-                self._bass = data["i_bass"]
-            if "i_treble" in data:
-                self._treble = data["i_treble"]
-            if "ai_eq_list" in data:
-                self._equalisers = data["ai_eq_list"]
-            if "i_curr_eq" in data:
-                self._equaliser = data["i_curr_eq"]
+            self._update_equalisers(data)
         elif response["msg"] == "SPK_LIST_VIEW_INFO":
             if "i_vol" in data:
                 self._volume = data["i_vol"]
@@ -111,6 +109,11 @@ class LGDevice(MediaPlayerEntity):
                 self._mute = data["b_mute"]
             if "i_curr_func" in data:
                 self._function = data["i_curr_func"]
+            if "b_powerstatus" in data:
+                if data["b_powerstatus"]:
+                    self._attr_state = MediaPlayerState.ON
+                else:
+                    self._attr_state = MediaPlayerState.OFF
         elif response["msg"] == "FUNC_VIEW_INFO":
             if "i_curr_func" in data:
                 self._function = data["i_curr_func"]
@@ -136,6 +139,17 @@ class LGDevice(MediaPlayerEntity):
 
         self.schedule_update_ha_state()
 
+    def _update_equalisers(self, data: dict[str, Any]) -> None:
+        """Update the equalisers."""
+        if "i_bass" in data:
+            self._bass = data["i_bass"]
+        if "i_treble" in data:
+            self._treble = data["i_treble"]
+        if "ai_eq_list" in data:
+            self._equalisers = data["ai_eq_list"]
+        if "i_curr_eq" in data:
+            self._equaliser = data["i_curr_eq"]
+
     def update(self) -> None:
         """Trigger updates from the device."""
         self._device.get_eq()
@@ -165,11 +179,11 @@ class LGDevice(MediaPlayerEntity):
     @property
     def sound_mode_list(self):
         """Return the available sound modes."""
-        modes = []
-        for equaliser in self._equalisers:
-            if equaliser < len(temescal.equalisers):
-                modes.append(temescal.equalisers[equaliser])
-        return sorted(modes)
+        return sorted(
+            temescal.equalisers[equaliser]
+            for equaliser in self._equalisers
+            if equaliser < len(temescal.equalisers)
+        )
 
     @property
     def source(self):
@@ -181,11 +195,11 @@ class LGDevice(MediaPlayerEntity):
     @property
     def source_list(self):
         """List of available input sources."""
-        sources = []
-        for function in self._functions:
-            if function < len(temescal.functions):
-                sources.append(temescal.functions[function])
-        return sorted(sources)
+        return sorted(
+            temescal.functions[function]
+            for function in self._functions
+            if function < len(temescal.functions)
+        )
 
     def set_volume_level(self, volume: float) -> None:
         """Set volume level, range 0..1."""
@@ -203,3 +217,17 @@ class LGDevice(MediaPlayerEntity):
     def select_sound_mode(self, sound_mode: str) -> None:
         """Set Sound Mode for Receiver.."""
         self._device.set_eq(temescal.equalisers.index(sound_mode))
+
+    def turn_on(self) -> None:
+        """Turn the media player on."""
+        self._set_power(True)
+
+    def turn_off(self) -> None:
+        """Turn the media player off."""
+        self._set_power(False)
+
+    def _set_power(self, status: bool) -> None:
+        """Set the media player state."""
+        self._device.send_packet(
+            {"cmd": "set", "data": {"b_powerkey": status}, "msg": "SPK_LIST_VIEW_INFO"}
+        )
