@@ -1,4 +1,5 @@
 """Provides device automations for Media player."""
+
 from __future__ import annotations
 
 import voluptuous as vol
@@ -10,6 +11,7 @@ from homeassistant.const import (
     CONF_DOMAIN,
     CONF_ENTITY_ID,
     CONF_TYPE,
+    STATE_BUFFERING,
     STATE_IDLE,
     STATE_OFF,
     STATE_ON,
@@ -17,17 +19,28 @@ from homeassistant.const import (
     STATE_PLAYING,
 )
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import condition, config_validation as cv, entity_registry
+from homeassistant.helpers import (
+    condition,
+    config_validation as cv,
+    entity_registry as er,
+)
 from homeassistant.helpers.config_validation import DEVICE_CONDITION_BASE_SCHEMA
 from homeassistant.helpers.typing import ConfigType, TemplateVarsType
 
-from . import DOMAIN
+from .const import DOMAIN
 
-CONDITION_TYPES = {"is_on", "is_off", "is_idle", "is_paused", "is_playing"}
+CONDITION_TYPES = {
+    "is_on",
+    "is_off",
+    "is_buffering",
+    "is_idle",
+    "is_paused",
+    "is_playing",
+}
 
 CONDITION_SCHEMA = DEVICE_CONDITION_BASE_SCHEMA.extend(
     {
-        vol.Required(CONF_ENTITY_ID): cv.entity_id,
+        vol.Required(CONF_ENTITY_ID): cv.entity_id_or_uuid,
         vol.Required(CONF_TYPE): vol.In(CONDITION_TYPES),
     }
 )
@@ -37,11 +50,11 @@ async def async_get_conditions(
     hass: HomeAssistant, device_id: str
 ) -> list[dict[str, str]]:
     """List device conditions for Media player devices."""
-    registry = await entity_registry.async_get_registry(hass)
-    conditions = []
+    registry = er.async_get(hass)
+    conditions: list[dict[str, str]] = []
 
     # Get all the integrations entities for this device
-    for entry in entity_registry.async_entries_for_device(registry, device_id):
+    for entry in er.async_entries_for_device(registry, device_id):
         if entry.domain != DOMAIN:
             continue
 
@@ -50,7 +63,7 @@ async def async_get_conditions(
             CONF_CONDITION: "device",
             CONF_DEVICE_ID: device_id,
             CONF_DOMAIN: DOMAIN,
-            CONF_ENTITY_ID: entry.entity_id,
+            CONF_ENTITY_ID: entry.id,
         }
 
         conditions += [{**base_condition, CONF_TYPE: cond} for cond in CONDITION_TYPES]
@@ -63,19 +76,24 @@ def async_condition_from_config(
     hass: HomeAssistant, config: ConfigType
 ) -> condition.ConditionCheckerType:
     """Create a function to test a device condition."""
-    if config[CONF_TYPE] == "is_playing":
-        state = STATE_PLAYING
+    if config[CONF_TYPE] == "is_buffering":
+        state = STATE_BUFFERING
     elif config[CONF_TYPE] == "is_idle":
         state = STATE_IDLE
-    elif config[CONF_TYPE] == "is_paused":
-        state = STATE_PAUSED
+    elif config[CONF_TYPE] == "is_off":
+        state = STATE_OFF
     elif config[CONF_TYPE] == "is_on":
         state = STATE_ON
-    else:
-        state = STATE_OFF
+    elif config[CONF_TYPE] == "is_paused":
+        state = STATE_PAUSED
+    else:  # is_playing
+        state = STATE_PLAYING
+
+    registry = er.async_get(hass)
+    entity_id = er.async_resolve_entity_id(registry, config[ATTR_ENTITY_ID])
 
     def test_is_state(hass: HomeAssistant, variables: TemplateVarsType) -> bool:
         """Test if an entity is a certain state."""
-        return condition.state(hass, config[ATTR_ENTITY_ID], state)
+        return condition.state(hass, entity_id, state)
 
     return test_is_state
