@@ -24,6 +24,7 @@ from homeassistant.components.climate import (
     SWING_VERTICAL,
     ClimateEntity,
     ClimateEntityFeature,
+    HVACAction,
     HVACMode,
 )
 from homeassistant.const import (
@@ -61,6 +62,16 @@ from .const import (
     CONF_FAN_MODE_REGISTER,
     CONF_FAN_MODE_TOP,
     CONF_FAN_MODE_VALUES,
+    CONF_HVAC_ACTION_COOLING,
+    CONF_HVAC_ACTION_DEFROSTING,
+    CONF_HVAC_ACTION_DRYING,
+    CONF_HVAC_ACTION_FAN,
+    CONF_HVAC_ACTION_HEATING,
+    CONF_HVAC_ACTION_IDLE,
+    CONF_HVAC_ACTION_OFF,
+    CONF_HVAC_ACTION_PREHEATING,
+    CONF_HVAC_ACTION_REGISTER,
+    CONF_HVAC_ACTION_VALUES,
     CONF_HVAC_MODE_AUTO,
     CONF_HVAC_MODE_COOL,
     CONF_HVAC_MODE_DRY,
@@ -74,6 +85,7 @@ from .const import (
     CONF_HVAC_ON_VALUE,
     CONF_HVAC_ONOFF_COIL,
     CONF_HVAC_ONOFF_REGISTER,
+    CONF_INPUT_TYPE,
     CONF_MAX_TEMP,
     CONF_MIN_TEMP,
     CONF_STEP,
@@ -188,6 +200,34 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
             self._attr_hvac_mode = HVACMode.AUTO
             self._attr_hvac_modes = [HVACMode.AUTO]
 
+        if CONF_HVAC_ACTION_REGISTER in config:
+            action_config = config[CONF_HVAC_ACTION_REGISTER]
+            self._hvac_action_register = action_config[CONF_ADDRESS]
+            self._hvac_action_type = action_config[CONF_INPUT_TYPE]
+
+            self._attr_hvac_action = None
+            self._hvac_action_mapping: list[tuple[int, HVACAction]] = []
+            action_value_config = action_config[CONF_HVAC_ACTION_VALUES]
+
+            for hvac_action_kw, hvac_action in (
+                (CONF_HVAC_ACTION_COOLING, HVACAction.COOLING),
+                (CONF_HVAC_ACTION_DEFROSTING, HVACAction.DEFROSTING),
+                (CONF_HVAC_ACTION_DRYING, HVACAction.DRYING),
+                (CONF_HVAC_ACTION_FAN, HVACAction.FAN),
+                (CONF_HVAC_ACTION_HEATING, HVACAction.HEATING),
+                (CONF_HVAC_ACTION_IDLE, HVACAction.IDLE),
+                (CONF_HVAC_ACTION_OFF, HVACAction.OFF),
+                (CONF_HVAC_ACTION_PREHEATING, HVACAction.PREHEATING),
+            ):
+                if hvac_action_kw in action_value_config:
+                    values = action_value_config[hvac_action_kw]
+                    if not isinstance(values, list):
+                        values = [values]
+                    for value in values:
+                        self._hvac_action_mapping.append((value, hvac_action))
+        else:
+            self._hvac_action_register = None
+
         if CONF_FAN_MODE_REGISTER in config:
             self._attr_supported_features = (
                 self._attr_supported_features | ClimateEntityFeature.FAN_MODE
@@ -216,7 +256,6 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
                     self._fan_mode_mapping_from_modbus[value] = fan_mode
                     self._fan_mode_mapping_to_modbus[fan_mode] = value
                     self._attr_fan_modes.append(fan_mode)
-
         else:
             # No FAN modes defined
             self._fan_mode_register = None
@@ -457,6 +496,20 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
                         self._attr_hvac_mode = mode
                         break
 
+        # Read the HVAC action register if defined
+        if self._hvac_action_register is not None:
+            hvac_action = await self._async_read_register(
+                self._hvac_action_type, self._hvac_action_register, raw=True
+            )
+
+            # Translate the value received
+            if hvac_action is not None:
+                self._attr_hvac_action = None
+                for value, action in self._hvac_action_mapping:
+                    if hvac_action == value:
+                        self._attr_hvac_action = action
+                        break
+
         # Read the Fan mode register if defined
         if self._fan_mode_register is not None:
             fan_mode = await self._async_read_register(
