@@ -1,4 +1,5 @@
 """Constants used in modbus integration."""
+
 from enum import Enum
 
 from homeassistant.const import (
@@ -15,25 +16,19 @@ from homeassistant.const import (
 CONF_BAUDRATE = "baudrate"
 CONF_BYTESIZE = "bytesize"
 CONF_CLIMATES = "climates"
-CONF_CLOSE_COMM_ON_ERROR = "close_comm_on_error"
-CONF_COILS = "coils"
-CONF_CURRENT_TEMP = "current_temp_register"
-CONF_CURRENT_TEMP_REGISTER_TYPE = "current_temp_register_type"
+CONF_BRIGHTNESS_REGISTER = "brightness_address"
+CONF_COLOR_TEMP_REGISTER = "color_temp_address"
 CONF_DATA_TYPE = "data_type"
+CONF_DEVICE_ADDRESS = "device_address"
 CONF_FANS = "fans"
-CONF_HUB = "hub"
-CONF_INPUTS = "inputs"
 CONF_INPUT_TYPE = "input_type"
-CONF_LAZY_ERROR = "lazy_error_count"
 CONF_MAX_TEMP = "max_temp"
+CONF_MAX_VALUE = "max_value"
 CONF_MIN_TEMP = "min_temp"
+CONF_MIN_VALUE = "min_value"
 CONF_MSG_WAIT = "message_wait_milliseconds"
+CONF_NAN_VALUE = "nan_value"
 CONF_PARITY = "parity"
-CONF_REGISTER = "register"
-CONF_REGISTER_TYPE = "register_type"
-CONF_REGISTERS = "registers"
-CONF_RETRIES = "retries"
-CONF_RETRY_ON_EMPTY = "retry_on_empty"
 CONF_PRECISION = "precision"
 CONF_SCALE = "scale"
 CONF_SLAVE_COUNT = "slave_count"
@@ -49,13 +44,37 @@ CONF_STEP = "temp_step"
 CONF_STOPBITS = "stopbits"
 CONF_SWAP = "swap"
 CONF_SWAP_BYTE = "byte"
-CONF_SWAP_NONE = "none"
 CONF_SWAP_WORD = "word"
 CONF_SWAP_WORD_BYTE = "word_byte"
 CONF_TARGET_TEMP = "target_temp_register"
+CONF_TARGET_TEMP_WRITE_REGISTERS = "target_temp_write_registers"
+CONF_FAN_MODE_REGISTER = "fan_mode_register"
+CONF_FAN_MODE_ON = "state_fan_on"
+CONF_FAN_MODE_OFF = "state_fan_off"
+CONF_FAN_MODE_LOW = "state_fan_low"
+CONF_FAN_MODE_MEDIUM = "state_fan_medium"
+CONF_FAN_MODE_HIGH = "state_fan_high"
+CONF_FAN_MODE_AUTO = "state_fan_auto"
+CONF_FAN_MODE_TOP = "state_fan_top"
+CONF_FAN_MODE_MIDDLE = "state_fan_middle"
+CONF_FAN_MODE_FOCUS = "state_fan_focus"
+CONF_FAN_MODE_DIFFUSE = "state_fan_diffuse"
+CONF_FAN_MODE_VALUES = "values"
 CONF_HVAC_MODE_REGISTER = "hvac_mode_register"
-CONF_HVAC_MODE_VALUES = "values"
 CONF_HVAC_ONOFF_REGISTER = "hvac_onoff_register"
+CONF_HVAC_ON_VALUE = "hvac_on_value"
+CONF_HVAC_OFF_VALUE = "hvac_off_value"
+CONF_HVAC_ONOFF_COIL = "hvac_onoff_coil"
+CONF_HVAC_ACTION_REGISTER = "hvac_action_register"
+CONF_HVAC_ACTION_COOLING = "action_cooling"
+CONF_HVAC_ACTION_DEFROSTING = "action_defrosting"
+CONF_HVAC_ACTION_DRYING = "action_drying"
+CONF_HVAC_ACTION_FAN = "action_fan"
+CONF_HVAC_ACTION_HEATING = "action_heating"
+CONF_HVAC_ACTION_IDLE = "action_idle"
+CONF_HVAC_ACTION_OFF = "action_off"
+CONF_HVAC_ACTION_PREHEATING = "action_preheating"
+CONF_HVAC_ACTION_VALUES = "values"
 CONF_HVAC_MODE_OFF = "state_off"
 CONF_HVAC_MODE_HEAT = "state_heat"
 CONF_HVAC_MODE_COOL = "state_cool"
@@ -63,10 +82,19 @@ CONF_HVAC_MODE_HEAT_COOL = "state_heat_cool"
 CONF_HVAC_MODE_AUTO = "state_auto"
 CONF_HVAC_MODE_DRY = "state_dry"
 CONF_HVAC_MODE_FAN_ONLY = "state_fan_only"
+CONF_HVAC_MODE_VALUES = "values"
+CONF_SWING_MODE_REGISTER = "swing_mode_register"
+CONF_SWING_MODE_SWING_BOTH = "swing_mode_state_both"
+CONF_SWING_MODE_SWING_HORIZ = "swing_mode_state_horizontal"
+CONF_SWING_MODE_SWING_OFF = "swing_mode_state_off"
+CONF_SWING_MODE_SWING_ON = "swing_mode_state_on"
+CONF_SWING_MODE_SWING_VERT = "swing_mode_state_vertical"
+CONF_SWING_MODE_VALUES = "values"
+CONF_WRITE_REGISTERS = "write_registers"
 CONF_VERIFY = "verify"
-CONF_VERIFY_REGISTER = "verify_register"
-CONF_VERIFY_STATE = "verify_state"
+CONF_VIRTUAL_COUNT = "virtual_count"
 CONF_WRITE_TYPE = "write_type"
+CONF_ZERO_SUPPRESS = "zero_suppress"
 
 RTUOVERTCP = "rtuovertcp"
 SERIAL = "serial"
@@ -76,7 +104,7 @@ UDP = "udp"
 
 # service call attributes
 ATTR_ADDRESS = CONF_ADDRESS
-ATTR_HUB = CONF_HUB
+ATTR_HUB = "hub"
 ATTR_UNIT = "unit"
 ATTR_SLAVE = "slave"
 ATTR_VALUE = "value"
@@ -87,11 +115,9 @@ class DataType(str, Enum):
 
     CUSTOM = "custom"
     STRING = "string"
-    INT8 = "int8"
     INT16 = "int16"
     INT32 = "int32"
     INT64 = "int64"
-    UINT8 = "uint8"
     UINT16 = "uint16"
     UINT32 = "uint32"
     UINT64 = "uint64"
@@ -128,6 +154,8 @@ DEFAULT_SCAN_INTERVAL = 15  # seconds
 DEFAULT_SLAVE = 1
 DEFAULT_STRUCTURE_PREFIX = ">f"
 DEFAULT_TEMP_UNIT = "C"
+DEFAULT_HVAC_ON_VALUE = 1
+DEFAULT_HVAC_OFF_VALUE = 0
 MODBUS_DOMAIN = "modbus"
 
 ACTIVE_SCAN_INTERVAL = 2  # limit to force an extra update
@@ -141,3 +169,11 @@ PLATFORMS = (
     (Platform.SENSOR, CONF_SENSORS),
     (Platform.SWITCH, CONF_SWITCHES),
 )
+
+LIGHT_DEFAULT_MIN_KELVIN = 2000
+LIGHT_DEFAULT_MAX_KELVIN = 7000
+LIGHT_MIN_BRIGHTNESS = 0
+LIGHT_MAX_BRIGHTNESS = 255
+LIGHT_MODBUS_SCALE_MIN = 0
+LIGHT_MODBUS_SCALE_MAX = 100
+LIGHT_MODBUS_INVALID_VALUE = 0xFFFF
