@@ -55,13 +55,28 @@ from homeassistant.util.json import json_loads_object
 from .. import subscription
 from ..config import DEFAULT_QOS, DEFAULT_RETAIN, MQTT_RW_SCHEMA
 from ..const import (
+    CONF_COLOR_MODE,
     CONF_COLOR_TEMP_KELVIN,
     CONF_COMMAND_TOPIC,
+    CONF_EFFECT_LIST,
+    CONF_FLASH,
+    CONF_FLASH_TIME_LONG,
+    CONF_FLASH_TIME_SHORT,
     CONF_MAX_KELVIN,
+    CONF_MAX_MIREDS,
     CONF_MIN_KELVIN,
+    CONF_MIN_MIREDS,
     CONF_QOS,
     CONF_RETAIN,
     CONF_STATE_TOPIC,
+    CONF_SUPPORTED_COLOR_MODES,
+    CONF_TRANSITION,
+    DEFAULT_BRIGHTNESS,
+    DEFAULT_BRIGHTNESS_SCALE,
+    DEFAULT_EFFECT,
+    DEFAULT_FLASH_TIME_LONG,
+    DEFAULT_FLASH_TIME_SHORT,
+    DEFAULT_WHITE_SCALE,
 )
 from ..entity import MqttEntity
 from ..models import ReceiveMessage
@@ -78,25 +93,10 @@ _LOGGER = logging.getLogger(__name__)
 
 DOMAIN = "mqtt_json"
 
-DEFAULT_BRIGHTNESS = False
-DEFAULT_EFFECT = False
-DEFAULT_FLASH_TIME_LONG = 10
-DEFAULT_FLASH_TIME_SHORT = 2
 DEFAULT_NAME = "MQTT JSON Light"
-DEFAULT_BRIGHTNESS_SCALE = 255
-DEFAULT_WHITE_SCALE = 255
-
-CONF_COLOR_MODE = "color_mode"
-CONF_SUPPORTED_COLOR_MODES = "supported_color_modes"
-
-CONF_EFFECT_LIST = "effect_list"
-
-CONF_FLASH_TIME_LONG = "flash_time_long"
-CONF_FLASH_TIME_SHORT = "flash_time_short"
-
-CONF_MAX_MIREDS = "max_mireds"
-CONF_MIN_MIREDS = "min_mireds"
 
+DEFAULT_FLASH = True
+DEFAULT_TRANSITION = True
 
 _PLATFORM_SCHEMA_BASE = (
     MQTT_RW_SCHEMA.extend(
@@ -108,6 +108,7 @@ _PLATFORM_SCHEMA_BASE = (
             vol.Optional(CONF_COLOR_TEMP_KELVIN, default=False): cv.boolean,
             vol.Optional(CONF_EFFECT, default=DEFAULT_EFFECT): cv.boolean,
             vol.Optional(CONF_EFFECT_LIST): vol.All(cv.ensure_list, [cv.string]),
+            vol.Optional(CONF_FLASH, default=DEFAULT_FLASH): cv.boolean,
             vol.Optional(
                 CONF_FLASH_TIME_LONG, default=DEFAULT_FLASH_TIME_LONG
             ): cv.positive_int,
@@ -130,6 +131,7 @@ _PLATFORM_SCHEMA_BASE = (
                 vol.Unique(),
                 valid_supported_color_modes,
             ),
+            vol.Optional(CONF_TRANSITION, default=DEFAULT_TRANSITION): cv.boolean,
             vol.Optional(CONF_WHITE_SCALE, default=DEFAULT_WHITE_SCALE): vol.All(
                 vol.Coerce(int), vol.Range(min=1)
             ),
@@ -204,12 +206,13 @@ class MqttLightJson(MqttEntity, LightEntity, RestoreEntity):
             for key in (CONF_FLASH_TIME_SHORT, CONF_FLASH_TIME_LONG)
         }
 
-        self._attr_supported_features = (
-            LightEntityFeature.TRANSITION | LightEntityFeature.FLASH
-        )
         self._attr_supported_features |= (
             config[CONF_EFFECT] and LightEntityFeature.EFFECT
         )
+        self._attr_supported_features |= config[CONF_FLASH] and LightEntityFeature.FLASH
+        self._attr_supported_features |= (
+            config[CONF_TRANSITION] and LightEntityFeature.TRANSITION
+        )
         if supported_color_modes := self._config.get(CONF_SUPPORTED_COLOR_MODES):
             self._attr_supported_color_modes = supported_color_modes
             if self.supported_color_modes and len(self.supported_color_modes) == 1:
