@@ -1,24 +1,29 @@
 """Models used by multiple MQTT modules."""
+
 from __future__ import annotations
 
 from ast import literal_eval
 import asyncio
 from collections import deque
-from collections.abc import Callable, Coroutine
+from collections.abc import Callable
 from dataclasses import dataclass, field
-import datetime as dt
 from enum import StrEnum
 import logging
 from typing import TYPE_CHECKING, Any, TypedDict
 
-import voluptuous as vol
-
-from homeassistant.const import ATTR_ENTITY_ID, ATTR_NAME
-from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
+from homeassistant.const import ATTR_ENTITY_ID, ATTR_NAME, Platform
+from homeassistant.core import CALLBACK_TYPE, callback
+from homeassistant.exceptions import ServiceValidationError, TemplateError
 from homeassistant.helpers import template
 from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.service_info.mqtt import ReceivePayloadType
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType, TemplateVarsType
+from homeassistant.helpers.typing import (
+    ConfigType,
+    DiscoveryInfoType,
+    TemplateVarsType,
+    VolSchemaType,
+)
+from homeassistant.util.hass_dict import HassKey
 
 if TYPE_CHECKING:
     from paho.mqtt.client import MQTTMessage
@@ -29,6 +34,8 @@ if TYPE_CHECKING:
     from .discovery import MQTTDiscoveryPayload
     from .tag import MQTTTagScanner
 
+from .const import DOMAIN, TEMPLATE_ERRORS
+
 
 class PayloadSentinel(StrEnum):
     """Sentinel for `async_render_with_possible_json_value`."""
@@ -41,7 +48,23 @@ _LOGGER = logging.getLogger(__name__)
 
 ATTR_THIS = "this"
 
-PublishPayloadType = str | bytes | int | float | None
+type PublishPayloadType = str | bytes | int | float | None
+
+
+def convert_outgoing_mqtt_payload(
+    payload: PublishPayloadType,
+) -> PublishPayloadType:
+    """Ensure correct raw MQTT payload is passed as bytes for publishing."""
+    if isinstance(payload, str) and payload.startswith(("b'", 'b"')):
+        try:
+            native_object = literal_eval(payload)
+        except (ValueError, TypeError, SyntaxError, MemoryError):
+            pass
+        else:
+            if isinstance(native_object, bytes):
+                return native_object
+
+    return payload
 
 
 @dataclass
@@ -54,7 +77,10 @@ class PublishMessage:
     retain: bool
 
 
-@dataclass
+# eq=False so we use the id() of the object for comparison
+# since client will only generate one instance of this object
+# per messages/subscribed_topic.
+@dataclass(slots=True, frozen=True, eq=False)
 class ReceiveMessage:
     """MQTT Message received."""
 
@@ -63,11 +89,10 @@ class ReceiveMessage:
     qos: int
     retain: bool
     subscribed_topic: str
-    timestamp: dt.datetime
+    timestamp: float
 
 
-AsyncMessageCallbackType = Callable[[ReceiveMessage], Coroutine[Any, Any, None]]
-MessageCallbackType = Callable[[ReceiveMessage], None]
+type MessageCallbackType = Callable[[ReceiveMessage], None]
 
 
 class SubscriptionDebugInfo(TypedDict):
@@ -109,6 +134,40 @@ class MqttOriginInfo(TypedDict, total=False):
     support_url: str
 
 
+class MqttCommandTemplateException(ServiceValidationError):
+    """Handle MqttCommandTemplate exceptions."""
+
+    _message: str
+
+    def __init__(
+        self,
+        *args: object,
+        base_exception: Exception,
+        command_template: str,
+        value: PublishPayloadType,
+        entity_id: str | None = None,
+    ) -> None:
+        """Initialize exception."""
+        super().__init__(base_exception, *args)
+        value_log = str(value)
+        self.translation_domain = DOMAIN
+        self.translation_key = "command_template_error"
+        self.translation_placeholders = {
+            "error": str(base_exception),
+            "entity_id": str(entity_id),
+            "command_template": command_template,
+        }
+        entity_id_log = "" if entity_id is None else f" for entity '{entity_id}'"
+        self._message = (
+            f"{type(base_exception).__name__}: {base_exception} rendering template{entity_id_log}"
+            f", template: '{command_template}' and payload: {value_log}"
+        )
+
+    def __str__(self) -> str:
+        """Return exception message string."""
+        return self._message
+
+
 class MqttCommandTemplate:
     """Class for rendering MQTT payload with command templates."""
 
@@ -116,22 +175,13 @@ class MqttCommandTemplate:
         self,
         command_template: template.Template | None,
         *,
-        hass: HomeAssistant | None = None,
         entity: Entity | None = None,
     ) -> None:
         """Instantiate a command template."""
         self._template_state: template.TemplateStateFromEntityId | None = None
         self._command_template = command_template
-        if command_template is None:
-            return
-
         self._entity = entity
 
-        command_template.hass = hass
-
-        if entity:
-            command_template.hass = entity.hass
-
     @callback
     def async_render(
         self,
@@ -139,22 +189,6 @@ class MqttCommandTemplate:
         variables: TemplateVarsType = None,
     ) -> PublishPayloadType:
         """Render or convert the command template with given value or variables."""
-
-        def _convert_outgoing_payload(
-            payload: PublishPayloadType,
-        ) -> PublishPayloadType:
-            """Ensure correct raw MQTT payload is passed as bytes for publishing."""
-            if isinstance(payload, str):
-                try:
-                    native_object = literal_eval(payload)
-                    if isinstance(native_object, bytes):
-                        return native_object
-
-                except (ValueError, TypeError, SyntaxError, MemoryError):
-                    pass
-
-            return payload
-
         if self._command_template is None:
             return value
 
@@ -175,9 +209,49 @@ class MqttCommandTemplate:
             values,
             self._command_template,
         )
-        return _convert_outgoing_payload(
-            self._command_template.async_render(values, parse_result=False)
+        try:
+            return convert_outgoing_mqtt_payload(
+                self._command_template.async_render(values, parse_result=False)
+            )
+        except TemplateError as exc:
+            raise MqttCommandTemplateException(
+                base_exception=exc,
+                command_template=self._command_template.template,
+                value=value,
+                entity_id=self._entity.entity_id if self._entity is not None else None,
+            ) from exc
+
+
+class MqttValueTemplateException(TemplateError):
+    """Handle MqttValueTemplate exceptions."""
+
+    _message: str
+
+    def __init__(
+        self,
+        *args: object,
+        base_exception: Exception,
+        value_template: str,
+        default: ReceivePayloadType | PayloadSentinel,
+        payload: ReceivePayloadType,
+        entity_id: str | None = None,
+    ) -> None:
+        """Initialize exception."""
+        super().__init__(base_exception, *args)
+        entity_id_log = "" if entity_id is None else f" for entity '{entity_id}'"
+        default_log = str(default)
+        default_payload_log = (
+            "" if default is PayloadSentinel.NONE else f", default value: {default_log}"
         )
+        payload_log = str(payload)
+        self._message = (
+            f"{type(base_exception).__name__}: {base_exception} rendering template{entity_id_log}"
+            f", template: '{value_template}'{default_payload_log} and payload: {payload_log}"
+        )
+
+    def __str__(self) -> str:
+        """Return exception message string."""
+        return self._message
 
 
 class MqttValueTemplate:
@@ -187,7 +261,6 @@ class MqttValueTemplate:
         self,
         value_template: template.Template | None,
         *,
-        hass: HomeAssistant | None = None,
         entity: Entity | None = None,
         config_attributes: TemplateVarsType = None,
     ) -> None:
@@ -195,15 +268,8 @@ class MqttValueTemplate:
         self._template_state: template.TemplateStateFromEntityId | None = None
         self._value_template = value_template
         self._config_attributes = config_attributes
-        if value_template is None:
-            return
-
-        value_template.hass = hass
         self._entity = entity
 
-        if entity:
-            value_template.hass = entity.hass
-
     @callback
     def async_render_with_possible_json_value(
         self,
@@ -247,15 +313,14 @@ class MqttValueTemplate:
                         payload, variables=values
                     )
                 )
-            except Exception as exc:
-                _LOGGER.error(
-                    "%s: %s rendering template for entity '%s', template: '%s'",
-                    type(exc).__name__,
-                    exc,
-                    self._entity.entity_id if self._entity else "n/a",
-                    self._value_template.template,
-                )
-                raise exc
+            except TEMPLATE_ERRORS as exc:
+                raise MqttValueTemplateException(
+                    base_exception=exc,
+                    value_template=self._value_template.template,
+                    default=default,
+                    payload=payload,
+                    entity_id=self._entity.entity_id if self._entity else None,
+                ) from exc
             return rendered_payload
 
         _LOGGER.debug(
@@ -274,18 +339,14 @@ class MqttValueTemplate:
                     payload, default, variables=values
                 )
             )
-        except Exception as ex:
-            _LOGGER.error(
-                "%s: %s rendering template for entity '%s', template: "
-                "'%s', default value: %s and payload: %s",
-                type(ex).__name__,
-                ex,
-                self._entity.entity_id if self._entity else "n/a",
-                self._value_template.template,
-                default,
-                payload,
-            )
-            raise ex
+        except TEMPLATE_ERRORS as exc:
+            raise MqttValueTemplateException(
+                base_exception=exc,
+                value_template=self._value_template.template,
+                default=default,
+                payload=payload,
+                entity_id=self._entity.entity_id if self._entity else None,
+            ) from exc
         return rendered_payload
 
 
@@ -300,14 +361,14 @@ class EntityTopicState:
     def process_write_state_requests(self, msg: MQTTMessage) -> None:
         """Process the write state requests."""
         while self.subscribe_calls:
-            _, entity = self.subscribe_calls.popitem()
+            entity_id, entity = self.subscribe_calls.popitem()
             try:
                 entity.async_write_ha_state()
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 _LOGGER.exception(
-                    "Exception raised when updating state of %s, topic: "
+                    "Exception raised while updating state of %s, topic: "
                     "'%s' with payload: %s",
-                    entity.entity_id,
+                    entity_id,
                     msg.topic,
                     msg.payload,
                 )
@@ -340,9 +401,60 @@ class MqttData:
     discovery_unsubscribe: list[CALLBACK_TYPE] = field(default_factory=list)
     integration_unsubscribe: dict[str, CALLBACK_TYPE] = field(default_factory=dict)
     last_discovery: float = 0.0
+    platforms_loaded: set[Platform | str] = field(default_factory=set)
     reload_dispatchers: list[CALLBACK_TYPE] = field(default_factory=list)
     reload_handlers: dict[str, CALLBACK_TYPE] = field(default_factory=dict)
-    reload_schema: dict[str, vol.Schema] = field(default_factory=dict)
+    reload_schema: dict[str, VolSchemaType] = field(default_factory=dict)
     state_write_requests: EntityTopicState = field(default_factory=EntityTopicState)
-    subscriptions_to_restore: list[Subscription] = field(default_factory=list)
+    subscriptions_to_restore: set[Subscription] = field(default_factory=set)
     tags: dict[str, dict[str, MQTTTagScanner]] = field(default_factory=dict)
+
+
+@dataclass(slots=True)
+class MqttComponentConfig:
+    """(component, object_id, node_id, discovery_payload)."""
+
+    component: str
+    object_id: str
+    node_id: str | None
+    discovery_payload: MQTTDiscoveryPayload
+
+
+class DeviceMqttOptions(TypedDict, total=False):
+    """Hold the shared MQTT specific options for an MQTT device."""
+
+    qos: int
+
+
+class MqttDeviceData(TypedDict, total=False):
+    """Hold the data for an MQTT device."""
+
+    name: str
+    identifiers: str
+    configuration_url: str
+    sw_version: str
+    hw_version: str
+    model: str
+    model_id: str
+    mqtt_settings: DeviceMqttOptions
+
+
+class MqttAvailabilityData(TypedDict, total=False):
+    """Hold the availability configuration for a device."""
+
+    availability_topic: str
+    availability_template: str
+    payload_available: str
+    payload_not_available: str
+
+
+class MqttSubentryData(TypedDict, total=False):
+    """Hold the data for a MQTT subentry."""
+
+    device: MqttDeviceData
+    components: dict[str, dict[str, Any]]
+    availability: MqttAvailabilityData
+
+
+DATA_MQTT: HassKey[MqttData] = HassKey("mqtt")
+DATA_MQTT_AVAILABLE: HassKey[asyncio.Future[bool]] = HassKey("mqtt_client_available")
