@@ -1,4 +1,6 @@
 """Support for departure information for public transport in Munich."""
+
+# mypy: ignore-errors
 from __future__ import annotations
 
 from copy import deepcopy
@@ -8,10 +10,13 @@ import logging
 import MVGLive
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA, SensorEntity
-from homeassistant.const import ATTR_ATTRIBUTION, CONF_NAME, TIME_MINUTES
+from homeassistant.components.sensor import (
+    PLATFORM_SCHEMA as SENSOR_PLATFORM_SCHEMA,
+    SensorEntity,
+)
+from homeassistant.const import CONF_NAME, UnitOfTime
 from homeassistant.core import HomeAssistant
-import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
@@ -43,7 +48,7 @@ ATTRIBUTION = "Data provided by MVG-live.de"
 
 SCAN_INTERVAL = timedelta(seconds=30)
 
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
+PLATFORM_SCHEMA = SENSOR_PLATFORM_SCHEMA.extend(
     {
         vol.Required(CONF_NEXT_DEPARTURE): [
             {
@@ -70,9 +75,8 @@ def setup_platform(
     discovery_info: DiscoveryInfoType | None = None,
 ) -> None:
     """Set up the MVGLive sensor."""
-    sensors = []
-    for nextdeparture in config[CONF_NEXT_DEPARTURE]:
-        sensors.append(
+    add_entities(
+        (
             MVGLiveSensor(
                 nextdeparture.get(CONF_STATION),
                 nextdeparture.get(CONF_DESTINATIONS),
@@ -83,13 +87,17 @@ def setup_platform(
                 nextdeparture.get(CONF_NUMBER),
                 nextdeparture.get(CONF_NAME),
             )
-        )
-    add_entities(sensors, True)
+            for nextdeparture in config[CONF_NEXT_DEPARTURE]
+        ),
+        True,
+    )
 
 
 class MVGLiveSensor(SensorEntity):
     """Implementation of an MVG Live sensor."""
 
+    _attr_attribution = ATTRIBUTION
+
     def __init__(
         self,
         station,
@@ -139,7 +147,7 @@ class MVGLiveSensor(SensorEntity):
     @property
     def native_unit_of_measurement(self):
         """Return the unit this state is expressed in."""
-        return TIME_MINUTES
+        return UnitOfTime.MINUTES
 
     def update(self) -> None:
         """Get the latest data and update the state."""
@@ -210,7 +218,7 @@ class MVGLiveData:
                 continue
 
             # now select the relevant data
-            _nextdep = {ATTR_ATTRIBUTION: ATTRIBUTION}
+            _nextdep = {}
             for k in ("destination", "linename", "time", "direction", "product"):
                 _nextdep[k] = _departure.get(k, "")
             _nextdep["time"] = int(_nextdep["time"])
