@@ -1,20 +1,22 @@
 """Support for Notion binary sensors."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
 from typing import Literal
 
+from aionotion.listener.models import ListenerKind
+
 from homeassistant.components.binary_sensor import (
     BinarySensorDeviceClass,
     BinarySensorEntity,
     BinarySensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity import EntityCategory
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.const import EntityCategory
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from . import NotionEntity
 from .const import (
     DOMAIN,
     LOGGER,
@@ -26,111 +28,105 @@ from .const import (
     SENSOR_SAFE,
     SENSOR_SLIDING,
     SENSOR_SMOKE_CO,
-    SENSOR_WINDOW_HINGED_HORIZONTAL,
-    SENSOR_WINDOW_HINGED_VERTICAL,
+    SENSOR_WINDOW_HINGED,
 )
+from .coordinator import NotionDataUpdateCoordinator
+from .entity import NotionEntity, NotionEntityDescription
 
 
-@dataclass
-class NotionBinarySensorDescriptionMixin:
-    """Define an entity description mixin for binary and regular sensors."""
-
-    on_state: Literal["alarm", "critical", "leak", "not_missing", "open"]
-
-
-@dataclass
+@dataclass(frozen=True, kw_only=True)
 class NotionBinarySensorDescription(
-    BinarySensorEntityDescription, NotionBinarySensorDescriptionMixin
+    BinarySensorEntityDescription, NotionEntityDescription
 ):
     """Describe a Notion binary sensor."""
 
+    on_state: Literal["alarm", "leak", "low", "not_missing", "open"]
+
 
 BINARY_SENSOR_DESCRIPTIONS = (
     NotionBinarySensorDescription(
         key=SENSOR_BATTERY,
-        name="Low battery",
         device_class=BinarySensorDeviceClass.BATTERY,
         entity_category=EntityCategory.DIAGNOSTIC,
-        on_state="critical",
+        listener_kind=ListenerKind.BATTERY,
+        on_state="low",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_DOOR,
-        name="Door",
         device_class=BinarySensorDeviceClass.DOOR,
+        listener_kind=ListenerKind.DOOR,
         on_state="open",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_GARAGE_DOOR,
-        name="Garage door",
         device_class=BinarySensorDeviceClass.GARAGE_DOOR,
+        listener_kind=ListenerKind.GARAGE_DOOR,
         on_state="open",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_LEAK,
-        name="Leak detector",
         device_class=BinarySensorDeviceClass.MOISTURE,
+        listener_kind=ListenerKind.LEAK_STATUS,
         on_state="leak",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_MISSING,
-        name="Missing",
         device_class=BinarySensorDeviceClass.CONNECTIVITY,
         entity_category=EntityCategory.DIAGNOSTIC,
+        listener_kind=ListenerKind.CONNECTED,
         on_state="not_missing",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_SAFE,
-        name="Safe",
+        translation_key="safe",
         device_class=BinarySensorDeviceClass.DOOR,
+        listener_kind=ListenerKind.SAFE,
         on_state="open",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_SLIDING,
-        name="Sliding door/window",
+        translation_key="sliding_door_window",
         device_class=BinarySensorDeviceClass.DOOR,
+        listener_kind=ListenerKind.SLIDING_DOOR_OR_WINDOW,
         on_state="open",
     ),
     NotionBinarySensorDescription(
         key=SENSOR_SMOKE_CO,
-        name="Smoke/Carbon monoxide detector",
+        translation_key="smoke_carbon_monoxide_detector",
         device_class=BinarySensorDeviceClass.SMOKE,
+        listener_kind=ListenerKind.SMOKE,
         on_state="alarm",
     ),
     NotionBinarySensorDescription(
-        key=SENSOR_WINDOW_HINGED_HORIZONTAL,
-        name="Hinged window",
-        device_class=BinarySensorDeviceClass.WINDOW,
-        on_state="open",
-    ),
-    NotionBinarySensorDescription(
-        key=SENSOR_WINDOW_HINGED_VERTICAL,
-        name="Hinged window",
-        device_class=BinarySensorDeviceClass.WINDOW,
+        key=SENSOR_WINDOW_HINGED,
+        translation_key="hinged_window",
+        listener_kind=ListenerKind.HINGED_WINDOW,
         on_state="open",
     ),
 )
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: ConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up Notion sensors based on a config entry."""
-    coordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator: NotionDataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
 
     async_add_entities(
         [
             NotionBinarySensor(
                 coordinator,
-                task_id,
-                sensor["id"],
-                sensor["bridge"]["id"],
-                sensor["system_id"],
+                listener_id,
+                sensor.uuid,
+                sensor.bridge.id,
                 description,
             )
-            for task_id, task in coordinator.data["tasks"].items()
+            for listener_id, listener in coordinator.data.listeners.items()
             for description in BINARY_SENSOR_DESCRIPTIONS
-            if description.key == task["task_type"]
-            and (sensor := coordinator.data["sensors"][task["sensor_id"]])
+            if description.listener_kind.value == listener.definition_id
+            and (sensor := coordinator.data.sensors[listener.sensor_id])
         ]
     )
 
@@ -140,17 +136,10 @@ class NotionBinarySensor(NotionEntity, BinarySensorEntity):
 
     entity_description: NotionBinarySensorDescription
 
-    @callback
-    def _async_update_from_latest_data(self) -> None:
-        """Fetch new state data for the sensor."""
-        task = self.coordinator.data["tasks"][self._task_id]
-
-        if "value" in task["status"]:
-            state = task["status"]["value"]
-        elif task["status"].get("insights", {}).get("primary"):
-            state = task["status"]["insights"]["primary"]["to_state"]
-        else:
-            LOGGER.warning("Unknown data payload: %s", task["status"])
-            state = None
-
-        self._attr_is_on = self.entity_description.on_state == state
+    @property
+    def is_on(self) -> bool | None:
+        """Return true if the binary sensor is on."""
+        if not self.listener.insights.primary.value:
+            LOGGER.warning("Unknown listener structure: %s", self.listener)
+            return False
+        return self.listener.insights.primary.value == self.entity_description.on_state
