@@ -1,87 +1,61 @@
 """Support for getting statistical data from a Pi-hole system."""
+
 from __future__ import annotations
 
 from hole import Hole
 
 from homeassistant.components.sensor import SensorEntity, SensorEntityDescription
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_NAME, PERCENTAGE
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.helpers.typing import StateType
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
-from . import PiHoleEntity
-from .const import DATA_KEY_API, DATA_KEY_COORDINATOR, DOMAIN as PIHOLE_DOMAIN
+from . import PiHoleConfigEntry
+from .entity import PiHoleEntity
 
 SENSOR_TYPES: tuple[SensorEntityDescription, ...] = (
     SensorEntityDescription(
         key="ads_blocked_today",
         translation_key="ads_blocked_today",
-        native_unit_of_measurement="ads",
-        icon="mdi:close-octagon-outline",
     ),
     SensorEntityDescription(
         key="ads_percentage_today",
         translation_key="ads_percentage_today",
         native_unit_of_measurement=PERCENTAGE,
-        icon="mdi:close-octagon-outline",
     ),
     SensorEntityDescription(
         key="clients_ever_seen",
         translation_key="clients_ever_seen",
-        native_unit_of_measurement="clients",
-        icon="mdi:account-outline",
     ),
     SensorEntityDescription(
-        key="dns_queries_today",
-        translation_key="dns_queries_today",
-        native_unit_of_measurement="queries",
-        icon="mdi:comment-question-outline",
+        key="dns_queries_today", translation_key="dns_queries_today"
     ),
     SensorEntityDescription(
         key="domains_being_blocked",
         translation_key="domains_being_blocked",
-        native_unit_of_measurement="domains",
-        icon="mdi:block-helper",
     ),
+    SensorEntityDescription(key="queries_cached", translation_key="queries_cached"),
     SensorEntityDescription(
-        key="queries_cached",
-        translation_key="queries_cached",
-        native_unit_of_measurement="queries",
-        icon="mdi:comment-question-outline",
-    ),
-    SensorEntityDescription(
-        key="queries_forwarded",
-        translation_key="queries_forwarded",
-        native_unit_of_measurement="queries",
-        icon="mdi:comment-question-outline",
-    ),
-    SensorEntityDescription(
-        key="unique_clients",
-        translation_key="unique_clients",
-        native_unit_of_measurement="clients",
-        icon="mdi:account-outline",
-    ),
-    SensorEntityDescription(
-        key="unique_domains",
-        translation_key="unique_domains",
-        native_unit_of_measurement="domains",
-        icon="mdi:domain",
+        key="queries_forwarded", translation_key="queries_forwarded"
     ),
+    SensorEntityDescription(key="unique_clients", translation_key="unique_clients"),
+    SensorEntityDescription(key="unique_domains", translation_key="unique_domains"),
 )
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: PiHoleConfigEntry,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up the Pi-hole sensor."""
     name = entry.data[CONF_NAME]
-    hole_data = hass.data[PIHOLE_DOMAIN][entry.entry_id]
+    hole_data = entry.runtime_data
     sensors = [
         PiHoleSensor(
-            hole_data[DATA_KEY_API],
-            hole_data[DATA_KEY_COORDINATOR],
+            hole_data.api,
+            hole_data.coordinator,
             name,
             entry.entry_id,
             description,
@@ -100,7 +74,7 @@ class PiHoleSensor(PiHoleEntity, SensorEntity):
     def __init__(
         self,
         api: Hole,
-        coordinator: DataUpdateCoordinator,
+        coordinator: DataUpdateCoordinator[None],
         name: str,
         server_unique_id: str,
         description: SensorEntityDescription,
