@@ -1,5 +1,8 @@
 """The profiler integration."""
+
 import asyncio
+from collections.abc import Generator
+import contextlib
 from contextlib import suppress
 from datetime import timedelta
 from functools import _lru_cache_wrapper
@@ -19,7 +22,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_SCAN_INTERVAL, CONF_TYPE
 from homeassistant.core import HomeAssistant, ServiceCall, callback
 from homeassistant.exceptions import HomeAssistantError
-import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.event import async_track_time_interval
 from homeassistant.helpers.service import async_register_admin_service
 
@@ -35,6 +38,8 @@ SERVICE_DUMP_LOG_OBJECTS = "dump_log_objects"
 SERVICE_LRU_STATS = "lru_stats"
 SERVICE_LOG_THREAD_FRAMES = "log_thread_frames"
 SERVICE_LOG_EVENT_LOOP_SCHEDULED = "log_event_loop_scheduled"
+SERVICE_SET_ASYNCIO_DEBUG = "set_asyncio_debug"
+SERVICE_LOG_CURRENT_TASKS = "log_current_tasks"
 
 _LRU_CACHE_WRAPPER_OBJECT = _lru_cache_wrapper.__name__
 _SQLALCHEMY_LRU_OBJECT = "LRUCache"
@@ -45,7 +50,6 @@ _KNOWN_LRU_CLASSES = (
     "StatesMetaManager",
     "StateAttributesManager",
     "StatisticsMetaManager",
-    "IntegrationMatcher",
 )
 
 SERVICES = (
@@ -57,12 +61,15 @@ SERVICES = (
     SERVICE_LRU_STATS,
     SERVICE_LOG_THREAD_FRAMES,
     SERVICE_LOG_EVENT_LOOP_SCHEDULED,
+    SERVICE_SET_ASYNCIO_DEBUG,
+    SERVICE_LOG_CURRENT_TASKS,
 )
 
 DEFAULT_SCAN_INTERVAL = timedelta(seconds=30)
 
 DEFAULT_MAX_OBJECTS = 5
 
+CONF_ENABLED = "enabled"
 CONF_SECONDS = "seconds"
 CONF_MAX_OBJECTS = "max_objects"
 
@@ -159,7 +166,7 @@ async def async_setup_entry(  # noqa: C901
         # Imports deferred to avoid loading modules
         # in memory since usually only one part of this
         # integration is used at a time
-        import objgraph  # pylint: disable=import-outside-toplevel
+        import objgraph  # noqa: PLC0415
 
         obj_type = call.data[CONF_TYPE]
 
@@ -185,7 +192,7 @@ async def async_setup_entry(  # noqa: C901
         # Imports deferred to avoid loading modules
         # in memory since usually only one part of this
         # integration is used at a time
-        import objgraph  # pylint: disable=import-outside-toplevel
+        import objgraph  # noqa: PLC0415
 
         for lru in objgraph.by_type(_LRU_CACHE_WRAPPER_OBJECT):
             lru = cast(_lru_cache_wrapper, lru)
@@ -226,7 +233,7 @@ async def async_setup_entry(  # noqa: C901
 
     async def _async_dump_thread_frames(call: ServiceCall) -> None:
         """Log all thread frames."""
-        frames = sys._current_frames()  # pylint: disable=protected-access
+        frames = sys._current_frames()  # noqa: SLF001
         main_thread = threading.main_thread()
         for thread in threading.enumerate():
             if thread == main_thread:
@@ -238,21 +245,33 @@ async def async_setup_entry(  # noqa: C901
                 "".join(traceback.format_stack(frames.get(ident))).strip(),
             )
 
+    async def _async_dump_current_tasks(call: ServiceCall) -> None:
+        """Log all current tasks in the event loop."""
+        with _increase_repr_limit():
+            for task in asyncio.all_tasks():
+                if not task.cancelled():
+                    _LOGGER.critical("Task: %s", _safe_repr(task))
+
     async def _async_dump_scheduled(call: ServiceCall) -> None:
         """Log all scheduled in the event loop."""
-        arepr = reprlib.aRepr
-        original_maxstring = arepr.maxstring
-        original_maxother = arepr.maxother
-        arepr.maxstring = 300
-        arepr.maxother = 300
-        handle: asyncio.Handle
-        try:
-            for handle in getattr(hass.loop, "_scheduled"):
+        with _increase_repr_limit():
+            handle: asyncio.Handle
+            for handle in getattr(hass.loop, "_scheduled"):  # noqa: B009
                 if not handle.cancelled():
                     _LOGGER.critical("Scheduled: %s", handle)
-        finally:
-            arepr.maxstring = original_maxstring
-            arepr.maxother = original_maxother
+
+    async def _async_asyncio_debug(call: ServiceCall) -> None:
+        """Enable or disable asyncio debug."""
+        enabled = call.data[CONF_ENABLED]
+        # Always log this at critical level so we know when
+        # it's been changed when reviewing logs
+        _LOGGER.critical("Setting asyncio debug to %s", enabled)
+        # Make sure the logger is set to at least INFO or
+        # we won't see the messages
+        base_logger = logging.getLogger()
+        if enabled and base_logger.getEffectiveLevel() > logging.INFO:
+            base_logger.setLevel(logging.INFO)
+        hass.loop.set_debug(enabled)
 
     async_register_admin_service(
         hass,
@@ -348,6 +367,21 @@ async def async_setup_entry(  # noqa: C901
         _async_dump_scheduled,
     )
 
+    async_register_admin_service(
+        hass,
+        DOMAIN,
+        SERVICE_SET_ASYNCIO_DEBUG,
+        _async_asyncio_debug,
+        schema=vol.Schema({vol.Optional(CONF_ENABLED, default=True): cv.boolean}),
+    )
+
+    async_register_admin_service(
+        hass,
+        DOMAIN,
+        SERVICE_LOG_CURRENT_TASKS,
+        _async_dump_current_tasks,
+    )
+
     return True
 
 
@@ -365,7 +399,7 @@ async def _async_generate_profile(hass: HomeAssistant, call: ServiceCall):
     # Imports deferred to avoid loading modules
     # in memory since usually only one part of this
     # integration is used at a time
-    import cProfile  # pylint: disable=import-outside-toplevel
+    import cProfile  # noqa: PLC0415
 
     start_time = int(time.time() * 1000000)
     persistent_notification.async_create(
@@ -402,7 +436,7 @@ async def _async_generate_memory_profile(hass: HomeAssistant, call: ServiceCall)
     # Imports deferred to avoid loading modules
     # in memory since usually only one part of this
     # integration is used at a time
-    from guppy import hpy  # pylint: disable=import-outside-toplevel
+    from guppy import hpy  # noqa: PLC0415
 
     start_time = int(time.time() * 1000000)
     persistent_notification.async_create(
@@ -433,7 +467,7 @@ def _write_profile(profiler, cprofile_path, callgrind_path):
     # Imports deferred to avoid loading modules
     # in memory since usually only one part of this
     # integration is used at a time
-    from pyprof2calltree import convert  # pylint: disable=import-outside-toplevel
+    from pyprof2calltree import convert  # noqa: PLC0415
 
     profiler.create_stats()
     profiler.dump_stats(cprofile_path)
@@ -448,14 +482,14 @@ def _log_objects(*_):
     # Imports deferred to avoid loading modules
     # in memory since usually only one part of this
     # integration is used at a time
-    import objgraph  # pylint: disable=import-outside-toplevel
+    import objgraph  # noqa: PLC0415
 
     _LOGGER.critical("Memory Growth: %s", objgraph.growth(limit=1000))
 
 
 def _get_function_absfile(func: Any) -> str | None:
     """Get the absolute file path of a function."""
-    import inspect  # pylint: disable=import-outside-toplevel
+    import inspect  # noqa: PLC0415
 
     abs_file: str | None = None
     with suppress(Exception):
@@ -471,12 +505,12 @@ def _safe_repr(obj: Any) -> str:
     """
     try:
         return repr(obj)
-    except Exception:  # pylint: disable=broad-except
+    except Exception:  # noqa: BLE001
         return f"Failed to serialize {type(obj)}"
 
 
 def _find_backrefs_not_to_self(_object: Any) -> list[str]:
-    import objgraph  # pylint: disable=import-outside-toplevel
+    import objgraph  # noqa: PLC0415
 
     return [
         _safe_repr(backref)
@@ -492,7 +526,7 @@ def _log_object_sources(
     # Imports deferred to avoid loading modules
     # in memory since usually only one part of this
     # integration is used at a time
-    import gc  # pylint: disable=import-outside-toplevel
+    import gc  # noqa: PLC0415
 
     gc.collect()
 
@@ -549,3 +583,18 @@ def _log_object_sources(
         _LOGGER.critical("New objects overflowed by %s", new_objects_overflow)
     elif not had_new_object_growth:
         _LOGGER.critical("No new object growth found")
+
+
+@contextlib.contextmanager
+def _increase_repr_limit() -> Generator[None]:
+    """Increase the repr limit."""
+    arepr = reprlib.aRepr
+    original_maxstring = arepr.maxstring
+    original_maxother = arepr.maxother
+    arepr.maxstring = 300
+    arepr.maxother = 300
+    try:
+        yield
+    finally:
+        arepr.maxstring = original_maxstring
+        arepr.maxother = original_maxother
