@@ -1,4 +1,5 @@
 """Support for recording details."""
+
 from __future__ import annotations
 
 import abc
@@ -6,23 +7,28 @@ import asyncio
 from collections.abc import Callable, Iterable
 from dataclasses import dataclass
 from datetime import datetime
+import logging
 import threading
 from typing import TYPE_CHECKING, Any
 
-from homeassistant.core import Event
+from homeassistant.helpers.recorder import DATA_RECORDER
 from homeassistant.helpers.typing import UndefinedType
+from homeassistant.util.event_type import EventType
 
-from . import purge, statistics
-from .const import DOMAIN, EXCLUDE_ATTRIBUTES
+from . import entity_registry, purge, statistics
 from .db_schema import Statistics, StatisticsShortTerm
 from .models import StatisticData, StatisticMetaData
-from .util import periodic_db_cleanups
+from .util import periodic_db_cleanups, session_scope
+
+_LOGGER = logging.getLogger(__name__)
+
 
 if TYPE_CHECKING:
     from .core import Recorder
 
 
-class RecorderTask(abc.ABC):
+@dataclass(slots=True)
+class RecorderTask:
     """ABC for recorder tasks."""
 
     commit_before = True
@@ -32,7 +38,7 @@ class RecorderTask(abc.ABC):
         """Handle the task."""
 
 
-@dataclass
+@dataclass(slots=True)
 class ChangeStatisticsUnitTask(RecorderTask):
     """Object to store statistics_id and unit to convert unit of statistics."""
 
@@ -50,21 +56,25 @@ class ChangeStatisticsUnitTask(RecorderTask):
         )
 
 
-@dataclass
+@dataclass(slots=True)
 class ClearStatisticsTask(RecorderTask):
     """Object to store statistics_ids which for which to remove statistics."""
 
+    on_done: Callable[[], None] | None
     statistic_ids: list[str]
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
         statistics.clear_statistics(instance, self.statistic_ids)
+        if self.on_done:
+            self.on_done()
 
 
-@dataclass
+@dataclass(slots=True)
 class UpdateStatisticsMetadataTask(RecorderTask):
     """Object to store statistics_id and unit for update of statistics metadata."""
 
+    on_done: Callable[[], None] | None
     statistic_id: str
     new_statistic_id: str | None | UndefinedType
     new_unit_of_measurement: str | None | UndefinedType
@@ -77,9 +87,27 @@ class UpdateStatisticsMetadataTask(RecorderTask):
             self.new_statistic_id,
             self.new_unit_of_measurement,
         )
+        if self.on_done:
+            self.on_done()
 
 
-@dataclass
+@dataclass(slots=True)
+class UpdateStatesMetadataTask(RecorderTask):
+    """Task to update states metadata."""
+
+    entity_id: str
+    new_entity_id: str
+
+    def run(self, instance: Recorder) -> None:
+        """Handle the task."""
+        entity_registry.update_states_metadata(
+            instance,
+            self.entity_id,
+            self.new_entity_id,
+        )
+
+
+@dataclass(slots=True)
 class PurgeTask(RecorderTask):
     """Object to store information about purge task."""
 
@@ -92,8 +120,6 @@ class PurgeTask(RecorderTask):
         if purge.purge_old_data(
             instance, self.purge_before, self.repack, self.apply_filter
         ):
-            with instance.get_session() as session:
-                instance.run_history.load_from_db(session)
             # We always need to do the db cleanups after a purge
             # is finished to ensure the WAL checkpoint and other
             # tasks happen after a vacuum.
@@ -105,30 +131,34 @@ class PurgeTask(RecorderTask):
         )
 
 
-@dataclass
+@dataclass(slots=True)
 class PurgeEntitiesTask(RecorderTask):
     """Object to store entity information about purge task."""
 
     entity_filter: Callable[[str], bool]
+    purge_before: datetime
 
     def run(self, instance: Recorder) -> None:
         """Purge entities from the database."""
-        if purge.purge_entity_data(instance, self.entity_filter):
+        if purge.purge_entity_data(instance, self.entity_filter, self.purge_before):
             return
         # Schedule a new purge task if this one didn't finish
-        instance.queue_task(PurgeEntitiesTask(self.entity_filter))
+        instance.queue_task(PurgeEntitiesTask(self.entity_filter, self.purge_before))
 
 
-@dataclass
+@dataclass(slots=True)
 class PerodicCleanupTask(RecorderTask):
-    """An object to insert into the recorder to trigger cleanup tasks when auto purge is disabled."""
+    """An object to insert into the recorder to trigger cleanup tasks.
+
+    Trigger cleanup tasks when auto purge is disabled.
+    """
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
         periodic_db_cleanups(instance)
 
 
-@dataclass
+@dataclass(slots=True)
 class StatisticsTask(RecorderTask):
     """An object to insert into the recorder queue to run a statistics task."""
 
@@ -143,7 +173,19 @@ class StatisticsTask(RecorderTask):
         instance.queue_task(StatisticsTask(self.start, self.fire_events))
 
 
-@dataclass
+@dataclass(slots=True)
+class CompileMissingStatisticsTask(RecorderTask):
+    """An object to insert into the recorder queue to run a compile missing statistics."""
+
+    def run(self, instance: Recorder) -> None:
+        """Run statistics task to compile missing statistics."""
+        if statistics.compile_missing_statistics(instance):
+            return
+        # Schedule a new statistics task if this one didn't finish
+        instance.queue_task(CompileMissingStatisticsTask())
+
+
+@dataclass(slots=True)
 class ImportStatisticsTask(RecorderTask):
     """An object to insert into the recorder queue to run an import statistics task."""
 
@@ -163,7 +205,7 @@ class ImportStatisticsTask(RecorderTask):
         )
 
 
-@dataclass
+@dataclass(slots=True)
 class AdjustStatisticsTask(RecorderTask):
     """An object to insert into the recorder queue to run an adjust statistics task."""
 
@@ -193,18 +235,21 @@ class AdjustStatisticsTask(RecorderTask):
         )
 
 
-@dataclass
+@dataclass(slots=True)
 class WaitTask(RecorderTask):
-    """An object to insert into the recorder queue to tell it set the _queue_watch event."""
+    """An object to insert into the recorder queue.
+
+    Tell it set the _queue_watch event.
+    """
 
     commit_before = False
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
-        instance._queue_watch.set()  # pylint: disable=[protected-access]
+        instance._queue_watch.set()  # noqa: SLF001
 
 
-@dataclass
+@dataclass(slots=True)
 class DatabaseLockTask(RecorderTask):
     """An object to insert into the recorder queue to prevent writes to the database."""
 
@@ -214,10 +259,10 @@ class DatabaseLockTask(RecorderTask):
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
-        instance._lock_database(self)  # pylint: disable=[protected-access]
+        instance._lock_database(self)  # noqa: SLF001
 
 
-@dataclass
+@dataclass(slots=True)
 class StopTask(RecorderTask):
     """An object to insert into the recorder queue to stop the event handler."""
 
@@ -228,20 +273,7 @@ class StopTask(RecorderTask):
         instance.stop_requested = True
 
 
-@dataclass
-class EventTask(RecorderTask):
-    """An event to be processed."""
-
-    event: Event
-    commit_before = False
-
-    def run(self, instance: Recorder) -> None:
-        """Handle the task."""
-        # pylint: disable-next=[protected-access]
-        instance._process_one_event(self.event)
-
-
-@dataclass
+@dataclass(slots=True)
 class KeepAliveTask(RecorderTask):
     """A keep alive to be sent."""
 
@@ -249,11 +281,10 @@ class KeepAliveTask(RecorderTask):
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
-        # pylint: disable-next=[protected-access]
-        instance._send_keep_alive()
+        instance._send_keep_alive()  # noqa: SLF001
 
 
-@dataclass
+@dataclass(slots=True)
 class CommitTask(RecorderTask):
     """Commit the event session."""
 
@@ -261,11 +292,10 @@ class CommitTask(RecorderTask):
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
-        # pylint: disable-next=[protected-access]
-        instance._commit_event_session_or_retry()
+        instance._commit_event_session_or_retry()  # noqa: SLF001
 
 
-@dataclass
+@dataclass(slots=True)
 class AddRecorderPlatformTask(RecorderTask):
     """Add a recorder platform."""
 
@@ -278,22 +308,49 @@ class AddRecorderPlatformTask(RecorderTask):
         hass = instance.hass
         domain = self.domain
         platform = self.platform
-
-        platforms: dict[str, Any] = hass.data[DOMAIN].recorder_platforms
+        platforms: dict[str, Any] = hass.data[DATA_RECORDER].recorder_platforms
         platforms[domain] = platform
-        if hasattr(self.platform, "exclude_attributes"):
-            hass.data[EXCLUDE_ATTRIBUTES][domain] = platform.exclude_attributes(hass)
 
 
-@dataclass
+@dataclass(slots=True)
 class SynchronizeTask(RecorderTask):
     """Ensure all pending data has been committed."""
 
     # commit_before is the default
-    event: asyncio.Event
+    future: asyncio.Future
 
     def run(self, instance: Recorder) -> None:
         """Handle the task."""
         # Does not use a tracked task to avoid
         # blocking shutdown if the recorder is broken
-        instance.hass.loop.call_soon_threadsafe(self.event.set)
+        instance.hass.loop.call_soon_threadsafe(self._set_result_if_not_done)
+
+    def _set_result_if_not_done(self) -> None:
+        """Set the result if not done."""
+        if not self.future.done():
+            self.future.set_result(None)
+
+
+@dataclass(slots=True)
+class AdjustLRUSizeTask(RecorderTask):
+    """An object to insert into the recorder queue to adjust the LRU size."""
+
+    commit_before = False
+
+    def run(self, instance: Recorder) -> None:
+        """Handle the task to adjust the size."""
+        instance._adjust_lru_size()  # noqa: SLF001
+
+
+@dataclass(slots=True)
+class RefreshEventTypesTask(RecorderTask):
+    """An object to insert into the recorder queue to refresh event types."""
+
+    event_types: list[EventType[Any] | str]
+
+    def run(self, instance: Recorder) -> None:
+        """Refresh event types."""
+        with session_scope(session=instance.get_session(), read_only=True) as session:
+            instance.event_type_manager.get_many(
+                self.event_types, session, from_recorder=True
+            )
