@@ -1,4 +1,5 @@
 """The Renson integration."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass
@@ -19,6 +20,7 @@ PLATFORMS = [
     Platform.FAN,
     Platform.NUMBER,
     Platform.SENSOR,
+    Platform.SWITCH,
     Platform.TIME,
 ]
 
@@ -35,7 +37,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Renson from a config entry."""
 
     api = RensonVentilation(entry.data[CONF_HOST])
-    coordinator = RensonCoordinator("Renson", hass, api)
+    coordinator = RensonCoordinator(hass, entry, api)
 
     if not await hass.async_add_executor_job(api.connect):
         raise ConfigEntryNotReady("Cannot connect to Renson device")
