@@ -1,12 +1,19 @@
 """Camera platform that has a Raspberry Pi camera."""
+
+from __future__ import annotations
+
 import logging
 import os
 import shutil
 import subprocess
 from tempfile import NamedTemporaryFile
+from typing import Any
 
 from homeassistant.components.camera import Camera
 from homeassistant.const import CONF_FILE_PATH, CONF_NAME, EVENT_HOMEASSISTANT_STOP
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from .const import (
     CONF_HORIZONTAL_FLIP,
@@ -26,12 +33,21 @@ _LOGGER = logging.getLogger(__name__)
 
 def kill_raspistill(*args):
     """Kill any previously running raspistill process.."""
-    subprocess.Popen(
-        ["killall", "raspistill"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT
-    )
+    with subprocess.Popen(
+        ["killall", "raspistill"],
+        stdout=subprocess.DEVNULL,
+        stderr=subprocess.STDOUT,
+        close_fds=False,  # required for posix_spawn
+    ):
+        pass
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Raspberry Camera."""
     # We only want this platform to be set up via discovery.
     # prevent initializing by erroneous platform config section in yaml conf
@@ -72,11 +88,11 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 class RaspberryCamera(Camera):
     """Representation of a Raspberry Pi camera."""
 
-    def __init__(self, device_info):
+    def __init__(self, device_info: dict[str, Any]) -> None:
         """Initialize Raspberry Pi camera component."""
         super().__init__()
 
-        self._name = device_info[CONF_NAME]
+        self._attr_name = device_info[CONF_NAME]
         self._config = device_info
 
         # Kill if there's raspistill instance
@@ -116,19 +132,25 @@ class RaspberryCamera(Camera):
             cmd_args.append("-a")
             cmd_args.append(str(device_info[CONF_OVERLAY_TIMESTAMP]))
 
-        subprocess.Popen(cmd_args, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
+        # The raspistill process started below must run "forever" in
+        # the background until killed when Home Assistant is stopped.
+        # Therefore it must not be wrapped with "with", since that
+        # waits for the subprocess to exit before continuing.
+        subprocess.Popen(  # pylint: disable=consider-using-with
+            cmd_args,
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.STDOUT,
+            close_fds=False,  # required for posix_spawn
+        )
 
-    def camera_image(self):
+    def camera_image(
+        self, width: int | None = None, height: int | None = None
+    ) -> bytes | None:
         """Return raspistill image response."""
         with open(self._config[CONF_FILE_PATH], "rb") as file:
             return file.read()
 
     @property
-    def name(self):
-        """Return the name of this camera."""
-        return self._name
-
-    @property
-    def frame_interval(self):
+    def frame_interval(self) -> float:
         """Return the interval between frames of the stream."""
         return self._config[CONF_TIMELAPSE] / 1000
