@@ -2,11 +2,8 @@
 
 from __future__ import annotations
 
-from typing import Any
-
 from homeassistant.components.sensor import SensorEntity
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_FRIENDLY_NAME, ATTR_LOCATION
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.device_registry import DeviceEntryType, DeviceInfo
 from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
@@ -14,15 +11,7 @@ from homeassistant.helpers.typing import StateType
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from . import SeventeenTrackCoordinator
-from .const import (
-    ATTR_INFO_TEXT,
-    ATTR_PACKAGES,
-    ATTR_STATUS,
-    ATTR_TIMESTAMP,
-    ATTR_TRACKING_NUMBER,
-    ATTRIBUTION,
-    DOMAIN,
-)
+from .const import ATTRIBUTION, DOMAIN
 
 
 async def async_setup_entry(
@@ -81,22 +70,3 @@ class SeventeenTrackSummarySensor(SeventeenTrackSensor):
     def native_value(self) -> StateType:
         """Return the state of the sensor."""
         return self.coordinator.data.summary[self._status]["quantity"]
-
-    # This has been deprecated in 2024.8, will be removed in 2025.2
-    @property
-    def extra_state_attributes(self) -> dict[str, Any] | None:
-        """Return the state attributes."""
-        packages = self.coordinator.data.summary[self._status]["packages"]
-        return {
-            ATTR_PACKAGES: [
-                {
-                    ATTR_TRACKING_NUMBER: package.tracking_number,
-                    ATTR_LOCATION: package.location,
-                    ATTR_STATUS: package.status,
-                    ATTR_TIMESTAMP: package.timestamp,
-                    ATTR_INFO_TEXT: package.info_text,
-                    ATTR_FRIENDLY_NAME: package.friendly_name,
-                }
-                for package in packages
-            ]
-        }
