@@ -1,16 +1,17 @@
 """Slack platform for notify component."""
+
 from __future__ import annotations
 
 import asyncio
 import logging
 import os
-from typing import Any, TypedDict
+from typing import Any, TypedDict, cast
 from urllib.parse import urlparse
 
-from aiohttp import BasicAuth, FormData
+from aiohttp import BasicAuth
 from aiohttp.client_exceptions import ClientError
-from slack import WebClient
-from slack.errors import SlackApiError
+from slack_sdk.errors import SlackApiError
+from slack_sdk.web.async_client import AsyncWebClient
 import voluptuous as vol
 
 from homeassistant.components.notify import (
@@ -37,6 +38,7 @@ from .const import (
     DATA_CLIENT,
     SLACK_DATA,
 )
+from .utils import upload_file_to_slack
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -135,7 +137,7 @@ class SlackNotificationService(BaseNotificationService):
     def __init__(
         self,
         hass: HomeAssistant,
-        client: WebClient,
+        client: AsyncWebClient,
         config: dict[str, str],
     ) -> None:
         """Initialize."""
@@ -159,17 +161,23 @@ class SlackNotificationService(BaseNotificationService):
         parsed_url = urlparse(path)
         filename = os.path.basename(parsed_url.path)
 
-        try:
-            await self._client.files_upload(
-                channels=",".join(targets),
-                file=path,
-                filename=filename,
-                initial_comment=message,
-                title=title or filename,
-                thread_ts=thread_ts or "",
-            )
-        except (SlackApiError, ClientError) as err:
-            _LOGGER.error("Error while uploading file-based message: %r", err)
+        channel_ids = [await self._async_get_channel_id(target) for target in targets]
+        channel_ids = [cid for cid in channel_ids if cid]  # Remove None values
+
+        if not channel_ids:
+            _LOGGER.error("No valid channel IDs resolved for targets: %s", targets)
+            return
+
+        await upload_file_to_slack(
+            client=self._client,
+            channel_ids=channel_ids,
+            file_content=None,
+            file_path=path,
+            filename=filename,
+            title=title,
+            message=message,
+            thread_ts=thread_ts,
+        )
 
     async def _async_send_remote_file_message(
         self,
@@ -182,12 +190,7 @@ class SlackNotificationService(BaseNotificationService):
         username: str | None = None,
         password: str | None = None,
     ) -> None:
-        """Upload a remote file (with message) to Slack.
-
-        Note that we bypass the python-slackclient WebClient and use aiohttp directly,
-        as the former would require us to download the entire remote file into memory
-        first before uploading it to Slack.
-        """
+        """Upload a remote file (with message) to Slack."""
         if not self._hass.config.is_allowed_external_url(url):
             _LOGGER.error("URL is not allowed: %s", url)
             return
@@ -195,36 +198,35 @@ class SlackNotificationService(BaseNotificationService):
         filename = _async_get_filename_from_url(url)
         session = aiohttp_client.async_get_clientsession(self._hass)
 
+        # Fetch the remote file
         kwargs: AuthDictT = {}
-        if username and password is not None:
+        if username and password:
             kwargs = {"auth": BasicAuth(username, password=password)}
 
-        resp = await session.request("get", url, **kwargs)
-
         try:
-            resp.raise_for_status()
+            async with session.get(url, **kwargs) as resp:
+                resp.raise_for_status()
+                file_content = await resp.read()
         except ClientError as err:
             _LOGGER.error("Error while retrieving %s: %r", url, err)
             return
 
-        form_data: FormDataT = {
-            "channels": ",".join(targets),
-            "filename": filename,
-            "initial_comment": message,
-            "title": title or filename,
-            "token": self._client.token,
-        }
+        channel_ids = [await self._async_get_channel_id(target) for target in targets]
+        channel_ids = [cid for cid in channel_ids if cid]  # Remove None values
 
-        if thread_ts:
-            form_data["thread_ts"] = thread_ts
+        if not channel_ids:
+            _LOGGER.error("No valid channel IDs resolved for targets: %s", targets)
+            return
 
-        data = FormData(form_data, charset="utf-8")
-        data.add_field("file", resp.content, filename=filename)
-
-        try:
-            await session.post("https://slack.com/api/files.upload", data=data)
-        except ClientError as err:
-            _LOGGER.error("Error while uploading file message: %r", err)
+        await upload_file_to_slack(
+            client=self._client,
+            channel_ids=channel_ids,
+            file_content=file_content,
+            filename=filename,
+            title=title,
+            message=message,
+            thread_ts=thread_ts,
+        )
 
     async def _async_send_text_only_message(
         self,
@@ -261,7 +263,7 @@ class SlackNotificationService(BaseNotificationService):
         }
 
         results = await asyncio.gather(*tasks.values(), return_exceptions=True)
-        for target, result in zip(tasks, results):
+        for target, result in zip(tasks, results, strict=False):
             if isinstance(result, SlackApiError):
                 _LOGGER.error(
                     "There was a Slack API error while sending to %s: %r",
@@ -290,7 +292,6 @@ class SlackNotificationService(BaseNotificationService):
         if ATTR_FILE not in data:
             if ATTR_BLOCKS_TEMPLATE in data:
                 value = cv.template_complex(data[ATTR_BLOCKS_TEMPLATE])
-                template.attach(self._hass, value)
                 blocks = template.render_complex(value)
             elif ATTR_BLOCKS in data:
                 blocks = data[ATTR_BLOCKS]
@@ -327,3 +328,46 @@ class SlackNotificationService(BaseNotificationService):
             title,
             thread_ts=data.get(ATTR_THREAD_TS),
         )
+
+    async def _async_get_channel_id(self, channel_name: str) -> str | None:
+        """Get the Slack channel ID from the channel name.
+
+        This method retrieves the channel ID for a given Slack channel name by
+        querying the Slack API. It handles both public and private channels.
+        Including this so users can  provide channel names instead of IDs.
+
+        Args:
+            channel_name (str): The name of the Slack channel.
+
+        Returns:
+            str | None: The ID of the Slack channel if found, otherwise None.
+
+        Raises:
+            SlackApiError: If there is an error while communicating with the Slack API.
+
+        """
+        try:
+            # Remove # if present
+            channel_name = channel_name.lstrip("#")
+
+            # Get channel list
+            # Multiple types is not working. Tested here: https://api.slack.com/methods/conversations.list/test
+            # response = await self._client.conversations_list(types="public_channel,private_channel")
+            #
+            # Workaround for the types parameter not working
+            channels = []
+            for channel_type in ("public_channel", "private_channel"):
+                response = await self._client.conversations_list(types=channel_type)
+                channels.extend(response["channels"])
+
+            # Find channel ID
+            for channel in channels:
+                if channel["name"] == channel_name:
+                    return cast(str, channel["id"])
+
+            _LOGGER.error("Channel %s not found", channel_name)
+
+        except SlackApiError as err:
+            _LOGGER.error("Error getting channel ID: %r", err)
+
+        return None
