@@ -1,6 +1,7 @@
 """Config flow for Smart Meter Texas integration."""
-import asyncio
+
 import logging
+from typing import Any
 
 from aiohttp import ClientError
 from smart_meter_texas import Account, Client
@@ -10,9 +11,12 @@ from smart_meter_texas.exceptions import (
 )
 import voluptuous as vol
 
-from homeassistant import config_entries, core, exceptions
+from homeassistant.config_entries import ConfigFlow, ConfigFlowResult
 from homeassistant.const import CONF_PASSWORD, CONF_USERNAME
+from homeassistant.core import HomeAssistant
+from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import aiohttp_client
+from homeassistant.util.ssl import get_default_context
 
 from .const import DOMAIN
 
@@ -23,19 +27,19 @@ DATA_SCHEMA = vol.Schema(
 )
 
 
-async def validate_input(hass: core.HomeAssistant, data):
+async def validate_input(hass: HomeAssistant, data):
     """Validate the user input allows us to connect.
 
     Data has the keys from DATA_SCHEMA with values provided by the user.
     """
-
+    ssl_context = get_default_context()
     client_session = aiohttp_client.async_get_clientsession(hass)
     account = Account(data["username"], data["password"])
-    client = Client(client_session, account)
+    client = Client(client_session, account, ssl_context)
 
     try:
         await client.authenticate()
-    except (asyncio.TimeoutError, ClientError, SmartMeterTexasAPIError) as error:
+    except (TimeoutError, ClientError, SmartMeterTexasAPIError) as error:
         raise CannotConnect from error
     except SmartMeterTexasAuthError as error:
         raise InvalidAuth(error) from error
@@ -44,13 +48,14 @@ async def validate_input(hass: core.HomeAssistant, data):
     return {"title": account.username}
 
 
-class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
+class SMTConfigFlow(ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Smart Meter Texas."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_CLOUD_POLL
 
-    async def async_step_user(self, user_input=None):
+    async def async_step_user(
+        self, user_input: dict[str, Any] | None = None
+    ) -> ConfigFlowResult:
         """Handle the initial step."""
 
         errors = {}
@@ -61,7 +66,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 errors["base"] = "cannot_connect"
             except InvalidAuth:
                 errors["base"] = "invalid_auth"
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 _LOGGER.exception("Unexpected exception")
                 errors["base"] = "unknown"
             else:
@@ -77,9 +82,9 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
 
-class CannotConnect(exceptions.HomeAssistantError):
+class CannotConnect(HomeAssistantError):
     """Error to indicate we cannot connect."""
 
 
-class InvalidAuth(exceptions.HomeAssistantError):
+class InvalidAuth(HomeAssistantError):
     """Error to indicate there is invalid auth."""
