@@ -7,9 +7,10 @@ from dataclasses import dataclass
 from datetime import datetime
 from typing import Any, cast
 
-from pysmartthings import Attribute, Capability, SmartThings, Status
+from pysmartthings import Attribute, Capability, ComponentStatus, SmartThings, Status
 
 from homeassistant.components.sensor import (
+    DOMAIN as SENSOR_DOMAIN,
     SensorDeviceClass,
     SensorEntity,
     SensorEntityDescription,
@@ -25,16 +26,19 @@ from homeassistant.const import (
     UnitOfEnergy,
     UnitOfMass,
     UnitOfPower,
+    UnitOfPressure,
     UnitOfTemperature,
     UnitOfVolume,
 )
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 from homeassistant.util import dt as dt_util
 
 from . import FullDevice, SmartThingsConfigEntry
 from .const import MAIN
 from .entity import SmartThingsEntity
+from .util import deprecate_entity
 
 THERMOSTAT_CAPABILITIES = {
     Capability.TEMPERATURE_MEASUREMENT,
@@ -42,6 +46,17 @@ THERMOSTAT_CAPABILITIES = {
     Capability.THERMOSTAT_MODE,
 }
 
+COOKTOP_HEATING_MODES = {
+    "off": "off",
+    "manual": "manual",
+    "boost": "boost",
+    "keepWarm": "keep_warm",
+    "quickPreheat": "quick_preheat",
+    "defrost": "defrost",
+    "melt": "melt",
+    "simmer": "simmer",
+}
+
 JOB_STATE_MAP = {
     "airWash": "air_wash",
     "airwash": "air_wash",
@@ -128,10 +143,15 @@ class SmartThingsSensorEntityDescription(SensorEntityDescription):
 
     value_fn: Callable[[Any], str | float | int | datetime | None] = lambda value: value
     extra_state_attributes_fn: Callable[[Any], dict[str, Any]] | None = None
-    unique_id_separator: str = "."
     capability_ignore_list: list[set[Capability]] | None = None
     options_attribute: Attribute | None = None
+    options_map: dict[str, str] | None = None
+    translation_placeholders_fn: Callable[[str], dict[str, str]] | None = None
+    component_fn: Callable[[str], bool] | None = None
     exists_fn: Callable[[Status], bool] | None = None
+    use_temperature_unit: bool = False
+    deprecated: Callable[[ComponentStatus], tuple[str, str] | None] | None = None
+    component_translation_key: dict[str, str] | None = None
 
 
 CAPABILITY_TO_SENSORS: dict[
@@ -182,12 +202,26 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
+    Capability.ATMOSPHERIC_PRESSURE_MEASUREMENT: {
+        Attribute.ATMOSPHERIC_PRESSURE: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.ATMOSPHERIC_PRESSURE,
+                device_class=SensorDeviceClass.ATMOSPHERIC_PRESSURE,
+                state_class=SensorStateClass.MEASUREMENT,
+            )
+        ]
+    },
     Capability.AUDIO_VOLUME: {
         Attribute.VOLUME: [
             SmartThingsSensorEntityDescription(
                 key=Attribute.VOLUME,
                 translation_key="audio_volume",
                 native_unit_of_measurement=PERCENTAGE,
+                deprecated=(
+                    lambda status: ("2025.10.0", "media_player")
+                    if Capability.AUDIO_MUTE in status
+                    else None
+                ),
             )
         ]
     },
@@ -224,7 +258,6 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
-    # Haven't seen at devices yet
     Capability.CARBON_DIOXIDE_MEASUREMENT: {
         Attribute.CARBON_DIOXIDE: [
             SmartThingsSensorEntityDescription(
@@ -257,6 +290,51 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
+    Capability.SAMSUNG_CE_COOKTOP_HEATING_POWER: {
+        Attribute.MANUAL_LEVEL: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.MANUAL_LEVEL,
+                translation_key="manual_level",
+                translation_placeholders_fn=lambda component: {
+                    "burner_id": component.split("-0")[-1]
+                },
+                component_fn=lambda component: component.startswith("burner-0"),
+            )
+        ],
+        Attribute.HEATING_MODE: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.HEATING_MODE,
+                translation_key="heating_mode",
+                options_attribute=Attribute.SUPPORTED_HEATING_MODES,
+                options_map=COOKTOP_HEATING_MODES,
+                device_class=SensorDeviceClass.ENUM,
+                translation_placeholders_fn=lambda component: {
+                    "burner_id": component.split("-0")[-1]
+                },
+                component_fn=lambda component: component.startswith("burner-0"),
+            )
+        ],
+    },
+    Capability.CUSTOM_COOKTOP_OPERATING_STATE: {
+        Attribute.COOKTOP_OPERATING_STATE: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.COOKTOP_OPERATING_STATE,
+                translation_key="cooktop_operating_state",
+                device_class=SensorDeviceClass.ENUM,
+                options_attribute=Attribute.SUPPORTED_COOKTOP_OPERATING_STATE,
+            )
+        ]
+    },
+    Capability.CUSTOM_WATER_FILTER: {
+        Attribute.WATER_FILTER_USAGE: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.WATER_FILTER_USAGE,
+                translation_key="water_filter_usage",
+                native_unit_of_measurement=PERCENTAGE,
+                state_class=SensorStateClass.MEASUREMENT,
+            )
+        ]
+    },
     Capability.DISHWASHER_OPERATING_STATE: {
         Attribute.MACHINE_STATE: [
             SmartThingsSensorEntityDescription(
@@ -366,6 +444,16 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ],
     },
+    Capability.SAMSUNG_CE_EHS_DIVERTER_VALVE: {
+        Attribute.POSITION: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.POSITION,
+                translation_key="diverter_valve_position",
+                device_class=SensorDeviceClass.ENUM,
+                options=["room", "tank"],
+            )
+        ]
+    },
     Capability.ENERGY_METER: {
         Attribute.ENERGY: [
             SmartThingsSensorEntityDescription(
@@ -399,7 +487,6 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
-    # Haven't seen at devices yet
     Capability.GAS_METER: {
         Attribute.GAS_METER: [
             SmartThingsSensorEntityDescription(
@@ -407,7 +494,7 @@ CAPABILITY_TO_SENSORS: dict[
                 translation_key="gas_meter",
                 native_unit_of_measurement=UnitOfEnergy.KILO_WATT_HOUR,
                 device_class=SensorDeviceClass.ENERGY,
-                state_class=SensorStateClass.MEASUREMENT,
+                state_class=SensorStateClass.TOTAL,
             )
         ],
         Attribute.GAS_METER_CALORIFIC: [
@@ -429,7 +516,7 @@ CAPABILITY_TO_SENSORS: dict[
                 key=Attribute.GAS_METER_VOLUME,
                 native_unit_of_measurement=UnitOfVolume.CUBIC_METERS,
                 device_class=SensorDeviceClass.GAS,
-                state_class=SensorStateClass.MEASUREMENT,
+                state_class=SensorStateClass.TOTAL,
             )
         ],
     },
@@ -463,24 +550,25 @@ CAPABILITY_TO_SENSORS: dict[
                 device_class=SensorDeviceClass.ENUM,
                 options_attribute=Attribute.SUPPORTED_INPUT_SOURCES,
                 value_fn=lambda value: value.lower() if value else None,
+                deprecated=lambda _: ("2025.10.0", "media_player"),
             )
         ]
     },
-    # part of the proposed spec, Haven't seen at devices yet
     Capability.MEDIA_PLAYBACK_REPEAT: {
         Attribute.PLAYBACK_REPEAT_MODE: [
             SmartThingsSensorEntityDescription(
                 key=Attribute.PLAYBACK_REPEAT_MODE,
                 translation_key="media_playback_repeat",
+                deprecated=lambda _: ("2025.10.0", "media_player"),
             )
         ]
     },
-    # part of the proposed spec, Haven't seen at devices yet
     Capability.MEDIA_PLAYBACK_SHUFFLE: {
         Attribute.PLAYBACK_SHUFFLE: [
             SmartThingsSensorEntityDescription(
                 key=Attribute.PLAYBACK_SHUFFLE,
                 translation_key="media_playback_shuffle",
+                deprecated=lambda _: ("2025.10.0", "media_player"),
             )
         ]
     },
@@ -499,6 +587,7 @@ CAPABILITY_TO_SENSORS: dict[
                 ],
                 device_class=SensorDeviceClass.ENUM,
                 value_fn=lambda value: MEDIA_PLAYBACK_STATE_MAP.get(value, value),
+                deprecated=lambda _: ("2025.10.0", "media_player"),
             )
         ]
     },
@@ -573,8 +662,9 @@ CAPABILITY_TO_SENSORS: dict[
                 key=Attribute.OVEN_SETPOINT,
                 translation_key="oven_setpoint",
                 device_class=SensorDeviceClass.TEMPERATURE,
-                native_unit_of_measurement=UnitOfTemperature.FAHRENHEIT,
-                value_fn=lambda value: value if value != 0 else None,
+                use_temperature_unit=True,
+                # Set the value to None if it is 0 F (-17 C)
+                value_fn=lambda value: None if value in {-17, 0, 1} else value,
             )
         ]
     },
@@ -621,7 +711,7 @@ CAPABILITY_TO_SENSORS: dict[
             SmartThingsSensorEntityDescription(
                 key="powerEnergy_meter",
                 translation_key="power_energy",
-                state_class=SensorStateClass.TOTAL_INCREASING,
+                state_class=SensorStateClass.TOTAL,
                 device_class=SensorDeviceClass.ENERGY,
                 native_unit_of_measurement=UnitOfEnergy.KILO_WATT_HOUR,
                 value_fn=lambda value: value["powerEnergy"] / 1000,
@@ -676,6 +766,15 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
+    Capability.RELATIVE_BRIGHTNESS: {
+        Attribute.BRIGHTNESS_INTENSITY: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.BRIGHTNESS_INTENSITY,
+                translation_key="brightness_intensity",
+                state_class=SensorStateClass.MEASUREMENT,
+            )
+        ]
+    },
     Capability.RELATIVE_HUMIDITY_MEASUREMENT: {
         Attribute.HUMIDITY: [
             SmartThingsSensorEntityDescription(
@@ -769,6 +868,16 @@ CAPABILITY_TO_SENSORS: dict[
                 key=Attribute.TEMPERATURE,
                 device_class=SensorDeviceClass.TEMPERATURE,
                 state_class=SensorStateClass.MEASUREMENT,
+                deprecated=(
+                    lambda status: ("2025.12.0", "dhw")
+                    if Capability.CUSTOM_OUTING_MODE in status
+                    else None
+                ),
+                component_fn=lambda component: component in {"freezer", "cooler"},
+                component_translation_key={
+                    "freezer": "freezer_temperature",
+                    "cooler": "cooler_temperature",
+                },
             )
         ]
     },
@@ -786,6 +895,11 @@ CAPABILITY_TO_SENSORS: dict[
                     },
                     THERMOSTAT_CAPABILITIES,
                 ],
+                deprecated=(
+                    lambda status: ("2025.12.0", "dhw")
+                    if Capability.CUSTOM_OUTING_MODE in status
+                    else None
+                ),
             )
         ]
     },
@@ -847,21 +961,18 @@ CAPABILITY_TO_SENSORS: dict[
     Capability.THREE_AXIS: {
         Attribute.THREE_AXIS: [
             SmartThingsSensorEntityDescription(
-                key="X Coordinate",
+                key="x_coordinate",
                 translation_key="x_coordinate",
-                unique_id_separator=" ",
                 value_fn=lambda value: value[0],
             ),
             SmartThingsSensorEntityDescription(
-                key="Y Coordinate",
+                key="y_coordinate",
                 translation_key="y_coordinate",
-                unique_id_separator=" ",
                 value_fn=lambda value: value[1],
             ),
             SmartThingsSensorEntityDescription(
-                key="Z Coordinate",
+                key="z_coordinate",
                 translation_key="z_coordinate",
-                unique_id_separator=" ",
                 value_fn=lambda value: value[2],
             ),
         ]
@@ -901,6 +1012,16 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ]
     },
+    Capability.VERY_FINE_DUST_SENSOR: {
+        Attribute.VERY_FINE_DUST_LEVEL: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.VERY_FINE_DUST_LEVEL,
+                native_unit_of_measurement=CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
+                device_class=SensorDeviceClass.PM1,
+                state_class=SensorStateClass.MEASUREMENT,
+            )
+        ]
+    },
     Capability.VOLTAGE_MEASUREMENT: {
         Attribute.VOLTAGE: [
             SmartThingsSensorEntityDescription(
@@ -964,15 +1085,29 @@ CAPABILITY_TO_SENSORS: dict[
             )
         ],
     },
+    Capability.SAMSUNG_CE_WATER_CONSUMPTION_REPORT: {
+        Attribute.WATER_CONSUMPTION: [
+            SmartThingsSensorEntityDescription(
+                key=Attribute.WATER_CONSUMPTION,
+                translation_key="water_consumption",
+                state_class=SensorStateClass.TOTAL_INCREASING,
+                device_class=SensorDeviceClass.WATER,
+                native_unit_of_measurement=UnitOfVolume.LITERS,
+                value_fn=lambda value: value["cumulativeAmount"] / 1000,
+            )
+        ]
+    },
 }
 
 
 UNITS = {
     "C": UnitOfTemperature.CELSIUS,
     "F": UnitOfTemperature.FAHRENHEIT,
+    "ccf": UnitOfVolume.CENTUM_CUBIC_FEET,
     "lux": LIGHT_LUX,
     "mG": None,
     "μg/m^3": CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
+    "kPa": UnitOfPressure.KPA,
 }
 
 
@@ -983,25 +1118,82 @@ async def async_setup_entry(
 ) -> None:
     """Add sensors for a config entry."""
     entry_data = entry.runtime_data
-    async_add_entities(
-        SmartThingsSensor(entry_data.client, device, description, capability, attribute)
-        for device in entry_data.devices.values()
-        for capability, attributes in CAPABILITY_TO_SENSORS.items()
-        if capability in device.status[MAIN]
-        for attribute, descriptions in attributes.items()
-        for description in descriptions
-        if (
-            not description.capability_ignore_list
-            or not any(
-                all(capability in device.status[MAIN] for capability in capability_list)
-                for capability_list in description.capability_ignore_list
-            )
-        )
-        and (
-            not description.exists_fn
-            or description.exists_fn(device.status[MAIN][capability][attribute])
-        )
-    )
+    entities = []
+
+    entity_registry = er.async_get(hass)
+
+    for device in entry_data.devices.values():  # pylint: disable=too-many-nested-blocks
+        for capability, attributes in CAPABILITY_TO_SENSORS.items():
+            for component, capabilities in device.status.items():
+                if capability in capabilities:
+                    for attribute, descriptions in attributes.items():
+                        for description in descriptions:
+                            if (
+                                (
+                                    not description.capability_ignore_list
+                                    or not any(
+                                        all(
+                                            capability in device.status[MAIN]
+                                            for capability in capability_list
+                                        )
+                                        for capability_list in description.capability_ignore_list
+                                    )
+                                )
+                                and (
+                                    not description.exists_fn
+                                    or description.exists_fn(
+                                        device.status[MAIN][capability][attribute]
+                                    )
+                                )
+                                and (
+                                    component == MAIN
+                                    or (
+                                        description.component_fn is not None
+                                        and description.component_fn(component)
+                                    )
+                                )
+                            ):
+                                if (
+                                    description.deprecated
+                                    and (
+                                        deprecation_info := description.deprecated(
+                                            device.status[MAIN]
+                                        )
+                                    )
+                                    is not None
+                                ):
+                                    version, reason = deprecation_info
+                                    if deprecate_entity(
+                                        hass,
+                                        entity_registry,
+                                        SENSOR_DOMAIN,
+                                        f"{device.device.device_id}_{MAIN}_{capability}_{attribute}_{description.key}",
+                                        f"deprecated_{reason}",
+                                        version,
+                                    ):
+                                        entities.append(
+                                            SmartThingsSensor(
+                                                entry_data.client,
+                                                device,
+                                                description,
+                                                MAIN,
+                                                capability,
+                                                attribute,
+                                            )
+                                        )
+                                    continue
+                                entities.append(
+                                    SmartThingsSensor(
+                                        entry_data.client,
+                                        device,
+                                        description,
+                                        component,
+                                        capability,
+                                        attribute,
+                                    )
+                                )
+
+    async_add_entities(entities)
 
 
 class SmartThingsSensor(SmartThingsEntity, SensorEntity):
@@ -1014,26 +1206,45 @@ class SmartThingsSensor(SmartThingsEntity, SensorEntity):
         client: SmartThings,
         device: FullDevice,
         entity_description: SmartThingsSensorEntityDescription,
+        component: str,
         capability: Capability,
         attribute: Attribute,
     ) -> None:
         """Init the class."""
-        super().__init__(client, device, {capability})
-        self._attr_unique_id = f"{device.device.device_id}{entity_description.unique_id_separator}{entity_description.key}"
+        capabilities_to_subscribe = {capability}
+        if entity_description.use_temperature_unit:
+            capabilities_to_subscribe.add(Capability.TEMPERATURE_MEASUREMENT)
+        super().__init__(client, device, capabilities_to_subscribe, component=component)
+        self._attr_unique_id = f"{device.device.device_id}_{component}_{capability}_{attribute}_{entity_description.key}"
         self._attribute = attribute
         self.capability = capability
         self.entity_description = entity_description
+        if self.entity_description.translation_placeholders_fn:
+            self._attr_translation_placeholders = (
+                self.entity_description.translation_placeholders_fn(component)
+            )
+        if self.entity_description.component_translation_key and component != MAIN:
+            self._attr_translation_key = (
+                self.entity_description.component_translation_key[component]
+            )
 
     @property
     def native_value(self) -> str | float | datetime | int | None:
         """Return the state of the sensor."""
         res = self.get_attribute_value(self.capability, self._attribute)
+        if options_map := self.entity_description.options_map:
+            return options_map.get(res)
         return self.entity_description.value_fn(res)
 
     @property
     def native_unit_of_measurement(self) -> str | None:
         """Return the unit this state is expressed in."""
-        unit = self._internal_state[self.capability][self._attribute].unit
+        if self.entity_description.use_temperature_unit:
+            unit = self._internal_state[Capability.TEMPERATURE_MEASUREMENT][
+                Attribute.TEMPERATURE
+            ].unit
+        else:
+            unit = self._internal_state[self.capability][self._attribute].unit
         return (
             UNITS.get(unit, unit)
             if unit
@@ -1059,5 +1270,7 @@ class SmartThingsSensor(SmartThingsEntity, SensorEntity):
                 )
             ) is None:
                 return []
+            if options_map := self.entity_description.options_map:
+                return [options_map[option] for option in options]
             return [option.lower() for option in options]
         return super().options
