@@ -1,4 +1,5 @@
 """Base class for common speaker tasks."""
+
 from __future__ import annotations
 
 import asyncio
@@ -8,7 +9,7 @@ import datetime
 from functools import partial
 import logging
 import time
-from typing import Any, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import defusedxml.ElementTree as ET
 from soco.core import SoCo
@@ -20,7 +21,6 @@ from soco.snapshot import Snapshot
 from sonos_websocket import SonosWebsocket
 
 from homeassistant.components.media_player import DOMAIN as MP_DOMAIN
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import entity_registry as er
@@ -30,14 +30,13 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
     dispatcher_send,
 )
-from homeassistant.helpers.event import async_track_time_interval, track_time_interval
+from homeassistant.helpers.event import async_track_time_interval
 from homeassistant.util import dt as dt_util
 
 from .alarms import SonosAlarms
 from .const import (
     AVAILABILITY_TIMEOUT,
     BATTERY_SCAN_INTERVAL,
-    DATA_SONOS,
     DOMAIN,
     SCAN_INTERVAL,
     SONOS_CHECK_ACTIVITY,
@@ -64,6 +63,10 @@ from .helpers import soco_error
 from .media import SonosMedia
 from .statistics import ActivityStatistics, EventStatistics
 
+if TYPE_CHECKING:
+    from .helpers import SonosConfigEntry
+
+
 NEVER_TIME = -1200.0
 RESUB_COOLDOWN_SECONDS = 10.0
 EVENT_CHARGING = {
@@ -91,12 +94,15 @@ class SonosSpeaker:
     def __init__(
         self,
         hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
         soco: SoCo,
         speaker_info: dict[str, Any],
         zone_group_state_sub: SubscriptionBase | None,
     ) -> None:
         """Initialize a SonosSpeaker."""
         self.hass = hass
+        self.config_entry = config_entry
+        self.data = config_entry.runtime_data
         self.soco = soco
         self.websocket: SonosWebsocket | None = None
         self.household_id: str = soco.household_id
@@ -122,7 +128,6 @@ class SonosSpeaker:
             zone_group_state_sub.callback = self.async_dispatch_event
             self._subscriptions.append(zone_group_state_sub)
         self._subscription_lock: asyncio.Lock | None = None
-        self._event_dispatchers: dict[str, Callable] = {}
         self._last_activity: float = NEVER_TIME
         self._last_event_cache: dict[str, Any] = {}
         self.activity_stats: ActivityStatistics = ActivityStatistics(self.zone_name)
@@ -174,16 +179,28 @@ class SonosSpeaker:
         self.snapshot_group: list[SonosSpeaker] = []
         self._group_members_missing: set[str] = set()
 
-    async def async_setup(self, entry: ConfigEntry) -> None:
+    async def async_setup(
+        self,
+        entry: SonosConfigEntry,
+        has_battery: bool,
+        dispatches: list[tuple[Any, ...]],
+    ) -> None:
         """Complete setup in async context."""
+        # Battery events can be infrequent, polling is still necessary
+        if has_battery:
+            self._battery_poll_timer = async_track_time_interval(
+                self.hass, self.async_poll_battery, BATTERY_SCAN_INTERVAL
+            )
+
         self.websocket = SonosWebsocket(
             self.soco.ip_address,
             player_id=self.soco.uid,
             session=async_get_clientsession(self.hass),
         )
+
         dispatch_pairs: tuple[tuple[str, Callable[..., Any]], ...] = (
             (SONOS_CHECK_ACTIVITY, self.async_check_activity),
-            (SONOS_SPEAKER_ADDED, self.update_group_for_uid),
+            (SONOS_SPEAKER_ADDED, self.async_update_group_for_uid),
             (f"{SONOS_REBOOTED}-{self.soco.uid}", self.async_rebooted),
             (f"{SONOS_SPEAKER_ACTIVITY}-{self.soco.uid}", self.speaker_activity),
             (f"{SONOS_VANISHED}-{self.soco.uid}", self.async_vanished),
@@ -198,7 +215,12 @@ class SonosSpeaker:
                 )
             )
 
-    def setup(self, entry: ConfigEntry) -> None:
+        for dispatch in dispatches:
+            async_dispatcher_send(self.hass, *dispatch)
+
+        await self.async_subscribe()
+
+    def setup(self, entry: SonosConfigEntry) -> None:
         """Run initial setup of the speaker."""
         self.media.play_mode = self.soco.play_mode
         self.update_volume()
@@ -206,53 +228,34 @@ class SonosSpeaker:
         if self.is_coordinator:
             self.media.poll_media()
 
-        future = asyncio.run_coroutine_threadsafe(
-            self.async_setup(entry), self.hass.loop
-        )
-        future.result(timeout=10)
-
-        dispatcher_send(self.hass, SONOS_CREATE_LEVELS, self)
+        dispatches: list[tuple[Any, ...]] = [(SONOS_CREATE_LEVELS, self)]
 
         if audio_format := self.soco.soundbar_audio_input_format:
-            dispatcher_send(
-                self.hass, SONOS_CREATE_AUDIO_FORMAT_SENSOR, self, audio_format
-            )
+            dispatches.append((SONOS_CREATE_AUDIO_FORMAT_SENSOR, self, audio_format))
 
+        has_battery = False
         try:
             self.battery_info = self.fetch_battery_info()
         except SonosUpdateError:
             _LOGGER.debug("No battery available for %s", self.zone_name)
         else:
-            # Battery events can be infrequent, polling is still necessary
-            self._battery_poll_timer = track_time_interval(
-                self.hass, self.async_poll_battery, BATTERY_SCAN_INTERVAL
-            )
+            has_battery = True
             dispatcher_send(self.hass, SONOS_CREATE_BATTERY, self)
 
         if (mic_enabled := self.soco.mic_enabled) is not None:
             self.mic_enabled = mic_enabled
-            dispatcher_send(self.hass, SONOS_CREATE_MIC_SENSOR, self)
+            dispatches.append((SONOS_CREATE_MIC_SENSOR, self))
 
         if new_alarms := [
             alarm.alarm_id for alarm in self.alarms if alarm.zone.uid == self.soco.uid
         ]:
-            dispatcher_send(self.hass, SONOS_CREATE_ALARM, self, new_alarms)
+            dispatches.append((SONOS_CREATE_ALARM, self, new_alarms))
 
-        dispatcher_send(self.hass, SONOS_CREATE_SWITCHES, self)
+        dispatches.append((SONOS_CREATE_SWITCHES, self))
+        dispatches.append((SONOS_CREATE_MEDIA_PLAYER, self))
+        dispatches.append((SONOS_SPEAKER_ADDED, self.soco.uid))
 
-        self._event_dispatchers = {
-            "AlarmClock": self.async_dispatch_alarms,
-            "AVTransport": self.async_dispatch_media_update,
-            "ContentDirectory": self.async_dispatch_favorites,
-            "DeviceProperties": self.async_dispatch_device_properties,
-            "RenderingControl": self.async_update_volume,
-            "ZoneGroupTopology": self.async_update_groups,
-        }
-
-        dispatcher_send(self.hass, SONOS_CREATE_MEDIA_PLAYER, self)
-        dispatcher_send(self.hass, SONOS_SPEAKER_ADDED, self.soco.uid)
-
-        self.hass.create_task(self.async_subscribe())
+        self.hass.create_task(self.async_setup(entry, has_battery, dispatches))
 
     #
     # Entity management
@@ -272,12 +275,12 @@ class SonosSpeaker:
     @property
     def alarms(self) -> SonosAlarms:
         """Return the SonosAlarms instance for this household."""
-        return self.hass.data[DATA_SONOS].alarms[self.household_id]
+        return self.data.alarms[self.household_id]
 
     @property
     def favorites(self) -> SonosFavorites:
         """Return the SonosFavorites instance for this household."""
-        return self.hass.data[DATA_SONOS].favorites[self.household_id]
+        return self.data.favorites[self.household_id]
 
     @property
     def is_coordinator(self) -> bool:
@@ -408,7 +411,9 @@ class SonosSpeaker:
     @callback
     def async_renew_failed(self, exception: Exception) -> None:
         """Handle a failed subscription renewal."""
-        self.hass.async_create_task(self._async_renew_failed(exception))
+        self.hass.async_create_background_task(
+            self._async_renew_failed(exception), "sonos renew failed", eager_start=True
+        )
 
     async def _async_renew_failed(self, exception: Exception) -> None:
         """Mark the speaker as offline after a subscription renewal failure.
@@ -443,20 +448,28 @@ class SonosSpeaker:
         # Save most recently processed event variables for cache and diagnostics
         self._last_event_cache[event.service.service_type] = event.variables
         dispatcher = self._event_dispatchers[event.service.service_type]
-        dispatcher(event)
+        dispatcher(self, event)
 
     @callback
     def async_dispatch_alarms(self, event: SonosEvent) -> None:
         """Add the soco instance associated with the event to the callback."""
         if "alarm_list_version" not in event.variables:
             return
-        self.hass.async_create_task(self.alarms.async_process_event(event, self))
+        self.hass.async_create_background_task(
+            self.alarms.async_process_event(event, self),
+            "sonos process event",
+            eager_start=True,
+        )
 
     @callback
     def async_dispatch_device_properties(self, event: SonosEvent) -> None:
         """Update device properties from an event."""
         self.event_stats.process(event)
-        self.hass.async_create_task(self.async_update_device_properties(event))
+        self.hass.async_create_background_task(
+            self.async_update_device_properties(event),
+            "sonos device properties",
+            eager_start=True,
+        )
 
     async def async_update_device_properties(self, event: SonosEvent) -> None:
         """Update device properties from an event."""
@@ -478,7 +491,11 @@ class SonosSpeaker:
             return
         if "container_update_i_ds" not in event.variables:
             return
-        self.hass.async_create_task(self.favorites.async_process_event(event, self))
+        self.hass.async_create_background_task(
+            self.favorites.async_process_event(event, self),
+            "sonos dispatch favorites",
+            eager_start=True,
+        )
 
     @callback
     def async_dispatch_media_update(self, event: SonosEvent) -> None:
@@ -494,9 +511,7 @@ class SonosSpeaker:
             "x-rincon:"
         ):
             new_coordinator_uid = av_transport_uri.split(":")[-1]
-            if new_coordinator_speaker := self.hass.data[DATA_SONOS].discovered.get(
-                new_coordinator_uid
-            ):
+            if new_coordinator_speaker := self.data.discovered.get(new_coordinator_uid):
                 _LOGGER.debug(
                     "Media update coordinator (%s) received for %s",
                     new_coordinator_speaker.zone_name,
@@ -602,7 +617,7 @@ class SonosSpeaker:
         self.available = True
         if not was_available:
             self.async_write_entity_states()
-            self.hass.async_create_task(self.async_subscribe())
+            self.hass.async_create_task(self.async_subscribe(), eager_start=True)
 
     @callback
     def async_check_activity(self, now: datetime.datetime) -> None:
@@ -613,7 +628,9 @@ class SonosSpeaker:
             return
         # Ensure the ping is canceled at shutdown
         self.hass.async_create_background_task(
-            self._async_check_activity(), f"sonos {self.uid} {self.zone_name} ping"
+            self._async_check_activity(),
+            f"sonos {self.uid} {self.zone_name} ping",
+            eager_start=True,
         )
 
     async def _async_check_activity(self) -> None:
@@ -653,7 +670,7 @@ class SonosSpeaker:
 
         await self.async_unsubscribe()
 
-        self.hass.data[DATA_SONOS].discovery_known.discard(self.soco.uid)
+        self.data.discovery_known.discard(self.soco.uid)
 
     async def async_vanished(self, reason: str) -> None:
         """Handle removal of speaker when marked as vanished."""
@@ -780,15 +797,16 @@ class SonosSpeaker:
         """Update group topology when polling."""
         self.hass.add_job(self.create_update_groups_coro())
 
-    def update_group_for_uid(self, uid: str) -> None:
+    @callback
+    def async_update_group_for_uid(self, uid: str) -> None:
         """Update group topology if uid is missing."""
         if uid not in self._group_members_missing:
             return
-        missing_zone = self.hass.data[DATA_SONOS].discovered[uid].zone_name
+        missing_zone = self.data.discovered[uid].zone_name
         _LOGGER.debug(
             "%s was missing, adding to %s group", missing_zone, self.zone_name
         )
-        self.update_groups()
+        self.hass.async_create_task(self.create_update_groups_coro(), eager_start=True)
 
     @callback
     def async_update_groups(self, event: SonosEvent) -> None:
@@ -812,11 +830,12 @@ class SonosSpeaker:
                     f"{SONOS_VANISHED}-{uid}",
                     reason,
                 )
-
-        if "zone_player_uui_ds_in_group" not in event.variables:
-            return
         self.event_stats.process(event)
-        self.hass.async_create_task(self.create_update_groups_coro(event))
+        self.hass.async_create_background_task(
+            self.create_update_groups_coro(event),
+            name=f"sonos group update {self.zone_name}",
+            eager_start=True,
+        )
 
     def create_update_groups_coro(self, event: SonosEvent | None = None) -> Coroutine:
         """Handle callback for topology change event."""
@@ -835,12 +854,11 @@ class SonosSpeaker:
                         if p.uid != coordinator_uid and p.is_visible
                     ]
 
-            return [coordinator_uid] + joined_uids
+            return [coordinator_uid, *joined_uids]
 
         async def _async_extract_group(event: SonosEvent | None) -> list[str]:
             """Extract group layout from a topology event."""
-            group = event and event.zone_player_uui_ds_in_group
-            if group:
+            if group := (event and getattr(event, "zone_player_uui_ds_in_group", None)):
                 assert isinstance(group, str)
                 return group.split(",")
 
@@ -849,11 +867,21 @@ class SonosSpeaker:
         @callback
         def _async_regroup(group: list[str]) -> None:
             """Rebuild internal group layout."""
+            _LOGGER.debug("async_regroup %s %s", self.zone_name, group)
             if (
                 group == [self.soco.uid]
                 and self.sonos_group == [self]
                 and self.sonos_group_entities
             ):
+                # Single speakers do not have a coodinator, check and clear
+                if self.coordinator is not None:
+                    _LOGGER.debug(
+                        "Zone %s Cleared coordinator [%s]",
+                        self.zone_name,
+                        self.coordinator.zone_name,
+                    )
+                    self.coordinator = None
+                    self.async_write_entity_states()
                 # Skip updating existing single speakers in polling mode
                 return
 
@@ -862,7 +890,7 @@ class SonosSpeaker:
             sonos_group_entities = []
 
             for uid in group:
-                speaker = self.hass.data[DATA_SONOS].discovered.get(uid)
+                speaker = self.data.discovered.get(uid)
                 if speaker:
                     self._group_members_missing.discard(uid)
                     sonos_group.append(speaker)
@@ -890,13 +918,15 @@ class SonosSpeaker:
             self.async_write_entity_states()
 
             for joined_uid in group[1:]:
-                joined_speaker: SonosSpeaker = self.hass.data[
-                    DATA_SONOS
-                ].discovered.get(joined_uid)
-                if joined_speaker:
+                if joined_speaker := self.data.discovered.get(joined_uid):
                     joined_speaker.coordinator = self
                     joined_speaker.sonos_group = sonos_group
                     joined_speaker.sonos_group_entities = sonos_group_entities
+                    _LOGGER.debug(
+                        "Zone %s Set coordinator [%s]",
+                        joined_speaker.zone_name,
+                        self.zone_name,
+                    )
                     joined_speaker.async_write_entity_states()
 
             _LOGGER.debug("Regrouped %s: %s", self.zone_name, self.sonos_group_entities)
@@ -904,13 +934,13 @@ class SonosSpeaker:
         async def _async_handle_group_event(event: SonosEvent | None) -> None:
             """Get async lock and handle event."""
 
-            async with self.hass.data[DATA_SONOS].topology_condition:
+            async with self.data.topology_condition:
                 group = await _async_extract_group(event)
 
                 if self.soco.uid == group[0]:
                     _async_regroup(group)
 
-                    self.hass.data[DATA_SONOS].topology_condition.notify_all()
+                    self.data.topology_condition.notify_all()
 
         return _async_handle_group_event(event)
 
@@ -935,15 +965,16 @@ class SonosSpeaker:
     @staticmethod
     async def join_multi(
         hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
         master: SonosSpeaker,
         speakers: list[SonosSpeaker],
     ) -> None:
         """Form a group with other players."""
-        async with hass.data[DATA_SONOS].topology_condition:
+        async with config_entry.runtime_data.topology_condition:
             group: list[SonosSpeaker] = await hass.async_add_executor_job(
                 master.join, speakers
             )
-            await SonosSpeaker.wait_for_groups(hass, [group])
+            await SonosSpeaker.wait_for_groups(hass, config_entry, [group])
 
     @soco_error()
     def unjoin(self) -> None:
@@ -954,7 +985,11 @@ class SonosSpeaker:
         self.coordinator = None
 
     @staticmethod
-    async def unjoin_multi(hass: HomeAssistant, speakers: list[SonosSpeaker]) -> None:
+    async def unjoin_multi(
+        hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
+        speakers: list[SonosSpeaker],
+    ) -> None:
         """Unjoin several players from their group."""
 
         def _unjoin_all(speakers: list[SonosSpeaker]) -> None:
@@ -966,9 +1001,11 @@ class SonosSpeaker:
             for speaker in joined_speakers + coordinators:
                 speaker.unjoin()
 
-        async with hass.data[DATA_SONOS].topology_condition:
+        async with config_entry.runtime_data.topology_condition:
             await hass.async_add_executor_job(_unjoin_all, speakers)
-            await SonosSpeaker.wait_for_groups(hass, [[s] for s in speakers])
+            await SonosSpeaker.wait_for_groups(
+                hass, config_entry, [[s] for s in speakers]
+            )
 
     @soco_error()
     def snapshot(self, with_group: bool) -> None:
@@ -982,7 +1019,10 @@ class SonosSpeaker:
 
     @staticmethod
     async def snapshot_multi(
-        hass: HomeAssistant, speakers: list[SonosSpeaker], with_group: bool
+        hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
+        speakers: list[SonosSpeaker],
+        with_group: bool,
     ) -> None:
         """Snapshot all the speakers and optionally their groups."""
 
@@ -997,7 +1037,7 @@ class SonosSpeaker:
             for speaker in list(speakers_set):
                 speakers_set.update(speaker.sonos_group)
 
-        async with hass.data[DATA_SONOS].topology_condition:
+        async with config_entry.runtime_data.topology_condition:
             await hass.async_add_executor_job(_snapshot_all, speakers_set)
 
     @soco_error()
@@ -1015,7 +1055,10 @@ class SonosSpeaker:
 
     @staticmethod
     async def restore_multi(
-        hass: HomeAssistant, speakers: list[SonosSpeaker], with_group: bool
+        hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
+        speakers: list[SonosSpeaker],
+        with_group: bool,
     ) -> None:
         """Restore snapshots for all the speakers."""
 
@@ -1093,16 +1136,18 @@ class SonosSpeaker:
                 assert len(speaker.snapshot_group)
                 speakers_set.update(speaker.snapshot_group)
 
-        async with hass.data[DATA_SONOS].topology_condition:
+        async with config_entry.runtime_data.topology_condition:
             groups = await hass.async_add_executor_job(
                 _restore_groups, speakers_set, with_group
             )
-            await SonosSpeaker.wait_for_groups(hass, groups)
+            await SonosSpeaker.wait_for_groups(hass, config_entry, groups)
             await hass.async_add_executor_job(_restore_players, speakers_set)
 
     @staticmethod
     async def wait_for_groups(
-        hass: HomeAssistant, groups: list[list[SonosSpeaker]]
+        hass: HomeAssistant,
+        config_entry: SonosConfigEntry,
+        groups: list[list[SonosSpeaker]],
     ) -> None:
         """Wait until all groups are present, or timeout."""
 
@@ -1125,11 +1170,18 @@ class SonosSpeaker:
         try:
             async with asyncio.timeout(5):
                 while not _test_groups(groups):
-                    await hass.data[DATA_SONOS].topology_condition.wait()
-        except asyncio.TimeoutError:
-            _LOGGER.warning("Timeout waiting for target groups %s", groups)
-
-        any_speaker = next(iter(hass.data[DATA_SONOS].discovered.values()))
+                    await config_entry.runtime_data.topology_condition.wait()
+        except TimeoutError:
+            group_description = [
+                f"{group[0].zone_name}: {', '.join(speaker.zone_name for speaker in group)}"
+                for group in groups
+            ]
+            raise HomeAssistantError(
+                translation_domain=DOMAIN,
+                translation_key="timeout_join",
+                translation_placeholders={"group_description": str(group_description)},
+            ) from TimeoutError
+        any_speaker = next(iter(config_entry.runtime_data.discovered.values()))
         any_speaker.soco.zone_group_state.clear_cache()
 
     #
@@ -1140,3 +1192,12 @@ class SonosSpeaker:
         """Update information about current volume settings."""
         self.volume = self.soco.volume
         self.muted = self.soco.mute
+
+    _event_dispatchers = {
+        "AlarmClock": async_dispatch_alarms,
+        "AVTransport": async_dispatch_media_update,
+        "ContentDirectory": async_dispatch_favorites,
+        "DeviceProperties": async_dispatch_device_properties,
+        "RenderingControl": async_update_volume,
+        "ZoneGroupTopology": async_update_groups,
+    }
