@@ -1,19 +1,28 @@
 """Config flow for Steam integration."""
+
 from __future__ import annotations
 
-from collections.abc import Mapping
+from collections.abc import Iterator, Mapping
 from typing import Any
 
 import steam
 import voluptuous as vol
 
-from homeassistant import config_entries
+from homeassistant.config_entries import (
+    SOURCE_REAUTH,
+    ConfigFlow,
+    ConfigFlowResult,
+    OptionsFlow,
+)
 from homeassistant.const import CONF_API_KEY, Platform
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers import config_validation as cv, entity_registry as er
 
 from .const import CONF_ACCOUNT, CONF_ACCOUNTS, DOMAIN, LOGGER, PLACEHOLDERS
+from .coordinator import SteamConfigEntry
+
+# To avoid too long request URIs, the amount of ids to request is limited
+MAX_IDS_TO_REQUEST = 275
 
 
 def validate_input(user_input: dict[str, str]) -> dict[str, str | int]:
@@ -24,28 +33,24 @@ def validate_input(user_input: dict[str, str]) -> dict[str, str | int]:
     return names["response"]["players"]["player"][0]
 
 
-class SteamFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
+class SteamFlowHandler(ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Steam."""
 
-    def __init__(self) -> None:
-        """Initialize the flow."""
-        self.entry: config_entries.ConfigEntry | None = None
-
     @staticmethod
     @callback
     def async_get_options_flow(
-        config_entry: config_entries.ConfigEntry,
-    ) -> config_entries.OptionsFlow:
+        config_entry: SteamConfigEntry,
+    ) -> SteamOptionsFlowHandler:
         """Get the options flow for this handler."""
         return SteamOptionsFlowHandler(config_entry)
 
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle a flow initiated by the user."""
         errors = {}
-        if user_input is None and self.entry:
-            user_input = {CONF_ACCOUNT: self.entry.data[CONF_ACCOUNT]}
+        if user_input is None and self.source == SOURCE_REAUTH:
+            user_input = {CONF_ACCOUNT: self._get_reauth_entry().data[CONF_ACCOUNT]}
         elif user_input is not None:
             try:
                 res = await self.hass.async_add_executor_job(validate_input, user_input)
@@ -57,12 +62,12 @@ class SteamFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
                 errors["base"] = "cannot_connect"
                 if "403" in str(ex):
                     errors["base"] = "invalid_auth"
-            except Exception as ex:  # pylint:disable=broad-except
+            except Exception as ex:  # noqa: BLE001
                 LOGGER.exception("Unknown exception: %s", ex)
                 errors["base"] = "unknown"
             if not errors:
                 entry = await self.async_set_unique_id(user_input[CONF_ACCOUNT])
-                if entry and self.source == config_entries.SOURCE_REAUTH:
+                if entry and self.source == SOURCE_REAUTH:
                     self.hass.config_entries.async_update_entry(entry, data=user_input)
                     await self.hass.config_entries.async_reload(entry.entry_id)
                     return self.async_abort(reason="reauth_successful")
@@ -89,15 +94,15 @@ class SteamFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             description_placeholders=PLACEHOLDERS,
         )
 
-    async def async_step_reauth(self, entry_data: Mapping[str, Any]) -> FlowResult:
+    async def async_step_reauth(
+        self, entry_data: Mapping[str, Any]
+    ) -> ConfigFlowResult:
         """Handle a reauthorization flow request."""
-        self.entry = self.hass.config_entries.async_get_entry(self.context["entry_id"])
-
         return await self.async_step_reauth_confirm()
 
     async def async_step_reauth_confirm(
         self, user_input: dict[str, str] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Confirm reauth dialog."""
         if user_input is not None:
             return await self.async_step_user()
@@ -108,20 +113,24 @@ class SteamFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
 
-class SteamOptionsFlowHandler(config_entries.OptionsFlow):
+def _batch_ids(ids: list[str]) -> Iterator[list[str]]:
+    for i in range(0, len(ids), MAX_IDS_TO_REQUEST):
+        yield ids[i : i + MAX_IDS_TO_REQUEST]
+
+
+class SteamOptionsFlowHandler(OptionsFlow):
     """Handle Steam client options."""
 
-    def __init__(self, entry: config_entries.ConfigEntry) -> None:
+    def __init__(self, entry: SteamConfigEntry) -> None:
         """Initialize options flow."""
-        self.entry = entry
         self.options = dict(entry.options)
 
     async def async_step_init(
         self, user_input: dict[str, dict[str, str]] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Manage Steam options."""
         if user_input is not None:
-            await self.hass.config_entries.async_unload(self.entry.entry_id)
+            await self.hass.config_entries.async_unload(self.config_entry.entry_id)
             for _id in self.options[CONF_ACCOUNTS]:
                 if _id not in user_input[CONF_ACCOUNTS] and (
                     entity_id := er.async_get(self.hass).async_get_entity_id(
@@ -136,7 +145,7 @@ class SteamOptionsFlowHandler(config_entries.OptionsFlow):
                     if _id in user_input[CONF_ACCOUNTS]
                 }
             }
-            await self.hass.config_entries.async_reload(self.entry.entry_id)
+            await self.hass.config_entries.async_reload(self.config_entry.entry_id)
             return self.async_create_entry(title="", data=channel_data)
         error = None
         try:
@@ -166,9 +175,17 @@ class SteamOptionsFlowHandler(config_entries.OptionsFlow):
         """Get accounts."""
         interface = steam.api.interface("ISteamUser")
         try:
-            friends = interface.GetFriendList(steamid=self.entry.data[CONF_ACCOUNT])
+            friends = interface.GetFriendList(
+                steamid=self.config_entry.data[CONF_ACCOUNT]
+            )
             _users_str = [user["steamid"] for user in friends["friendslist"]["friends"]]
         except steam.api.HTTPError:
             return []
-        names = interface.GetPlayerSummaries(steamids=_users_str)
-        return names["response"]["players"]["player"]
+        names = []
+        for id_batch in _batch_ids(_users_str):
+            names.extend(
+                interface.GetPlayerSummaries(steamids=id_batch)["response"]["players"][
+                    "player"
+                ]
+            )
+        return names
