@@ -1,13 +1,16 @@
 """Config flow to configure the Synology DSM integration."""
+
 from __future__ import annotations
 
 from collections.abc import Mapping
+from contextlib import suppress
 from ipaddress import ip_address as ip
 import logging
-from typing import Any, cast
+from typing import TYPE_CHECKING, Any, cast
 from urllib.parse import urlparse
 
 from synology_dsm import SynologyDSM
+from synology_dsm.api.file_station.models import SynoFileSharedFolder
 from synology_dsm.exceptions import (
     SynologyDSMException,
     SynologyDSMLogin2SAFailedException,
@@ -17,9 +20,12 @@ from synology_dsm.exceptions import (
 )
 import voluptuous as vol
 
-from homeassistant import exceptions
-from homeassistant.components import ssdp, zeroconf
-from homeassistant.config_entries import ConfigEntry, ConfigFlow, OptionsFlow
+from homeassistant.config_entries import (
+    ConfigEntry,
+    ConfigFlow,
+    ConfigFlowResult,
+    OptionsFlow,
+)
 from homeassistant.const import (
     CONF_DISKS,
     CONF_HOST,
@@ -27,32 +33,46 @@ from homeassistant.const import (
     CONF_NAME,
     CONF_PASSWORD,
     CONF_PORT,
-    CONF_SCAN_INTERVAL,
     CONF_SSL,
-    CONF_TIMEOUT,
     CONF_USERNAME,
     CONF_VERIFY_SSL,
 )
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
+from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.typing import DiscoveryInfoType
+from homeassistant.helpers.selector import (
+    SelectOptionDict,
+    SelectSelector,
+    SelectSelectorConfig,
+    SelectSelectorMode,
+)
+from homeassistant.helpers.service_info.ssdp import (
+    ATTR_UPNP_FRIENDLY_NAME,
+    ATTR_UPNP_SERIAL,
+    SsdpServiceInfo,
+)
+from homeassistant.helpers.service_info.zeroconf import ZeroconfServiceInfo
+from homeassistant.helpers.typing import DiscoveryInfoType, VolDictType
+from homeassistant.util import slugify
 from homeassistant.util.network import is_ip_address as is_ip
 
 from .const import (
+    CONF_BACKUP_PATH,
+    CONF_BACKUP_SHARE,
     CONF_DEVICE_TOKEN,
     CONF_SNAPSHOT_QUALITY,
     CONF_VOLUMES,
+    DEFAULT_BACKUP_PATH,
     DEFAULT_PORT,
     DEFAULT_PORT_SSL,
-    DEFAULT_SCAN_INTERVAL,
     DEFAULT_SNAPSHOT_QUALITY,
     DEFAULT_TIMEOUT,
     DEFAULT_USE_SSL,
     DEFAULT_VERIFY_SSL,
     DOMAIN,
+    SYNOLOGY_CONNECTION_EXCEPTIONS,
 )
+from .coordinator import SynologyDSMConfigEntry
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -75,7 +95,7 @@ def _reauth_schema() -> vol.Schema:
 
 
 def _user_schema_with_defaults(user_input: dict[str, Any]) -> vol.Schema:
-    user_schema = {
+    user_schema: VolDictType = {
         vol.Required(CONF_HOST, default=user_input.get(CONF_HOST, "")): str,
     }
     user_schema.update(_ordered_shared_schema(user_input))
@@ -83,9 +103,7 @@ def _user_schema_with_defaults(user_input: dict[str, Any]) -> vol.Schema:
     return vol.Schema(user_schema)
 
 
-def _ordered_shared_schema(
-    schema_input: dict[str, Any],
-) -> dict[vol.Required | vol.Optional, Any]:
+def _ordered_shared_schema(schema_input: dict[str, Any]) -> VolDictType:
     return {
         vol.Required(CONF_USERNAME, default=schema_input.get(CONF_USERNAME, "")): str,
         vol.Required(CONF_PASSWORD, default=schema_input.get(CONF_PASSWORD, "")): str,
@@ -113,10 +131,10 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
     @staticmethod
     @callback
     def async_get_options_flow(
-        config_entry: ConfigEntry,
+        config_entry: SynologyDSMConfigEntry,
     ) -> SynologyDSMOptionsFlowHandler:
         """Get the options flow for this handler."""
-        return SynologyDSMOptionsFlowHandler(config_entry)
+        return SynologyDSMOptionsFlowHandler()
 
     def __init__(self) -> None:
         """Initialize the synology_dsm config flow."""
@@ -124,19 +142,20 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         self.discovered_conf: dict[str, Any] = {}
         self.reauth_conf: Mapping[str, Any] = {}
         self.reauth_reason: str | None = None
+        self.shares: list[SynoFileSharedFolder] | None = None
 
     def _show_form(
         self,
         step_id: str,
         user_input: dict[str, Any] | None = None,
         errors: dict[str, str] | None = None,
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Show the setup form to the user."""
         if not user_input:
             user_input = {}
 
         description_placeholders = {}
-        data_schema = {}
+        data_schema = None
 
         if step_id == "link":
             user_input.update(self.discovered_conf)
@@ -156,7 +175,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
     async def async_validate_input_create_entry(
         self, user_input: dict[str, Any], step_id: str
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Process user input and create new or update existing config entry."""
         host = user_input[CONF_HOST]
         port = user_input.get(CONF_PORT)
@@ -166,6 +185,8 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         verify_ssl = user_input.get(CONF_VERIFY_SSL, DEFAULT_VERIFY_SSL)
         otp_code = user_input.get(CONF_OTP_CODE)
         friendly_name = user_input.get(CONF_NAME)
+        backup_path = user_input.get(CONF_BACKUP_PATH)
+        backup_share = user_input.get(CONF_BACKUP_SHARE)
 
         if not port:
             if use_ssl is True:
@@ -174,7 +195,9 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
                 port = DEFAULT_PORT
 
         session = async_get_clientsession(self.hass, verify_ssl)
-        api = SynologyDSM(session, host, port, username, password, use_ssl, timeout=30)
+        api = SynologyDSM(
+            session, host, port, username, password, use_ssl, timeout=DEFAULT_TIMEOUT
+        )
 
         errors = {}
         try:
@@ -200,6 +223,12 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         if errors:
             return self._show_form(step_id, user_input, errors)
 
+        with suppress(*SYNOLOGY_CONNECTION_EXCEPTIONS):
+            self.shares = await api.file.get_shared_folders(only_writable=True)
+
+        if self.shares and not backup_path:
+            return await self.async_step_backup_share(user_input)
+
         # unique_id should be serial for services purpose
         existing_entry = await self.async_set_unique_id(serial, raise_on_progress=False)
 
@@ -212,6 +241,10 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
             CONF_PASSWORD: password,
             CONF_MAC: api.network.macs,
         }
+        config_options = {
+            CONF_BACKUP_PATH: backup_path,
+            CONF_BACKUP_SHARE: backup_share,
+        }
         if otp_code:
             config_data[CONF_DEVICE_TOKEN] = api.device_token
         if user_input.get(CONF_DISKS):
@@ -224,14 +257,16 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
                 "reauth_successful" if self.reauth_conf else "reconfigure_successful"
             )
             return self.async_update_reload_and_abort(
-                existing_entry, data=config_data, reason=reason
+                existing_entry, data=config_data, options=config_options, reason=reason
             )
 
-        return self.async_create_entry(title=friendly_name or host, data=config_data)
+        return self.async_create_entry(
+            title=friendly_name or host, data=config_data, options=config_options
+        )
 
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle a flow initiated by the user."""
         step = "user"
         if not user_input:
@@ -239,8 +274,8 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         return await self.async_validate_input_create_entry(user_input, step_id=step)
 
     async def async_step_zeroconf(
-        self, discovery_info: zeroconf.ZeroconfServiceInfo
-    ) -> FlowResult:
+        self, discovery_info: ZeroconfServiceInfo
+    ) -> ConfigFlowResult:
         """Handle a discovered synology_dsm via zeroconf."""
         discovered_macs = [
             format_synology_mac(mac)
@@ -253,12 +288,14 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         friendly_name = discovery_info.name.removesuffix(HTTP_SUFFIX)
         return await self._async_from_discovery(host, friendly_name, discovered_macs)
 
-    async def async_step_ssdp(self, discovery_info: ssdp.SsdpServiceInfo) -> FlowResult:
+    async def async_step_ssdp(
+        self, discovery_info: SsdpServiceInfo
+    ) -> ConfigFlowResult:
         """Handle a discovered synology_dsm via ssdp."""
         parsed_url = urlparse(discovery_info.ssdp_location)
-        upnp_friendly_name: str = discovery_info.upnp[ssdp.ATTR_UPNP_FRIENDLY_NAME]
+        upnp_friendly_name: str = discovery_info.upnp[ATTR_UPNP_FRIENDLY_NAME]
         friendly_name = upnp_friendly_name.split("(", 1)[0].strip()
-        mac_address = discovery_info.upnp[ssdp.ATTR_UPNP_SERIAL]
+        mac_address = discovery_info.upnp[ATTR_UPNP_SERIAL]
         discovered_macs = [format_synology_mac(mac_address)]
         # Synology NAS can broadcast on multiple IP addresses, since they can be connected to multiple ethernets.
         # The serial of the NAS is actually its MAC address.
@@ -267,7 +304,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
     async def _async_from_discovery(
         self, host: str, friendly_name: str, discovered_macs: list[str]
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle a discovered synology_dsm via zeroconf or ssdp."""
         existing_entry = None
         for discovered_mac in discovered_macs:
@@ -283,7 +320,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
             and existing_entry.data[CONF_HOST] != host
             and ip(existing_entry.data[CONF_HOST]).version == ip(host).version
         ):
-            _LOGGER.info(
+            _LOGGER.debug(
                 "Update host from '%s' to '%s' for NAS '%s' via discovery",
                 existing_entry.data[CONF_HOST],
                 host,
@@ -307,7 +344,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
     async def async_step_link(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Link a config entry from discovery."""
         step = "link"
         if not user_input:
@@ -315,16 +352,22 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         user_input = {**self.discovered_conf, **user_input}
         return await self.async_validate_input_create_entry(user_input, step_id=step)
 
-    async def async_step_reauth(self, entry_data: Mapping[str, Any]) -> FlowResult:
+    async def async_step_reauth(
+        self, entry_data: Mapping[str, Any]
+    ) -> ConfigFlowResult:
         """Perform reauth upon an API authentication error."""
         self.reauth_conf = entry_data
-        self.context["title_placeholders"][CONF_HOST] = entry_data[CONF_HOST]
+        placeholders = {
+            **self.context["title_placeholders"],
+            CONF_HOST: entry_data[CONF_HOST],
+        }
+        self.context["title_placeholders"] = placeholders
 
         return await self.async_step_reauth_confirm()
 
     async def async_step_reauth_confirm(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Perform reauth confirm upon an API authentication error."""
         step = "reauth_confirm"
         if not user_input:
@@ -334,7 +377,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
     async def async_step_2sa(
         self, user_input: dict[str, Any], errors: dict[str, str] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Enter 2SA code to anthenticate."""
         if not self.saved_user_input:
             self.saved_user_input = user_input
@@ -351,6 +394,43 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
         return await self.async_step_user(user_input)
 
+    async def async_step_backup_share(
+        self, user_input: dict[str, Any], errors: dict[str, str] | None = None
+    ) -> ConfigFlowResult:
+        """Select backup location."""
+        if TYPE_CHECKING:
+            assert self.shares is not None
+
+        if not self.saved_user_input:
+            self.saved_user_input = user_input
+
+        if CONF_BACKUP_PATH not in user_input and CONF_BACKUP_SHARE not in user_input:
+            return self.async_show_form(
+                step_id="backup_share",
+                data_schema=vol.Schema(
+                    {
+                        vol.Required(CONF_BACKUP_SHARE): SelectSelector(
+                            SelectSelectorConfig(
+                                options=[
+                                    SelectOptionDict(value=s.path, label=s.name)
+                                    for s in self.shares
+                                ],
+                                mode=SelectSelectorMode.DROPDOWN,
+                            ),
+                        ),
+                        vol.Required(
+                            CONF_BACKUP_PATH,
+                            default=f"{DEFAULT_BACKUP_PATH}_{slugify(self.hass.config.location_name)}",
+                        ): str,
+                    }
+                ),
+            )
+
+        user_input = {**self.saved_user_input, **user_input}
+        self.saved_user_input = {}
+
+        return await self.async_step_user(user_input)
+
     def _async_get_existing_entry(self, discovered_mac: str) -> ConfigEntry | None:
         """See if we already have a configured NAS with this MAC address."""
         for entry in self._async_current_entries():
@@ -364,31 +444,19 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 class SynologyDSMOptionsFlowHandler(OptionsFlow):
     """Handle a option flow."""
 
-    def __init__(self, config_entry: ConfigEntry) -> None:
-        """Initialize options flow."""
-        self.config_entry = config_entry
+    config_entry: SynologyDSMConfigEntry
 
     async def async_step_init(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle options flow."""
         if user_input is not None:
             return self.async_create_entry(title="", data=user_input)
 
+        syno_data = self.config_entry.runtime_data
+
         data_schema = vol.Schema(
             {
-                vol.Required(
-                    CONF_SCAN_INTERVAL,
-                    default=self.config_entry.options.get(
-                        CONF_SCAN_INTERVAL, DEFAULT_SCAN_INTERVAL
-                    ),
-                ): cv.positive_int,
-                vol.Required(
-                    CONF_TIMEOUT,
-                    default=self.config_entry.options.get(
-                        CONF_TIMEOUT, DEFAULT_TIMEOUT
-                    ),
-                ): cv.positive_int,
                 vol.Required(
                     CONF_SNAPSHOT_QUALITY,
                     default=self.config_entry.options.get(
@@ -397,6 +465,36 @@ class SynologyDSMOptionsFlowHandler(OptionsFlow):
                 ): vol.All(vol.Coerce(int), vol.Range(min=0, max=2)),
             }
         )
+
+        shares: list[SynoFileSharedFolder] | None = None
+        if syno_data.api.file_station:
+            with suppress(*SYNOLOGY_CONNECTION_EXCEPTIONS):
+                shares = await syno_data.api.file_station.get_shared_folders(
+                    only_writable=True
+                )
+
+        if shares:
+            data_schema = data_schema.extend(
+                {
+                    vol.Required(
+                        CONF_BACKUP_SHARE,
+                        default=self.config_entry.options[CONF_BACKUP_SHARE],
+                    ): SelectSelector(
+                        SelectSelectorConfig(
+                            options=[
+                                SelectOptionDict(value=s.path, label=s.name)
+                                for s in shares
+                            ],
+                            mode=SelectSelectorMode.DROPDOWN,
+                        ),
+                    ),
+                    vol.Required(
+                        CONF_BACKUP_PATH,
+                        default=self.config_entry.options[CONF_BACKUP_PATH],
+                    ): str,
+                }
+            )
+
         return self.async_show_form(step_id="init", data_schema=data_schema)
 
 
@@ -416,8 +514,8 @@ async def _login_and_fetch_syno_info(api: SynologyDSM, otp_code: str | None) ->
     ):
         raise InvalidData
 
-    return api.information.serial  # type: ignore[no-any-return]
+    return api.information.serial
 
 
-class InvalidData(exceptions.HomeAssistantError):
+class InvalidData(HomeAssistantError):
     """Error to indicate we get invalid data from the nas."""
