@@ -10,11 +10,11 @@ from pytradfri.command import Command
 from homeassistant.components.fan import FanEntity, FanEntityFeature
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
-from .base_class import TradfriBaseEntity
 from .const import CONF_GATEWAY_ID, COORDINATOR, COORDINATOR_LIST, DOMAIN, KEY_API
 from .coordinator import TradfriDeviceDataUpdateCoordinator
+from .entity import TradfriBaseEntity
 
 ATTR_AUTO = "Auto"
 ATTR_MAX_FAN_STEPS = 49
@@ -33,7 +33,7 @@ def _from_fan_speed(fan_speed: int) -> int:
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Load Tradfri switches based on a config entry."""
     gateway_id = config_entry.data[CONF_GATEWAY_ID]
@@ -54,7 +54,21 @@ async def async_setup_entry(
 class TradfriAirPurifierFan(TradfriBaseEntity, FanEntity):
     """The platform class required by Home Assistant."""
 
-    _attr_supported_features = FanEntityFeature.PRESET_MODE | FanEntityFeature.SET_SPEED
+    _attr_name = None
+    _attr_supported_features = (
+        FanEntityFeature.PRESET_MODE
+        | FanEntityFeature.SET_SPEED
+        | FanEntityFeature.TURN_ON
+        | FanEntityFeature.TURN_OFF
+    )
+    _attr_preset_modes = [ATTR_AUTO]
+    # These are the steps:
+    # 0 = Off
+    # 1 = Preset: Auto mode
+    # 2 = Min
+    # ... with step size 1
+    # 50 = Max
+    _attr_speed_count = ATTR_MAX_FAN_STEPS
 
     def __init__(
         self,
@@ -76,19 +90,6 @@ class TradfriAirPurifierFan(TradfriBaseEntity, FanEntity):
         """Refresh the device."""
         self._device_data = self.coordinator.data.air_purifier_control.air_purifiers[0]
 
-    @property
-    def speed_count(self) -> int:
-        """Return the number of speeds the fan supports.
-
-        These are the steps:
-        0 = Off
-        1 = Preset: Auto mode
-        2 = Min
-        ... with step size 1
-        50 = Max
-        """
-        return ATTR_MAX_FAN_STEPS
-
     @property
     def is_on(self) -> bool:
         """Return true if switch is on."""
@@ -96,11 +97,6 @@ class TradfriAirPurifierFan(TradfriBaseEntity, FanEntity):
             return False
         return cast(bool, self._device_data.state)
 
-    @property
-    def preset_modes(self) -> list[str] | None:
-        """Return a list of available preset modes."""
-        return [ATTR_AUTO]
-
     @property
     def percentage(self) -> int | None:
         """Return the current speed percentage."""
@@ -128,8 +124,7 @@ class TradfriAirPurifierFan(TradfriBaseEntity, FanEntity):
         if not self._device_control:
             return
 
-        if not preset_mode == ATTR_AUTO:
-            raise ValueError("Preset must be 'Auto'.")
+        # Preset must be 'Auto'
 
         await self._api(self._device_control.turn_on_auto_mode())
 
