@@ -1,23 +1,22 @@
 """Diagnostics support for UptimeRobot."""
+
 from __future__ import annotations
 
 from typing import Any
 
 from pyuptimerobot import UptimeRobotException
 
-from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
 
-from . import UptimeRobotDataUpdateCoordinator
-from .const import DOMAIN
+from .coordinator import UptimeRobotConfigEntry
 
 
 async def async_get_config_entry_diagnostics(
     hass: HomeAssistant,
-    entry: ConfigEntry,
+    entry: UptimeRobotConfigEntry,
 ) -> dict[str, Any]:
     """Return diagnostics for a config entry."""
-    coordinator: UptimeRobotDataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator = entry.runtime_data
     account: dict[str, Any] | str | None = None
     try:
         response = await coordinator.api.async_get_account_details()
