@@ -1,8 +1,8 @@
 """Commands part of Websocket API."""
+
 from __future__ import annotations
 
 from collections.abc import Callable
-import datetime as dt
 from functools import lru_cache, partial
 import json
 import logging
@@ -21,6 +21,7 @@ from homeassistant.const import (
 from homeassistant.core import (
     Context,
     Event,
+    EventStateChangedData,
     HomeAssistant,
     ServiceResponse,
     State,
@@ -35,8 +36,11 @@ from homeassistant.exceptions import (
 )
 from homeassistant.helpers import config_validation as cv, entity, template
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entityfilter import (
+    INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA,
+    convert_include_exclude_filter,
+)
 from homeassistant.helpers.event import (
-    EventStateChangedData,
     TrackTemplate,
     TrackTemplateResult,
     async_track_template_result,
@@ -46,24 +50,34 @@ from homeassistant.helpers.json import (
     ExtendedJSONEncoder,
     find_paths_unserializable_data,
     json_bytes,
+    json_fragment,
+)
+from homeassistant.helpers.service import (
+    async_get_all_descriptions as async_get_all_service_descriptions,
+)
+from homeassistant.helpers.trigger import (
+    async_get_all_descriptions as async_get_all_trigger_descriptions,
+    async_subscribe_platform_events as async_subscribe_trigger_platform_events,
 )
-from homeassistant.helpers.service import async_get_all_descriptions
-from homeassistant.helpers.typing import EventType
 from homeassistant.loader import (
-    Integration,
     IntegrationNotFound,
     async_get_integration,
     async_get_integration_descriptions,
     async_get_integrations,
 )
-from homeassistant.setup import DATA_SETUP_TIME, async_get_loaded_integrations
+from homeassistant.setup import (
+    async_get_loaded_integrations,
+    async_get_setup_timings,
+    async_wait_component,
+)
 from homeassistant.util.json import format_unserializable_data
 
 from . import const, decorators, messages
 from .connection import ActiveConnection
-from .messages import construct_result_message
+from .messages import construct_event_message, construct_result_message
 
 ALL_SERVICE_DESCRIPTIONS_JSON_CACHE = "websocket_api_all_service_descriptions_json"
+ALL_TRIGGER_DESCRIPTIONS_JSON_CACHE = "websocket_api_all_trigger_descriptions_json"
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -89,12 +103,14 @@ def async_register_commands(
     async_reg(hass, handle_subscribe_bootstrap_integrations)
     async_reg(hass, handle_subscribe_events)
     async_reg(hass, handle_subscribe_trigger)
+    async_reg(hass, handle_subscribe_trigger_platforms)
     async_reg(hass, handle_test_condition)
     async_reg(hass, handle_unsubscribe_events)
     async_reg(hass, handle_validate_config)
     async_reg(hass, handle_subscribe_entities)
     async_reg(hass, handle_supported_features)
     async_reg(hass, handle_integration_descriptions)
+    async_reg(hass, handle_integration_wait)
 
 
 def pong_message(iden: int) -> dict[str, Any]:
@@ -104,9 +120,9 @@ def pong_message(iden: int) -> dict[str, Any]:
 
 @callback
 def _forward_events_check_permissions(
-    send_message: Callable[[bytes | str | dict[str, Any] | Callable[[], str]], None],
+    send_message: Callable[[bytes | str | dict[str, Any]], None],
     user: User,
-    msg_id: int,
+    message_id_as_bytes: bytes,
     event: Event,
 ) -> None:
     """Forward state changed events to websocket."""
@@ -119,17 +135,17 @@ def _forward_events_check_permissions(
         and not permissions.check_entity(event.data["entity_id"], POLICY_READ)
     ):
         return
-    send_message(messages.cached_event_message(msg_id, event))
+    send_message(messages.cached_event_message(message_id_as_bytes, event))
 
 
 @callback
 def _forward_events_unconditional(
-    send_message: Callable[[bytes | str | dict[str, Any] | Callable[[], str]], None],
-    msg_id: int,
+    send_message: Callable[[bytes | str | dict[str, Any]], None],
+    message_id_as_bytes: bytes,
     event: Event,
 ) -> None:
     """Forward events to websocket."""
-    send_message(messages.cached_event_message(msg_id, event))
+    send_message(messages.cached_event_message(message_id_as_bytes, event))
 
 
 @callback
@@ -153,20 +169,22 @@ def handle_subscribe_events(
         )
         raise Unauthorized(user_id=connection.user.id)
 
+    message_id_as_bytes = str(msg["id"]).encode()
+
     if event_type == EVENT_STATE_CHANGED:
         forward_events = partial(
             _forward_events_check_permissions,
             connection.send_message,
             connection.user,
-            msg["id"],
+            message_id_as_bytes,
         )
     else:
         forward_events = partial(
-            _forward_events_unconditional, connection.send_message, msg["id"]
+            _forward_events_unconditional, connection.send_message, message_id_as_bytes
         )
 
     connection.subscriptions[msg["id"]] = hass.bus.async_listen(
-        event_type, forward_events, run_immediately=True
+        event_type, forward_events
     )
 
     connection.send_result(msg["id"])
@@ -270,10 +288,10 @@ async def handle_call_service(
                 translation_domain=const.DOMAIN,
                 translation_key="child_service_not_found",
                 translation_placeholders={
-                    "domain": err.domain,
-                    "service": err.service,
-                    "child_domain": msg["domain"],
-                    "child_service": msg["service"],
+                    "domain": msg["domain"],
+                    "service": msg["service"],
+                    "child_domain": err.domain,
+                    "child_service": err.service,
                 },
             )
     except vol.Invalid as err:
@@ -290,7 +308,9 @@ async def handle_call_service(
             translation_placeholders=err.translation_placeholders,
         )
     except HomeAssistantError as err:
-        connection.logger.exception(err)
+        connection.logger.error(
+            "Error during service call to %s.%s: %s", msg["domain"], msg["service"], err
+        )
         connection.send_error(
             msg["id"],
             const.ERR_HOME_ASSISTANT_ERROR,
@@ -299,8 +319,8 @@ async def handle_call_service(
             translation_key=err.translation_key,
             translation_placeholders=err.translation_placeholders,
         )
-    except Exception as err:  # pylint: disable=broad-except
-        connection.logger.exception(err)
+    except Exception as err:
+        connection.logger.exception("Unexpected exception")
         connection.send_error(msg["id"], const.ERR_UNKNOWN_ERROR, str(err))
 
 
@@ -356,21 +376,26 @@ def _send_handle_get_states_response(
 ) -> None:
     """Send handle get states response."""
     connection.send_message(
-        construct_result_message(msg_id, b"[" + b",".join(serialized_states) + b"]")
+        construct_result_message(
+            msg_id, b"".join((b"[", b",".join(serialized_states), b"]"))
+        )
     )
 
 
 @callback
 def _forward_entity_changes(
-    send_message: Callable[[str | bytes | dict[str, Any] | Callable[[], str]], None],
-    entity_ids: set[str],
+    send_message: Callable[[str | bytes | dict[str, Any]], None],
+    entity_ids: set[str] | None,
+    entity_filter: Callable[[str], bool] | None,
     user: User,
-    msg_id: int,
-    event: Event,
+    message_id_as_bytes: bytes,
+    event: Event[EventStateChangedData],
 ) -> None:
     """Forward entity state changed events to websocket."""
     entity_id = event.data["entity_id"]
-    if entity_ids and entity_id not in entity_ids:
+    if (entity_ids and entity_id not in entity_ids) or (
+        entity_filter and not entity_filter(entity_id)
+    ):
         return
     # We have to lookup the permissions again because the user might have
     # changed since the subscription was created.
@@ -378,10 +403,10 @@ def _forward_entity_changes(
     if (
         not user.is_admin
         and not permissions.access_all_entities(POLICY_READ)
-        and not permissions.check_entity(event.data["entity_id"], POLICY_READ)
+        and not permissions.check_entity(entity_id, POLICY_READ)
     ):
         return
-    send_message(messages.cached_state_diff_message(msg_id, event))
+    send_message(messages.cached_state_diff_message(message_id_as_bytes, event))
 
 
 @callback
@@ -389,43 +414,55 @@ def _forward_entity_changes(
     {
         vol.Required("type"): "subscribe_entities",
         vol.Optional("entity_ids"): cv.entity_ids,
+        **INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA.schema,
     }
 )
 def handle_subscribe_entities(
     hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
 ) -> None:
     """Handle subscribe entities command."""
-    entity_ids = set(msg.get("entity_ids", []))
+    entity_ids = set(msg.get("entity_ids", [])) or None
+    _filter = convert_include_exclude_filter(msg)
+    entity_filter = None if _filter.empty_filter else _filter.get_filter()
     # We must never await between sending the states and listening for
     # state changed events or we will introduce a race condition
     # where some states are missed
     states = _async_get_allowed_states(hass, connection)
-    connection.subscriptions[msg["id"]] = hass.bus.async_listen(
+    msg_id = msg["id"]
+    message_id_as_bytes = str(msg_id).encode()
+    connection.subscriptions[msg_id] = hass.bus.async_listen(
         EVENT_STATE_CHANGED,
         partial(
             _forward_entity_changes,
             connection.send_message,
             entity_ids,
+            entity_filter,
             connection.user,
-            msg["id"],
+            message_id_as_bytes,
         ),
-        run_immediately=True,
     )
-    connection.send_result(msg["id"])
+    connection.send_result(msg_id)
 
     # JSON serialize here so we can recover if it blows up due to the
     # state machine containing unserializable data. This command is required
     # to succeed for the UI to show.
     try:
-        serialized_states = [
-            state.as_compressed_state_json
-            for state in states
-            if not entity_ids or state.entity_id in entity_ids
-        ]
+        if entity_ids or entity_filter:
+            serialized_states = [
+                state.as_compressed_state_json
+                for state in states
+                if (not entity_ids or state.entity_id in entity_ids)
+                and (not entity_filter or entity_filter(state.entity_id))
+            ]
+        else:
+            # Fast path when not filtering
+            serialized_states = [state.as_compressed_state_json for state in states]
     except (ValueError, TypeError):
         pass
     else:
-        _send_handle_entities_init_response(connection, msg["id"], serialized_states)
+        _send_handle_entities_init_response(
+            connection, message_id_as_bytes, serialized_states
+        )
         return
 
     serialized_states = []
@@ -440,18 +477,22 @@ def handle_subscribe_entities(
                 ),
             )
 
-    _send_handle_entities_init_response(connection, msg["id"], serialized_states)
+    _send_handle_entities_init_response(
+        connection, message_id_as_bytes, serialized_states
+    )
 
 
 def _send_handle_entities_init_response(
-    connection: ActiveConnection, msg_id: int, serialized_states: list[bytes]
+    connection: ActiveConnection,
+    message_id_as_bytes: bytes,
+    serialized_states: list[bytes],
 ) -> None:
     """Send handle entities init response."""
     connection.send_message(
         b"".join(
             (
                 b'{"id":',
-                str(msg_id).encode(),
+                message_id_as_bytes,
                 b',"type":"event","event":{"a":{',
                 b",".join(serialized_states),
                 b"}}}",
@@ -460,9 +501,9 @@ def _send_handle_entities_init_response(
     )
 
 
-async def _async_get_all_descriptions_json(hass: HomeAssistant) -> bytes:
+async def _async_get_all_service_descriptions_json(hass: HomeAssistant) -> bytes:
     """Return JSON of descriptions (i.e. user documentation) for all service calls."""
-    descriptions = await async_get_all_descriptions(hass)
+    descriptions = await async_get_all_service_descriptions(hass)
     if ALL_SERVICE_DESCRIPTIONS_JSON_CACHE in hass.data:
         cached_descriptions, cached_json_payload = hass.data[
             ALL_SERVICE_DESCRIPTIONS_JSON_CACHE
@@ -481,10 +522,57 @@ async def handle_get_services(
     hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
 ) -> None:
     """Handle get services command."""
-    payload = await _async_get_all_descriptions_json(hass)
+    payload = await _async_get_all_service_descriptions_json(hass)
     connection.send_message(construct_result_message(msg["id"], payload))
 
 
+async def _async_get_all_trigger_descriptions_json(hass: HomeAssistant) -> bytes:
+    """Return JSON of descriptions (i.e. user documentation) for all triggers."""
+    descriptions = await async_get_all_trigger_descriptions(hass)
+    if ALL_TRIGGER_DESCRIPTIONS_JSON_CACHE in hass.data:
+        cached_descriptions, cached_json_payload = hass.data[
+            ALL_TRIGGER_DESCRIPTIONS_JSON_CACHE
+        ]
+        # If the descriptions are the same, return the cached JSON payload
+        if cached_descriptions is descriptions:
+            return cast(bytes, cached_json_payload)
+    json_payload = json_bytes(
+        {
+            trigger: description
+            for trigger, description in descriptions.items()
+            if description is not None
+        }
+    )
+    hass.data[ALL_TRIGGER_DESCRIPTIONS_JSON_CACHE] = (descriptions, json_payload)
+    return json_payload
+
+
+@decorators.websocket_command({vol.Required("type"): "trigger_platforms/subscribe"})
+@decorators.async_response
+async def handle_subscribe_trigger_platforms(
+    hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
+) -> None:
+    """Handle subscribe triggers command."""
+
+    async def on_new_triggers(new_triggers: set[str]) -> None:
+        """Forward new triggers to websocket."""
+        descriptions = await async_get_all_trigger_descriptions(hass)
+        new_trigger_descriptions = {}
+        for trigger in new_triggers:
+            if (description := descriptions[trigger]) is not None:
+                new_trigger_descriptions[trigger] = description
+        if not new_trigger_descriptions:
+            return
+        connection.send_event(msg["id"], new_trigger_descriptions)
+
+    connection.subscriptions[msg["id"]] = async_subscribe_trigger_platform_events(
+        hass, on_new_triggers
+    )
+    connection.send_result(msg["id"])
+    triggers_json = await _async_get_all_trigger_descriptions_json(hass)
+    connection.send_message(construct_event_message(msg["id"], triggers_json))
+
+
 @callback
 @decorators.websocket_command({vol.Required("type"): "get_config"})
 def handle_get_config(
@@ -502,19 +590,15 @@ async def handle_manifest_list(
     hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
 ) -> None:
     """Handle integrations command."""
-    wanted_integrations = msg.get("integrations")
-    if wanted_integrations is None:
-        wanted_integrations = async_get_loaded_integrations(hass)
-
-    ints_or_excs = await async_get_integrations(hass, wanted_integrations)
-    integrations: list[Integration] = []
+    ints_or_excs = await async_get_integrations(
+        hass, msg.get("integrations") or async_get_loaded_integrations(hass)
+    )
+    manifest_json_fragments: list[json_fragment] = []
     for int_or_exc in ints_or_excs.values():
         if isinstance(int_or_exc, Exception):
             raise int_or_exc
-        integrations.append(int_or_exc)
-    connection.send_result(
-        msg["id"], [integration.manifest for integration in integrations]
-    )
+        manifest_json_fragments.append(int_or_exc.manifest_json_fragment)
+    connection.send_result(msg["id"], manifest_json_fragments)
 
 
 @decorators.websocket_command(
@@ -527,9 +611,10 @@ async def handle_manifest_get(
     """Handle integrations command."""
     try:
         integration = await async_get_integration(hass, msg["integration"])
-        connection.send_result(msg["id"], integration.manifest)
     except IntegrationNotFound:
         connection.send_error(msg["id"], const.ERR_NOT_FOUND, "Integration not found")
+    else:
+        connection.send_result(msg["id"], integration.manifest_json_fragment)
 
 
 @callback
@@ -541,10 +626,8 @@ def handle_integration_setup_info(
     connection.send_result(
         msg["id"],
         [
-            {"domain": integration, "seconds": timedelta.total_seconds()}
-            for integration, timedelta in cast(
-                dict[str, dt.timedelta], hass.data[DATA_SETUP_TIME]
-            ).items()
+            {"domain": integration, "seconds": seconds}
+            for integration, seconds in async_get_setup_timings(hass).items()
         ],
     )
 
@@ -621,7 +704,7 @@ async def handle_render_template(
 
     @callback
     def _template_listener(
-        event: EventType[EventStateChangedData] | None,
+        event: Event[EventStateChangedData] | None,
         updates: list[TrackTemplateResult],
     ) -> None:
         track_template_result = updates.pop()
@@ -707,8 +790,7 @@ async def handle_subscribe_trigger(
 ) -> None:
     """Handle subscribe trigger command."""
     # Circular dep
-    # pylint: disable-next=import-outside-toplevel
-    from homeassistant.helpers import trigger
+    from homeassistant.helpers import trigger  # noqa: PLC0415
 
     trigger_config = await trigger.async_validate_trigger_config(hass, msg["trigger"])
 
@@ -758,8 +840,7 @@ async def handle_test_condition(
 ) -> None:
     """Handle test condition command."""
     # Circular dep
-    # pylint: disable-next=import-outside-toplevel
-    from homeassistant.helpers import condition
+    from homeassistant.helpers import condition  # noqa: PLC0415
 
     # Do static + dynamic validation of the condition
     config = await condition.async_validate_condition_config(hass, msg["condition"])
@@ -784,8 +865,10 @@ async def handle_execute_script(
 ) -> None:
     """Handle execute script command."""
     # Circular dep
-    # pylint: disable-next=import-outside-toplevel
-    from homeassistant.helpers.script import Script, async_validate_actions_config
+    from homeassistant.helpers.script import (  # noqa: PLC0415
+        Script,
+        async_validate_actions_config,
+    )
 
     script_config = await async_validate_actions_config(hass, msg["sequence"])
 
@@ -838,9 +921,9 @@ def handle_fire_event(
 @decorators.websocket_command(
     {
         vol.Required("type"): "validate_config",
-        vol.Optional("trigger"): cv.match_all,
-        vol.Optional("condition"): cv.match_all,
-        vol.Optional("action"): cv.match_all,
+        vol.Optional("triggers"): cv.match_all,
+        vol.Optional("conditions"): cv.match_all,
+        vol.Optional("actions"): cv.match_all,
     }
 )
 @decorators.async_response
@@ -849,22 +932,28 @@ async def handle_validate_config(
 ) -> None:
     """Handle validate config command."""
     # Circular dep
-    # pylint: disable-next=import-outside-toplevel
-    from homeassistant.helpers import condition, script, trigger
+    from homeassistant.helpers import condition, script, trigger  # noqa: PLC0415
 
     result = {}
 
     for key, schema, validator in (
-        ("trigger", cv.TRIGGER_SCHEMA, trigger.async_validate_trigger_config),
-        ("condition", cv.CONDITIONS_SCHEMA, condition.async_validate_conditions_config),
-        ("action", cv.SCRIPT_SCHEMA, script.async_validate_actions_config),
+        ("triggers", cv.TRIGGER_SCHEMA, trigger.async_validate_trigger_config),
+        (
+            "conditions",
+            cv.CONDITIONS_SCHEMA,
+            condition.async_validate_conditions_config,
+        ),
+        ("actions", cv.SCRIPT_SCHEMA, script.async_validate_actions_config),
     ):
         if key not in msg:
             continue
 
         try:
             await validator(hass, schema(msg[key]))
-        except vol.Invalid as err:
+        except (
+            vol.Invalid,
+            HomeAssistantError,
+        ) as err:
             result[key] = {"valid": False, "error": str(err)}
         else:
             result[key] = {"valid": True, "error": None}
@@ -895,3 +984,21 @@ async def handle_integration_descriptions(
 ) -> None:
     """Get metadata for all brands and integrations."""
     connection.send_result(msg["id"], await async_get_integration_descriptions(hass))
+
+
+@decorators.websocket_command(
+    {
+        vol.Required("type"): "integration/wait",
+        vol.Required("domain"): str,
+    }
+)
+@decorators.async_response
+async def handle_integration_wait(
+    hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
+) -> None:
+    """Handle wait for integration command."""
+
+    domain: str = msg["domain"]
+    connection.send_result(
+        msg["id"], {"integration_loaded": await async_wait_component(hass, domain)}
+    )
