@@ -1,19 +1,22 @@
 """Adds config flow for Workday integration."""
+
 from __future__ import annotations
 
+from functools import partial
 from typing import Any
 
-from holidays import HolidayBase, country_holidays, list_supported_countries
+from holidays import PUBLIC, HolidayBase, country_holidays, list_supported_countries
 import voluptuous as vol
 
 from homeassistant.config_entries import (
     ConfigEntry,
     ConfigFlow,
-    OptionsFlowWithConfigEntry,
+    ConfigFlowResult,
+    OptionsFlow,
 )
 from homeassistant.const import CONF_COUNTRY, CONF_LANGUAGE, CONF_NAME
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import AbortFlow, FlowResult
+from homeassistant.data_entry_flow import AbortFlow
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers.selector import (
     CountrySelector,
@@ -23,6 +26,7 @@ from homeassistant.helpers.selector import (
     NumberSelector,
     NumberSelectorConfig,
     NumberSelectorMode,
+    SelectOptionDict,
     SelectSelector,
     SelectSelectorConfig,
     SelectSelectorMode,
@@ -33,6 +37,7 @@ from homeassistant.util import dt as dt_util
 from .const import (
     ALLOWED_DAYS,
     CONF_ADD_HOLIDAYS,
+    CONF_CATEGORY,
     CONF_EXCLUDES,
     CONF_OFFSET,
     CONF_PROVINCE,
@@ -62,30 +67,60 @@ def add_province_and_language_to_schema(
 
     _country = country_holidays(country=country)
     if country_default_language := (_country.default_language):
-        selectable_languages = _country.supported_languages
-        new_selectable_languages = []
-        for lang in selectable_languages:
-            new_selectable_languages.append(lang[:2])
+        new_selectable_languages = list(_country.supported_languages)
         language_schema = {
             vol.Optional(
                 CONF_LANGUAGE, default=country_default_language
             ): LanguageSelector(
-                LanguageSelectorConfig(languages=new_selectable_languages)
+                LanguageSelectorConfig(
+                    languages=new_selectable_languages, native_name=True
+                )
             )
         }
 
     if provinces := all_countries.get(country):
+        if _country.subdivisions_aliases and (
+            subdiv_aliases := _country.get_subdivision_aliases()
+        ):
+            province_options: list[Any] = [
+                SelectOptionDict(value=k, label=", ".join(v))
+                for k, v in subdiv_aliases.items()
+            ]
+        else:
+            province_options = provinces
         province_schema = {
             vol.Optional(CONF_PROVINCE): SelectSelector(
                 SelectSelectorConfig(
-                    options=provinces,
+                    options=province_options,
                     mode=SelectSelectorMode.DROPDOWN,
                     translation_key=CONF_PROVINCE,
                 )
             ),
         }
 
-    return vol.Schema({**DATA_SCHEMA_OPT.schema, **language_schema, **province_schema})
+    category_schema = {}
+    # PUBLIC will always be included and can therefore not be set/removed
+    _categories = [x for x in _country.supported_categories if x != PUBLIC]
+    if _categories:
+        category_schema = {
+            vol.Optional(CONF_CATEGORY): SelectSelector(
+                SelectSelectorConfig(
+                    options=_categories,
+                    mode=SelectSelectorMode.DROPDOWN,
+                    multiple=True,
+                    translation_key=CONF_CATEGORY,
+                )
+            ),
+        }
+
+    return vol.Schema(
+        {
+            **DATA_SCHEMA_OPT.schema,
+            **language_schema,
+            **province_schema,
+            **category_schema,
+        }
+    )
 
 
 def _is_valid_date_range(check_date: str, error: type[HomeAssistantError]) -> bool:
@@ -110,7 +145,7 @@ def validate_custom_dates(user_input: dict[str, Any]) -> None:
 
     year: int = dt_util.now().year
     if country := user_input.get(CONF_COUNTRY):
-        language = user_input.get(CONF_LANGUAGE)
+        language: str | None = user_input.get(CONF_LANGUAGE)
         province = user_input.get(CONF_PROVINCE)
         obj_holidays = country_holidays(
             country=country,
@@ -118,17 +153,7 @@ def validate_custom_dates(user_input: dict[str, Any]) -> None:
             years=year,
             language=language,
         )
-        if (
-            supported_languages := obj_holidays.supported_languages
-        ) and language == "en":
-            for lang in supported_languages:
-                if lang.startswith("en"):
-                    obj_holidays = country_holidays(
-                        country,
-                        subdiv=province,
-                        years=year,
-                        language=lang,
-                    )
+
     else:
         obj_holidays = HolidayBase(years=year)
 
@@ -141,17 +166,6 @@ def validate_custom_dates(user_input: dict[str, Any]) -> None:
             raise RemoveDatesError("Incorrect date or name")
 
 
-DATA_SCHEMA_SETUP = vol.Schema(
-    {
-        vol.Required(CONF_NAME, default=DEFAULT_NAME): TextSelector(),
-        vol.Optional(CONF_COUNTRY): CountrySelector(
-            CountrySelectorConfig(
-                countries=list(list_supported_countries(include_aliases=False)),
-            )
-        ),
-    }
-)
-
 DATA_SCHEMA_OPT = vol.Schema(
     {
         vol.Optional(CONF_WORKDAYS, default=DEFAULT_WORKDAYS): SelectSelector(
@@ -206,26 +220,39 @@ class WorkdayConfigFlow(ConfigFlow, domain=DOMAIN):
         config_entry: ConfigEntry,
     ) -> WorkdayOptionsFlowHandler:
         """Get the options flow for this handler."""
-        return WorkdayOptionsFlowHandler(config_entry)
+        return WorkdayOptionsFlowHandler()
 
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle the user initial step."""
         errors: dict[str, str] = {}
 
+        supported_countries = await self.hass.async_add_executor_job(
+            partial(list_supported_countries, include_aliases=False)
+        )
+
         if user_input is not None:
             self.data = user_input
             return await self.async_step_options()
         return self.async_show_form(
             step_id="user",
-            data_schema=DATA_SCHEMA_SETUP,
+            data_schema=vol.Schema(
+                {
+                    vol.Required(CONF_NAME, default=DEFAULT_NAME): TextSelector(),
+                    vol.Optional(CONF_COUNTRY): CountrySelector(
+                        CountrySelectorConfig(
+                            countries=list(supported_countries),
+                        )
+                    ),
+                }
+            ),
             errors=errors,
         )
 
     async def async_step_options(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Handle remaining flow."""
         errors: dict[str, str] = {}
         if user_input is not None:
@@ -253,6 +280,8 @@ class WorkdayConfigFlow(ConfigFlow, domain=DOMAIN):
                 CONF_REMOVE_HOLIDAYS: combined_input[CONF_REMOVE_HOLIDAYS],
                 CONF_PROVINCE: combined_input.get(CONF_PROVINCE),
             }
+            if CONF_CATEGORY in combined_input:
+                abort_match[CONF_CATEGORY] = combined_input[CONF_CATEGORY]
             LOGGER.debug("abort_check in options with %s", combined_input)
             self._async_abort_entries_match(abort_match)
 
@@ -277,22 +306,22 @@ class WorkdayConfigFlow(ConfigFlow, domain=DOMAIN):
             errors=errors,
             description_placeholders={
                 "name": self.data[CONF_NAME],
-                "country": self.data.get(CONF_COUNTRY),
+                "country": self.data.get(CONF_COUNTRY, "-"),
             },
         )
 
 
-class WorkdayOptionsFlowHandler(OptionsFlowWithConfigEntry):
+class WorkdayOptionsFlowHandler(OptionsFlow):
     """Handle Workday options."""
 
     async def async_step_init(
         self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
+    ) -> ConfigFlowResult:
         """Manage Workday options."""
         errors: dict[str, str] = {}
 
         if user_input is not None:
-            combined_input: dict[str, Any] = {**self.options, **user_input}
+            combined_input: dict[str, Any] = {**self.config_entry.options, **user_input}
             if CONF_PROVINCE not in user_input:
                 # Province not present, delete old value (if present) too
                 combined_input.pop(CONF_PROVINCE, None)
@@ -311,40 +340,40 @@ class WorkdayOptionsFlowHandler(OptionsFlowWithConfigEntry):
                 errors["remove_holidays"] = "remove_holiday_range_error"
             else:
                 LOGGER.debug("abort_check in options with %s", combined_input)
+                abort_match = {
+                    CONF_COUNTRY: self.config_entry.options.get(CONF_COUNTRY),
+                    CONF_EXCLUDES: combined_input[CONF_EXCLUDES],
+                    CONF_OFFSET: combined_input[CONF_OFFSET],
+                    CONF_WORKDAYS: combined_input[CONF_WORKDAYS],
+                    CONF_ADD_HOLIDAYS: combined_input[CONF_ADD_HOLIDAYS],
+                    CONF_REMOVE_HOLIDAYS: combined_input[CONF_REMOVE_HOLIDAYS],
+                    CONF_PROVINCE: combined_input.get(CONF_PROVINCE),
+                }
+                if CONF_CATEGORY in combined_input:
+                    abort_match[CONF_CATEGORY] = combined_input[CONF_CATEGORY]
                 try:
-                    self._async_abort_entries_match(
-                        {
-                            CONF_COUNTRY: self._config_entry.options.get(CONF_COUNTRY),
-                            CONF_EXCLUDES: combined_input[CONF_EXCLUDES],
-                            CONF_OFFSET: combined_input[CONF_OFFSET],
-                            CONF_WORKDAYS: combined_input[CONF_WORKDAYS],
-                            CONF_ADD_HOLIDAYS: combined_input[CONF_ADD_HOLIDAYS],
-                            CONF_REMOVE_HOLIDAYS: combined_input[CONF_REMOVE_HOLIDAYS],
-                            CONF_PROVINCE: combined_input.get(CONF_PROVINCE),
-                        }
-                    )
+                    self._async_abort_entries_match(abort_match)
                 except AbortFlow as err:
                     errors = {"base": err.reason}
                 else:
                     return self.async_create_entry(data=combined_input)
 
+        options = self.config_entry.options
         schema: vol.Schema = await self.hass.async_add_executor_job(
             add_province_and_language_to_schema,
             DATA_SCHEMA_OPT,
-            self.options.get(CONF_COUNTRY),
+            options.get(CONF_COUNTRY),
         )
 
-        new_schema = self.add_suggested_values_to_schema(
-            schema, user_input or self.options
-        )
+        new_schema = self.add_suggested_values_to_schema(schema, user_input or options)
         LOGGER.debug("Errors have occurred in options %s", errors)
         return self.async_show_form(
             step_id="init",
             data_schema=new_schema,
             errors=errors,
             description_placeholders={
-                "name": self.options[CONF_NAME],
-                "country": self.options.get(CONF_COUNTRY),
+                "name": options[CONF_NAME],
+                "country": options.get(CONF_COUNTRY, "-"),
             },
         )
 
