@@ -1,16 +1,17 @@
 """Config flow to configure Xiaomi Aqara."""
+
 import logging
 from socket import gaierror
+from typing import Any
 
 import voluptuous as vol
 from xiaomi_gateway import MULTICAST_PORT, XiaomiGateway, XiaomiGatewayDiscovery
 
-from homeassistant import config_entries
-from homeassistant.components import zeroconf
+from homeassistant.config_entries import ConfigFlow, ConfigFlowResult
 from homeassistant.const import CONF_HOST, CONF_MAC, CONF_NAME, CONF_PORT, CONF_PROTOCOL
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers.device_registry import format_mac
+from homeassistant.helpers.service_info.zeroconf import ZeroconfServiceInfo
 
 from .const import (
     CONF_INTERFACE,
@@ -44,18 +45,19 @@ GATEWAY_SETTINGS = vol.Schema(
 )
 
 
-class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
+class XiaomiAqaraFlowHandler(ConfigFlow, domain=DOMAIN):
     """Handle a Xiaomi Aqara config flow."""
 
     VERSION = 1
 
-    def __init__(self):
+    selected_gateway: XiaomiGateway
+    gateways: dict[str, XiaomiGateway]
+
+    def __init__(self) -> None:
         """Initialize."""
-        self.host = None
+        self.host: str | None = None
         self.interface = DEFAULT_INTERFACE
-        self.sid = None
-        self.gateways = None
-        self.selected_gateway = None
+        self.sid: str | None = None
 
     @callback
     def async_show_form_step_user(self, errors):
@@ -66,9 +68,11 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
 
         return self.async_show_form(step_id="user", data_schema=schema, errors=errors)
 
-    async def async_step_user(self, user_input=None):
+    async def async_step_user(
+        self, user_input: dict[str, Any] | None = None
+    ) -> ConfigFlowResult:
         """Handle a flow initialized by the user."""
-        errors = {}
+        errors: dict[str, str] = {}
         if user_input is None:
             return self.async_show_form_step_user(errors)
 
@@ -125,9 +129,11 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         errors["base"] = "discovery_error"
         return self.async_show_form_step_user(errors)
 
-    async def async_step_select(self, user_input=None):
+    async def async_step_select(
+        self, user_input: dict[str, str] | None = None
+    ) -> ConfigFlowResult:
         """Handle multiple aqara gateways found."""
-        errors = {}
+        errors: dict[str, str] = {}
         if user_input is not None:
             ip_adress = user_input["select_ip"]
             self.selected_gateway = self.gateways[ip_adress]
@@ -147,8 +153,8 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
     async def async_step_zeroconf(
-        self, discovery_info: zeroconf.ZeroconfServiceInfo
-    ) -> FlowResult:
+        self, discovery_info: ZeroconfServiceInfo
+    ) -> ConfigFlowResult:
         """Handle zeroconf discovery."""
         name = discovery_info.name
         self.host = discovery_info.host
@@ -158,9 +164,7 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             return self.async_abort(reason="not_xiaomi_aqara")
 
         # Check if the discovered device is an xiaomi aqara gateway.
-        if not (
-            name.startswith(ZEROCONF_GATEWAY) or name.startswith(ZEROCONF_ACPARTNER)
-        ):
+        if not (name.startswith((ZEROCONF_GATEWAY, ZEROCONF_ACPARTNER))):
             _LOGGER.debug(
                 (
                     "Xiaomi device '%s' discovered with host %s, not identified as"
@@ -187,7 +191,9 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
 
         return await self.async_step_user()
 
-    async def async_step_settings(self, user_input=None):
+    async def async_step_settings(
+        self, user_input: dict[str, str] | None = None
+    ) -> ConfigFlowResult:
         """Specify settings and connect aqara gateway."""
         errors = {}
         if user_input is not None:
@@ -206,7 +212,7 @@ class XiaomiAqaraFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
                 valid_key = True
 
             if valid_key:
-                # format_mac, for a gateway the sid equels the mac address
+                # format_mac, for a gateway the sid equals the mac address
                 mac_address = format_mac(self.sid)
 
                 # set unique_id
