@@ -17,13 +17,13 @@ from homeassistant.components.number import (
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.entity_platform import AddConfigEntryEntitiesCallback
 
 from .const import DOMAIN
 from .coordinator import YoLinkCoordinator
 from .entity import YoLinkEntity
 
-OPTIONS_VALUME = "options_volume"
+OPTIONS_VOLUME = "options_volume"
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -49,14 +49,13 @@ def get_volume_value(state: dict[str, Any]) -> int | None:
 
 DEVICE_CONFIG_DESCRIPTIONS: tuple[YoLinkNumberTypeConfigEntityDescription, ...] = (
     YoLinkNumberTypeConfigEntityDescription(
-        key=OPTIONS_VALUME,
+        key=OPTIONS_VOLUME,
         translation_key="config_volume",
         native_min_value=1,
         native_max_value=16,
         mode=NumberMode.SLIDER,
         native_step=1.0,
         native_unit_of_measurement=None,
-        icon="mdi:volume-high",
         exists_fn=lambda device: device.device_type in SUPPORT_SET_VOLUME_DEVICES,
         should_update_entity=lambda value: value is not None,
         value=get_volume_value,
@@ -67,7 +66,7 @@ DEVICE_CONFIG_DESCRIPTIONS: tuple[YoLinkNumberTypeConfigEntityDescription, ...]
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
+    async_add_entities: AddConfigEntryEntitiesCallback,
 ) -> None:
     """Set up device number type config option entity from a config entry."""
     device_coordinators = hass.data[DOMAIN][config_entry.entry_id].device_coordinators
@@ -76,18 +75,16 @@ async def async_setup_entry(
         for device_coordinator in device_coordinators.values()
         if device_coordinator.device.device_type in NUMBER_TYPE_CONF_SUPPORT_DEVICES
     ]
-    entities = []
-    for config_device_coordinator in config_device_coordinators:
-        for description in DEVICE_CONFIG_DESCRIPTIONS:
-            if description.exists_fn(config_device_coordinator.device):
-                entities.append(
-                    YoLinkNumberTypeConfigEntity(
-                        config_entry,
-                        config_device_coordinator,
-                        description,
-                    )
-                )
-    async_add_entities(entities)
+    async_add_entities(
+        YoLinkNumberTypeConfigEntity(
+            config_entry,
+            config_device_coordinator,
+            description,
+        )
+        for config_device_coordinator in config_device_coordinators
+        for description in DEVICE_CONFIG_DESCRIPTIONS
+        if description.exists_fn(config_device_coordinator.device)
+    )
 
 
 class YoLinkNumberTypeConfigEntity(YoLinkEntity, NumberEntity):
@@ -124,7 +121,7 @@ class YoLinkNumberTypeConfigEntity(YoLinkEntity, NumberEntity):
         """Update the current value."""
         if (
             self.coordinator.device.device_type == ATTR_DEVICE_SPEAKER_HUB
-            and self.entity_description.key == OPTIONS_VALUME
+            and self.entity_description.key == OPTIONS_VOLUME
         ):
             await self.update_speaker_hub_volume(value)
             self._attr_native_value = value
