@@ -522,8 +522,14 @@ class EntityPlatform:
         self, new_entities: Iterable[Entity], update_before_add: bool = False
     ) -> None:
         """Schedule adding entities for a single platform async."""
+        entities: list[Entity] = (
+            new_entities if type(new_entities) is list else list(new_entities)
+        )
+        # handle empty list from component/platform
+        if not entities:
+            return
         task = self.hass.async_create_task_internal(
-            self.async_add_entities(new_entities, update_before_add=update_before_add),
+            self.async_add_entities(entities, update_before_add=update_before_add),
             f"EntityPlatform async_add_entities {self.domain}.{self.platform_name}",
             eager_start=True,
         )
@@ -541,10 +547,16 @@ class EntityPlatform:
     ) -> None:
         """Schedule adding entities for a single platform async and track the task."""
         assert self.config_entry
+        entities: list[Entity] = (
+            new_entities if type(new_entities) is list else list(new_entities)
+        )
+        # handle empty list from component/platform
+        if not entities:
+            return
         task = self.config_entry.async_create_task(
             self.hass,
             self.async_add_entities(
-                new_entities,
+                entities,
                 update_before_add=update_before_add,
                 config_subentry_id=config_subentry_id,
             ),
@@ -573,9 +585,9 @@ class EntityPlatform:
 
     async def _async_add_and_update_entities(
         self,
-        coros: list[Coroutine[Any, Any, None]],
         entities: list[Entity],
         timeout: float,
+        config_subentry_id: str | None,
     ) -> None:
         """Add entities for a single platform and update them.
 
@@ -585,10 +597,21 @@ class EntityPlatform:
         event loop and will finish faster if we run them concurrently.
         """
         results: list[BaseException | None] | None = None
-        tasks = [create_eager_task(coro, loop=self.hass.loop) for coro in coros]
+        entity_registry = ent_reg.async_get(self.hass)
         try:
             async with self.hass.timeout.async_timeout(timeout, self.domain):
-                results = await asyncio.gather(*tasks, return_exceptions=True)
+                results = await asyncio.gather(
+                    *(
+                        create_eager_task(
+                            self._async_add_entity(
+                                entity, True, entity_registry, config_subentry_id
+                            ),
+                            loop=self.hass.loop,
+                        )
+                        for entity in entities
+                    ),
+                    return_exceptions=True,
+                )
         except TimeoutError:
             self.logger.warning(
                 "Timed out adding entities for domain %s with platform %s after %ds",
@@ -615,9 +638,9 @@ class EntityPlatform:
 
     async def _async_add_entities(
         self,
-        coros: list[Coroutine[Any, Any, None]],
         entities: list[Entity],
         timeout: float,
+        config_subentry_id: str | None,
     ) -> None:
         """Add entities for a single platform without updating.
 
@@ -626,13 +649,15 @@ class EntityPlatform:
         to the event loop so we can await the coros directly without
         scheduling them as tasks.
         """
+        entity_registry = ent_reg.async_get(self.hass)
         try:
             async with self.hass.timeout.async_timeout(timeout, self.domain):
-                for idx, coro in enumerate(coros):
+                for entity in entities:
                     try:
-                        await coro
+                        await self._async_add_entity(
+                            entity, False, entity_registry, config_subentry_id
+                        )
                     except Exception as ex:
-                        entity = entities[idx]
                         self.logger.exception(
                             "Error adding entity %s for domain %s with platform %s",
                             entity.entity_id,
@@ -670,33 +695,16 @@ class EntityPlatform:
                 f"entry {self.config_entry.entry_id if self.config_entry else None}"
             )
 
-        # handle empty list from component/platform
-        if not new_entities:  # type: ignore[truthy-iterable]
-            return
-
-        hass = self.hass
-        entity_registry = ent_reg.async_get(hass)
-        coros: list[Coroutine[Any, Any, None]] = []
-        entities: list[Entity] = []
-        for entity in new_entities:
-            coros.append(
-                self._async_add_entity(
-                    entity, update_before_add, entity_registry, config_subentry_id
-                )
-            )
-            entities.append(entity)
-
-        # No entities for processing
-        if not coros:
-            return
-
-        timeout = max(SLOW_ADD_ENTITY_MAX_WAIT * len(coros), SLOW_ADD_MIN_TIMEOUT)
+        entities: list[Entity] = (
+            new_entities if type(new_entities) is list else list(new_entities)
+        )
+        timeout = max(SLOW_ADD_ENTITY_MAX_WAIT * len(entities), SLOW_ADD_MIN_TIMEOUT)
         if update_before_add:
-            add_func = self._async_add_and_update_entities
+            await self._async_add_and_update_entities(
+                entities, timeout, config_subentry_id
+            )
         else:
-            add_func = self._async_add_entities
-
-        await add_func(coros, entities, timeout)
+            await self._async_add_entities(entities, timeout, config_subentry_id)
 
         if (
             (self.config_entry and self.config_entry.pref_disable_polling)
@@ -835,24 +843,23 @@ class EntityPlatform:
             else:
                 device = None
 
+            calculated_object_id: str | None = None
             # An entity may suggest the entity_id by setting entity_id itself
             suggested_entity_id: str | None = entity.entity_id
             if suggested_entity_id is not None:
                 suggested_object_id = split_entity_id(entity.entity_id)[1]
-            else:
-                if device and entity.has_entity_name:
-                    device_name = device.name_by_user or device.name
-                    if entity.use_device_name:
-                        suggested_object_id = device_name
-                    else:
-                        suggested_object_id = (
-                            f"{device_name} {entity.suggested_object_id}"
-                        )
-                if not suggested_object_id:
-                    suggested_object_id = entity.suggested_object_id
-
-            if self.entity_namespace is not None:
-                suggested_object_id = f"{self.entity_namespace} {suggested_object_id}"
+                if self.entity_namespace is not None:
+                    suggested_object_id = (
+                        f"{self.entity_namespace} {suggested_object_id}"
+                    )
+            if not registered_entity_id and suggested_entity_id is None:
+                # Do not bother working out a suggested_object_id
+                # if the entity is already registered as it will
+                # be ignored.
+                #
+                calculated_object_id = async_calculate_suggested_object_id(
+                    entity, device
+                )
 
             disabled_by: RegistryEntryDisabler | None = None
             if not entity.entity_registry_enabled_default:
@@ -866,6 +873,7 @@ class EntityPlatform:
                 self.domain,
                 self.platform_name,
                 entity.unique_id,
+                calculated_object_id=calculated_object_id,
                 capabilities=entity.capability_attributes,
                 config_entry=self.config_entry,
                 config_subentry_id=config_subentry_id,
@@ -875,7 +883,6 @@ class EntityPlatform:
                 get_initial_options=entity.get_initial_entity_options,
                 has_entity_name=entity.has_entity_name,
                 hidden_by=hidden_by,
-                known_object_ids=self.entities,
                 original_device_class=entity.device_class,
                 original_icon=entity.icon,
                 original_name=entity_name,
@@ -919,7 +926,7 @@ class EntityPlatform:
                         f"{self.entity_namespace} {suggested_object_id}"
                     )
                 entity.entity_id = entity_registry.async_generate_entity_id(
-                    self.domain, suggested_object_id, self.entities
+                    self.domain, suggested_object_id
                 )
 
             # Make sure it is valid in case an entity set the value themselves
@@ -1110,6 +1117,27 @@ class EntityPlatform:
                 await asyncio.gather(*tasks)
 
 
+@callback
+def async_calculate_suggested_object_id(
+    entity: Entity, device: dev_reg.DeviceEntry | None
+) -> str | None:
+    """Calculate the suggested object ID for an entity."""
+    calculated_object_id: str | None = None
+    if device and entity.has_entity_name:
+        device_name = device.name_by_user or device.name
+        if entity.use_device_name:
+            calculated_object_id = device_name
+        else:
+            calculated_object_id = f"{device_name} {entity.suggested_object_id}"
+    if not calculated_object_id:
+        calculated_object_id = entity.suggested_object_id
+
+    if (platform := entity.platform) and platform.entity_namespace is not None:
+        calculated_object_id = f"{platform.entity_namespace} {calculated_object_id}"
+
+    return calculated_object_id
+
+
 current_platform: ContextVar[EntityPlatform | None] = ContextVar(
     "current_platform", default=None
 )
