@@ -12,14 +12,13 @@ from awesomeversion import AwesomeVersion
 import pytest
 
 from homeassistant import loader
-from homeassistant.components import http, hue
+from homeassistant.components import hue
 from homeassistant.components.hue import light as hue_light
-from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import frame
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers.json import json_dumps
 from homeassistant.util.json import json_loads
 
-from .common import MockModule, async_get_persistent_notifications, mock_integration
+from .common import MockModule, mock_integration
 
 
 async def test_circular_component_dependencies(hass: HomeAssistant) -> None:
@@ -28,82 +27,91 @@ async def test_circular_component_dependencies(hass: HomeAssistant) -> None:
     mock_integration(hass, MockModule("mod2", dependencies=["mod1"]))
     mock_integration(hass, MockModule("mod3", dependencies=["mod1"]))
     mod_4 = mock_integration(hass, MockModule("mod4", dependencies=["mod2", "mod3"]))
+    all_domains = {"mod1", "mod2", "mod3", "mod4"}
 
-    deps = await loader._async_component_dependencies(hass, mod_4)
-    assert deps == {"mod1", "mod2", "mod3", "mod4"}
+    deps = await loader._resolve_integration_dependencies(mod_4, cache={})
+    assert deps == {"mod1", "mod2", "mod3"}
 
     # Create a circular dependency
     mock_integration(hass, MockModule("mod1", dependencies=["mod4"]))
     with pytest.raises(loader.CircularDependency):
-        await loader._async_component_dependencies(hass, mod_4)
+        await loader._resolve_integration_dependencies(mod_4, cache={})
 
     # Create a different circular dependency
     mock_integration(hass, MockModule("mod1", dependencies=["mod3"]))
     with pytest.raises(loader.CircularDependency):
-        await loader._async_component_dependencies(hass, mod_4)
+        await loader._resolve_integration_dependencies(mod_4, cache={})
 
     # Create a circular after_dependency
     mock_integration(
         hass, MockModule("mod1", partial_manifest={"after_dependencies": ["mod4"]})
     )
     with pytest.raises(loader.CircularDependency):
-        await loader._async_component_dependencies(hass, mod_4)
+        await loader._resolve_integration_dependencies(
+            mod_4,
+            cache={},
+            possible_after_dependencies=all_domains,
+        )
 
     # Create a different circular after_dependency
     mock_integration(
         hass, MockModule("mod1", partial_manifest={"after_dependencies": ["mod3"]})
     )
     with pytest.raises(loader.CircularDependency):
-        await loader._async_component_dependencies(hass, mod_4)
+        await loader._resolve_integration_dependencies(
+            mod_4,
+            cache={},
+            possible_after_dependencies=all_domains,
+        )
+
+    # Create a circular after_dependency without a hard dependency
+    mock_integration(
+        hass, MockModule("mod1", partial_manifest={"after_dependencies": ["mod4"]})
+    )
+    mod_4 = mock_integration(
+        hass, MockModule("mod4", partial_manifest={"after_dependencies": ["mod2"]})
+    )
+    with pytest.raises(loader.CircularDependency):
+        await loader._resolve_integration_dependencies(
+            mod_4,
+            cache={},
+            possible_after_dependencies=all_domains,
+        )
+
+    result = await loader.resolve_integrations_after_dependencies(hass, (mod_4,))
+    assert result == {}
+    result = await loader.resolve_integrations_after_dependencies(
+        hass, (mod_4,), ignore_exceptions=True
+    )
+    assert result["mod4"] == {"mod4", "mod2", "mod1"}
 
 
 async def test_nonexistent_component_dependencies(hass: HomeAssistant) -> None:
     """Test if we can detect nonexistent dependencies of components."""
     mod_1 = mock_integration(hass, MockModule("mod1", dependencies=["nonexistent"]))
+    mod_2 = mock_integration(hass, MockModule("mod2", dependencies=["mod1"]))
+
+    assert await mod_2.resolve_dependencies() is None
+    assert mod_2.all_dependencies_resolved
     with pytest.raises(loader.IntegrationNotFound):
-        await loader._async_component_dependencies(hass, mod_1)
+        mod_2.all_dependencies  # noqa: B018
 
+    assert mod_1.all_dependencies_resolved
+    assert await mod_1.resolve_dependencies() is None
+    with pytest.raises(loader.IntegrationNotFound):
+        mod_1.all_dependencies  # noqa: B018
 
-def test_component_loader(hass: HomeAssistant) -> None:
-    """Test loading components."""
-    components = loader.Components(hass)
-    assert components.http.CONFIG_SCHEMA is http.CONFIG_SCHEMA
-    assert hass.components.http.CONFIG_SCHEMA is http.CONFIG_SCHEMA
+    result = await loader.resolve_integrations_dependencies(hass, (mod_2, mod_1))
+    assert result == {}
 
+    mod_1 = mock_integration(
+        hass,
+        MockModule("mod1", partial_manifest={"after_dependencies": ["non.existent"]}),
+    )
+    mod_2 = mock_integration(hass, MockModule("mod2", dependencies=["mod1"]))
 
-def test_component_loader_non_existing(hass: HomeAssistant) -> None:
-    """Test loading components."""
-    components = loader.Components(hass)
-    with pytest.raises(ImportError):
-        _ = components.non_existing
-
-
-async def test_component_wrapper(hass: HomeAssistant) -> None:
-    """Test component wrapper."""
-    components = loader.Components(hass)
-    components.persistent_notification.async_create("message")
-
-    notifications = async_get_persistent_notifications(hass)
-    assert len(notifications)
-
-
-async def test_helpers_wrapper(hass: HomeAssistant) -> None:
-    """Test helpers wrapper."""
-    helpers = loader.Helpers(hass)
-
-    result = []
-
-    @callback
-    def discovery_callback(service, discovered):
-        """Handle discovery callback."""
-        result.append(discovered)
-
-    helpers.discovery.async_listen("service_name", discovery_callback)
-
-    await helpers.discovery.async_discover("service_name", "hello", None, {})
-    await hass.async_block_till_done()
-
-    assert result == ["hello"]
+    result = await loader.resolve_integrations_after_dependencies(hass, (mod_2, mod_1))
+    assert result == {}
 
 
 @pytest.mark.usefixtures("enable_custom_integrations")
@@ -118,10 +126,6 @@ async def test_custom_component_name(hass: HomeAssistant) -> None:
     assert int_comp.__name__ == "custom_components.test_package"
     assert int_comp.__package__ == "custom_components.test_package"
 
-    comp = hass.components.test_package
-    assert comp.__name__ == "custom_components.test_package"
-    assert comp.__package__ == "custom_components.test_package"
-
     integration = await loader.async_get_integration(hass, "test")
     platform = integration.get_platform("light")
     assert integration.get_platform_cached("light") is platform
@@ -130,8 +134,7 @@ async def test_custom_component_name(hass: HomeAssistant) -> None:
     assert platform.__package__ == "custom_components.test"
 
     # Test custom components is mounted
-    # pylint: disable-next=import-outside-toplevel
-    from custom_components.test_package import TEST
+    from custom_components.test_package import TEST  # noqa: PLC0415
 
     assert TEST == 5
 
@@ -1291,49 +1294,11 @@ async def test_config_folder_not_in_path() -> None:
 
     # Verify that we are unable to import this file from top level
     with pytest.raises(ImportError):
-        # pylint: disable-next=import-outside-toplevel
-        import check_config_not_in_path  # noqa: F401
+        import check_config_not_in_path  # noqa: F401, PLC0415
 
     # Verify that we are able to load the file with absolute path
-    # pylint: disable-next=import-outside-toplevel,hass-relative-import
-    import tests.testing_config.check_config_not_in_path  # noqa: F401
-
-
-@pytest.mark.parametrize(
-    ("integration_frame_path", "expected"),
-    [
-        pytest.param(
-            "custom_components/test_integration_frame", True, id="custom integration"
-        ),
-        pytest.param(
-            "homeassistant/components/test_integration_frame",
-            False,
-            id="core integration",
-        ),
-        pytest.param("homeassistant/test_integration_frame", False, id="core"),
-    ],
-)
-@pytest.mark.usefixtures("mock_integration_frame")
-@patch.object(frame, "_REPORTED_INTEGRATIONS", set())
-async def test_hass_components_use_reported(
-    hass: HomeAssistant,
-    caplog: pytest.LogCaptureFixture,
-    expected: bool,
-) -> None:
-    """Test whether use of hass.components is reported."""
-    with (
-        patch(
-            "homeassistant.components.http.start_http_server_and_save_config",
-            return_value=None,
-        ),
-    ):
-        await hass.components.http.start_http_server_and_save_config(hass, [], None)
-
-        reported = (
-            "Detected that custom integration 'test_integration_frame'"
-            " accesses hass.components.http, which should be updated"
-        ) in caplog.text
-        assert reported == expected
+    # pylint: disable-next=hass-relative-import
+    import tests.testing_config.check_config_not_in_path  # noqa: F401, PLC0415
 
 
 async def test_async_get_component_preloads_config_and_config_flow(
@@ -1995,43 +1960,6 @@ async def test_has_services(hass: HomeAssistant) -> None:
     assert integration.has_services is True
 
 
-@pytest.mark.parametrize(
-    ("integration_frame_path", "expected"),
-    [
-        pytest.param(
-            "custom_components/test_integration_frame", True, id="custom integration"
-        ),
-        pytest.param(
-            "homeassistant/components/test_integration_frame",
-            False,
-            id="core integration",
-        ),
-        pytest.param("homeassistant/test_integration_frame", False, id="core"),
-    ],
-)
-@pytest.mark.usefixtures("mock_integration_frame")
-@patch.object(frame, "_REPORTED_INTEGRATIONS", set())
-async def test_hass_helpers_use_reported(
-    hass: HomeAssistant,
-    caplog: pytest.LogCaptureFixture,
-    expected: bool,
-) -> None:
-    """Test whether use of hass.helpers is reported."""
-    with (
-        patch(
-            "homeassistant.helpers.aiohttp_client.async_get_clientsession",
-            return_value=None,
-        ),
-    ):
-        hass.helpers.aiohttp_client.async_get_clientsession()
-
-        reported = (
-            "Detected that custom integration 'test_integration_frame' "
-            "accesses hass.helpers.aiohttp_client, which should be updated"
-        ) in caplog.text
-        assert reported == expected
-
-
 async def test_manifest_json_fragment_round_trip(hass: HomeAssistant) -> None:
     """Test json_fragment roundtrip."""
     integration = await loader.async_get_integration(hass, "hue")
